import { Component, Injector, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { LocalStorage, vitisAlert, VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { DatasourceTree } from '../../models/DatasourceTree.model';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { AdminDatasourceFormService } from 'src/modules/vmap/components/admin-datasource/services/admin-datasource-form.service';
import { environment } from 'src/environments/environment';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { InterfaceService } from 'src/app/vitis/core/services/utils/interface/interface.service';
import { SessionStorageService } from 'src/app/vitis/core/services/utils/sessionstorage/session-storage.service';
import { LocalStorageService } from 'src/app/vitis/core/services/utils/localstorage/local-storage.service';
import { WmsService } from 'src/modules/vmap/components/admin-datasource/services/wms.service';
import { WmtsService } from 'src/modules/vmap/components/admin-datasource/services/wmts.service';

@Component({
  selector: 'app-admin-datasource-form',
  templateUrl: './admin-datasource-form.component.html',
  styleUrls: ['./admin-datasource-form.component.scss']
})
export class AdminDatasourceFormComponent implements OnInit, OnDestroy {

  @ViewChild(FormreaderComponent, { static: false }) oFormReader: FormreaderComponent;

  private treeSubscription: any;
  private routeDataSubscription: any;
  private routeParamsSubscription: any;
  public oOptions: any;
  public oParams: any;
  public aTree: DatasourceTree;
  public oFormOptions: any;
  private formIsLoad: boolean = false; // Pour eviter que le formulaire ce recharge avec le modification de l'arbre

  constructor(
    private route: ActivatedRoute,
    private adminDatasourceService: AdminDatasourceService,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private adminDatasourceFormService: AdminDatasourceFormService,
    private router: Router,
    private auth: AuthService,
    private translate: TranslateService,
    private formreaderSrvc: FormreaderService,
    private interfaceService: InterfaceService,
    private sessionStorageService: SessionStorageService,
    private localStorageService: LocalStorageService,
    private localStorage: LocalStorage,
    private injector: Injector,
    private wmsService: WmsService,
    private wmtsService: WmtsService,
  ) { }

  ngOnInit() {
    this.routeDataSubscription = this.route.data.subscribe(oOptions => {
      this.oOptions = oOptions;
      this.adminDatasourceFormService.oOptions = this.oOptions;
    });
    this.routeParamsSubscription = this.route.params.subscribe(oParams => {
      // Avant la réasignation de this.oParams
      // Si un objet était précédement selectionné on enlève la sélection (pour un objet sur la même branche)
      this.adminDatasourceService.setObjectSelected(false, this.oParams, this.aTree);

      // Les parametres de la route change c'est que l'on change de formulaire donc on considère le formulaire comme non chargé
      this.formIsLoad = false;

      this.oParams = oParams;
      this.adminDatasourceFormService.oParams = this.oParams;

      // Pour que initform soit lancé il faut l'arbre ainsi que l'id
      if (oParams &&
        oParams.treeid) {
        if (this.aTree) {
          this.initForm();
        }
      }
    });
    this.treeSubscription = this.adminDatasourceService.getTreeObs().subscribe((aTree) => {
      this.aTree = aTree;

      // Pour que initform soit lancé il faut l'arbre ainsi que l'id
      if (aTree &&
        this.oParams &&
        this.oParams.treeid && this.formIsLoad === false) {
        this.initForm();
      }
    });
  }

  ngOnDestroy() {
    // Enlève la sélection quand on change de branche
    this.adminDatasourceService.setObjectSelected(false, this.oParams, this.aTree);
    // Evite que getTreeObs() se redéclenche pour un ancien objet
    this.oParams = null;
    if (this.treeSubscription) {
      this.treeSubscription.unsubscribe();
    }
    if (this.routeDataSubscription) {
      this.routeDataSubscription.unsubscribe();
    }
    if (this.routeParamsSubscription) {
      this.routeParamsSubscription.unsubscribe();
    }
  }

  /**
   * Initialise le formulaire
   */
  public initForm() {
    let oObject;
    const idClone = this.localStorage.get('idClone');
    if (this.vitisCheck.isDefAndNotNull(idClone)) {
      oObject = this.adminDatasourceService.getObjectById(idClone);
      this.localStorage.set('idClone', null);
    } else {
      oObject = this.adminDatasourceService.getObjectById(this.oParams.treeid);
    }

    if (this.oOptions &&
      this.oOptions.form && this.oParams.formName && this.vitisCheck.isDefAndNotNull(oObject)) {

      // Charge le contenu du formulaire
      this.getJSONForm_(this.oOptions.form).then((oForm) => {
        this.oOptions.form = oForm;

        // selectionne l'objet actuel
        this.adminDatasourceService.setObjectSelected(true, { ...this.oParams, treeid: idClone || this.oParams.treeid }, this.aTree);

        // Charge le JS
        this.setEventContainer(oObject).then(() => {

          // Prépare les valeurs
          let oFormValuesOrObs: any = {};
          if (this.vitisCheck.isDefAndNotNull(idClone) || this.oParams.formName != 'insert') {
            // Si dans les paramètres get_function est définie, on récupère les données avec cette requête
            if (this.vitisCheck.isDefAndNotNull(this.oOptions.get_function)) {
              if (this.vitisCheck.isFunction(oObject[this.oOptions.get_function])) {
                oFormValuesOrObs = oObject[this.oOptions.get_function]();
              } else {
                console.error('La fonction ' + this.oOptions.get_function + ' (param get_function) n\'existe pas dans l\'objet ', oObject);
              }
            } else {
              // Sinon par défaut on récupère les données avec getFormValues
              if (this.vitisCheck.isFunction(oObject.getFormValues)) {
                oFormValuesOrObs = oObject.getFormValues();
              }
            }
          } else {
            if (this.vitisCheck.isFunction(oObject.getChildrenInitialFormValues)) {
              oFormValuesOrObs = oObject.getChildrenInitialFormValues();
            }
          }

          if (this.vitisCheck.isDefAndNotNull(oFormValuesOrObs) && this.vitisCheck.isFunction(oFormValuesOrObs.then)) {
            oFormValuesOrObs.then((oFormValues: any) => {
              if (oFormValues !== false) {
                this.setFormOption(oFormValues);
                // Le formulaire est chargé
                this.formIsLoad = true;
              } else {
                this.oFormOptions = null;
              }
            });
          } else {
            this.setFormOption(oFormValuesOrObs);
            // Le formulaire est chargé
            this.formIsLoad = true;
          }
        });

      }).catch(() => {
        console.error('Cannot get layer form');
      });
    }
  }

  /**
   * Récupère le formulaire d'après son url
   * Si le formulaire a déjà été récupéré alors ne fait rien
   * @param form
   * @returns
   */
  private getJSONForm_(form: string | any): Promise<any> {
    return new Promise((resolve, reject) => {
      if (this.vitisCheck.isString(form)) {
        // Récupère le formulaire
        this.vitisRequest.ajaxRequestPromise({
          method: 'GET',
          url: form,
        }).then((resp) => {
          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            resolve(resp.data);
          } else {
            console.error('Error getting JSON form');
            reject(false);
          }
        });
      } else {
        setTimeout(() => {
          resolve(form);
        });
      }
    });
  }

  /**
   * Charge le JS associé au formulaire
   * @param  {any} oFormObject
   * @returns Promise
   */
  private loadFormJavaScript_(oFormObject: any): Promise<boolean> {
    return new Promise((resolve, reject) => {

      // Si le mode contient un JS associé
      if (this.vitisCheck.isDefAndNotNull(oFormObject)) {

        // Charge le JS du mode
        this.vitisRequest.loadExternalJs(oFormObject).then(() => {
          resolve(true);
        });
      } else {
        resolve(false);
      }
    });
  }

  /**
   * Ajoute les méthodes communes aux formulaires
   * @param eventContainer
   */
  private setCommonFunction(eventContainer: any) {
    this.adminDatasourceFormService.oFormReader = this.oFormReader;

    this.adminDatasourceFormService.init(this.route, this.adminDatasourceService);
    eventContainer.sendForm = (oValues: any) => {
      this.adminDatasourceFormService.sendForm(oValues);
    };

    eventContainer.cancelForm = () => {
      this.adminDatasourceFormService.cancelForm();
    };

    eventContainer.setMode = (sMode: string) => {
      if (sMode === 'search') {
        this.adminDatasourceFormService.cancelForm();
      }
    };

    eventContainer.sendSimpleForm = (oValues: any) => {
      this.adminDatasourceFormService.sendForm(oValues);
    };

    return eventContainer;
  }

  /**
   * setFormOption pour charger le formulaire
   * @param oFormValues
   */
  private setFormOption(oFormValues: any) {
    // Charge le formulaire
    this.oFormOptions = {
      form: this.oOptions.form,
      formName: this.oParams.formName,
      formValues: oFormValues,
    };
  }

  /**
   * Prépare le event container et les fonction commune
   * @param oObject
   */
  private setEventContainer(oObject: any): Promise<any> {
    return new Promise((resolve, reject) => {

      // Sélectionne l'event container en fonction de l'option formobject de la route
      let eventContainer = this;
      if (this.vitisCheck.isString(this.oOptions.formobject)) {
        this.loadFormJavaScript_(this.oOptions.formobject).then((bJsLoaded) => {

          if (bJsLoaded) {
            // Lance le JS associé
            if ((<any>window).constructor_form) {
              (<any>window).constructor_form(this.oFormReader, environment, {
                vitisRequest: this.vitisRequest,
                vitisCheck: this.vitisCheck,
                vitisAlert: vitisAlert,
                auth: this.auth,
                translate: this.translate,
                formreaderSrvc: this.formreaderSrvc,
                route: this.route,
                router: this.router,
                SimpleformComponent: this,
                interfaceService: this.interfaceService,
                sessionStorageService: this.sessionStorageService,
                localStorageService: this.localStorageService,
                oFormReader: this.oFormReader,
                AdminDatasourceFormService: this.adminDatasourceFormService,
                AdminDatasourceService: this.adminDatasourceService,
                WmsService: this.wmsService,
                WmtsService: this.wmtsService,
              });
            }
          }

          // Ajout les méthodes communes dans le container
          eventContainer = this.setCommonFunction(eventContainer);

          // Ajout l'event container
          this.oOptions.form[this.oParams.formName].eventContainer = eventContainer;

          resolve(true);
        });
      } else {
        // Chargement du formulaire
        if (this.vitisCheck.isDefAndNotNull(this.oOptions.formobject)) {
          eventContainer = new this.oOptions.formobject(this.oFormReader, oObject, this.injector);
        }

        // Ajout les méthodes communes dans le container
        eventContainer = this.setCommonFunction(eventContainer);

        // Ajout l'event container
        this.oOptions.form[this.oParams.formName].eventContainer = eventContainer;

        resolve(true);
      }
    });
  }
}
