import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { WmtsService } from 'src/modules/vmap/components/admin-datasource/services/wmts.service';
import { VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewWMTSLayer } from '../../models/DatasourceTreeviewWMTS/DatasourceTreeviewWMTSLayer';
import { defaultEmptyMapOptions } from 'src/app/modules/vmap/admin-datasource/forms/DatasourceTreeviewFormVar';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { MapCoreFormreaderService } from 'src/app/vitis/map-core/services/map-core-fromreader/map-core-formreader.service';

export class DatasourceTreeviewWMTSLayerMapFormJS {

  private oObject: DatasourceTreeviewWMTSLayer;
  // Services
  private vitisCheck: VitisCheck;
  private mapCoreManagerService: MapCoreManagerService;
  private wmtsService: WmtsService;
  private vitisClone: VitisClone;
  private mapCoreFormreaderService: MapCoreFormreaderService;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {object} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: any,
    injector: any
  ) {
    this.oObject = oObject;

    this.vitisCheck = injector.get(VitisCheck);
    this.mapCoreManagerService = injector.get(MapCoreManagerService);
    this.wmtsService = injector.get(WmtsService);
    this.vitisClone = injector.get(VitisClone);
    this.mapCoreFormreaderService = injector.get(MapCoreFormreaderService);
  }

  /**
   * Evenement lancé par AdminDatasourceFormComponent juste avant l'envoi du formulaire
   */
  public prepareData() {
    // Si pas en insert (impossible en insert car this.oObject n'existe pas)
    if (this.oFormReader.sFormDefinitionName !== 'insert' && this.vitisCheck.isDefAndNotNull(this.oObject)) {
      const oMapField = this.oFormReader.getFormField('wmts_map_layer');
      // Réinitialise la carte
      oMapField.map_options = this.vitisClone.clone(defaultEmptyMapOptions);
      // Récupère oCapabilities
      const oCapabilities: any = this.oObject.layer_options;
      if (this.vitisCheck.isDefAndNotNull(oCapabilities) &&
        this.vitisCheck.isDefAndNotNull(oCapabilities) &&
        this.vitisCheck.isDefAndNotNull(oCapabilities.Contents) &&
        this.vitisCheck.isDefAndNotNull(oCapabilities.Contents.Layer) &&
        this.vitisCheck.isDefAndNotNull(this.oObject.layer_name)) {

        const oCapabilitiesLayers = oCapabilities.Contents.Layer;

        // Récupère la liste des couches dans le getCapabilities
        const aLayers = this.wmtsService.searchLayers(oCapabilitiesLayers);
        // Récupère la couche
        const oLayer = aLayers.find((layer: any) => layer.Identifier === this.oObject.layer_name);

        this.getMapProjFromWMTSLayer(oLayer, oCapabilities).then((sProj: string) => {

          // Prépare les données pour la carte et la grille à partir des couches du flux WMTS.
          const oService = this.prepareLayer_(oCapabilities);

          // Suppression service si existe déjà
          const iChildIndex = oMapField.map_options.tree.children.findIndex((oChild: any) => oChild.name === oService.name);
          if (iChildIndex !== -1) {
            oMapField.map_options.tree.children.splice(iChildIndex, 1);
          }

          // Ajout du service
          oMapField.map_options.tree.children.push(oService);

          this.mapCoreFormreaderService.emitMapFormreaderEvent(oMapField.id, {
            action: 'reload'
          });
        });
      } else {
        console.error('Erreur lors de la récupération du oCapabilities (layer_options) : ', oCapabilities);
      }
    }
  }

  /**
   * Prépare la couche pour la carte
   * @returns
   */
  private prepareLayer_(oCapabilities: any): any {
    const oChildrenService = {
      'name': 'Service WMTS',
      'children': [{
        'name': this.oObject.layer_name,
        'index': 1,
        'layer': this.oObject.layer_name,
        'visible': true,
        'layerType': 'wmts',
        'matrixSet': this.oObject.matrix_set,
        'style': this.oObject.layer_style,
        'format': this.oObject.layer_format,
        'service_options': JSON.stringify(oCapabilities),
        'version': this.oObject.oParent.flux_version,
        'url': this.oObject.oParent.flux_url,
        'service_login': this.oObject.oParent.flux_login,
        'service_password': this.oObject.oParent.flux_password
      }]
    };
    return oChildrenService;
  }


  /**
   * Récupère la projection de la carte à partir d'une couche
   */
  public getMapProjFromWMTSLayer(oLayer: any, oCapabilities: any): Promise<any> {
    return new Promise((resolve, reject) => {
      this.mapCoreManagerService.getProjections().subscribe((oProjections: any) => {
        // Récupère l'index de projection à utiliser
        let sUsedProjectionIndex = oProjections.projections.findIndex((oProj) => oProj.code === 'EPSG:3857'); // par défaut 3857
        for (let i = 0; i < oProjections.projections.length; i++) {
          const index = this.wmtsService.getTileMatrixSetsIndex(oLayer, oCapabilities, oProjections.projections[i]);

          if (index !== -1) {
            sUsedProjectionIndex = this.vitisClone.clone(i);
            break;
          }
        }
        // Par sécurité si la valeur vaut -1 on prend l'element 0
        if (sUsedProjectionIndex === -1) {
          sUsedProjectionIndex = 0;
        }
        resolve(oProjections.projections[sUsedProjectionIndex].code);
      });
    });
  }
}