import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewOSMConnection } from './DatasourceTreeviewOSMConnection';
import { Injector } from '@angular/core';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewOSM {
  uId: number;
  name: string;
  id: number;
  iconImage: string = 'conf/images/openstreetmap.svg';
  collapsed: boolean = false;
  aChildrenTypes = [DatasourceTreeviewOSMConnection];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: '',
    label: 'ADMIN_DATASOURCE_BTN_ADD_OSM_CONNECTION',
    event: 'insertConnection',
  }];
  route: string = DatasourceTreeviewOSM.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'osm';
  }

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.name = datas.label;
    this.id = datas.datasourcetype_id;
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasources`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'name',
          attributs: 'datasource_id|name|description|flux_url|flux_version|flux_login|flux_password',
          filter: { 'relation': 'AND', 'operators': [{ 'column': 'datasourcetype_id', 'compare_operator': '=', 'value': this.id }] },
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              for (let i = 0; i < oResult.data.length; i++) {
                const datasource = oResult.data[i];
                const child = new DatasourceTreeviewOSMConnection(this.vitisCheck, this.vitisRequest, this.injector, datasource);
                this.aChildrenObjects.push(child);
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Lance le formulaire de création de l'objet enfant
   * @param {AdminDatasourceComponent} scope
   */
  public insertConnection(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewOSMConnection.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }
}