import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewPgDatabase } from './DatasourceTreeviewPgDatabase';
import { DatasourceTreeviewPgMaterializedViewList } from './DatasourceTreeviewPgMaterializedViewList';
import { DatasourceTreeviewPgTableList } from './DatasourceTreeviewPgTableList';
import { DatasourceTreeviewPgViewList } from './DatasourceTreeviewPgViewList';
import { Injector } from '@angular/core';

export class DatasourceTreeviewPgSchema {
  uId: number;
  oParent: DatasourceTreeviewPgDatabase;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-square';
  name: string;
  aChildrenTypes = [DatasourceTreeviewPgTableList];
  aChildrenObjects = [];
  route: string = DatasourceTreeviewPgSchema.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgschema';
  }

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: string,
  ) {
    this.name = datas;
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [
        new DatasourceTreeviewPgTableList(this.vitisCheck, this.vitisRequest, this.injector),
        new DatasourceTreeviewPgViewList(this.vitisCheck, this.vitisRequest, this.injector),
        new DatasourceTreeviewPgMaterializedViewList(this.vitisCheck, this.vitisRequest, this.injector),
      ];
      resolve(this.aChildrenObjects);
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgSchemaForm {
    const oFormValues: DatasourcePgSchemaForm = {
      name: this.name
    };

    return oFormValues;
  }
}


export class DatasourcePgSchemaForm {
  name: string;
}