import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { AdminDatasourceComponent } from '../../admin-datasource.component';
import { DatasourceTreeviewPgLayer } from './DatasourceTreeviewPgLayer';
import { DatasourceTreeviewPgLayerList } from './DatasourceTreeviewPgLayerList';
import { DatasourceTreeviewPgMaterializedViewList } from './DatasourceTreeviewPgMaterializedViewList';
import { DatasourceTreeviewPgPublication } from './DatasourceTreeviewPgPublication';
import { DatasourceTreeviewPgPublicationList } from './DatasourceTreeviewPgPublicationList';
import { DatasourceTreeviewPgTableList } from './DatasourceTreeviewPgTableList';
import { DatasourceTreeviewPgViewList } from './DatasourceTreeviewPgViewList';
import { environment } from 'src/environments/environment';
import { Injector } from '@angular/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';

export class DatasourceTreeviewPgTable {
  uId: number;
  oParent: DatasourceTreeviewPgTableList | DatasourceTreeviewPgViewList | DatasourceTreeviewPgMaterializedViewList;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-table';
  name: string;
  iCountData: number;
  srid: number;
  geomType: string;
  geomColumnFinded: string;
  isGeomTable: boolean;

  public aValues: Array<any>;
  public oTableInfo: any;

  // Services
  private adminDatasourceService: AdminDatasourceService;

  aChildrenTypes = [DatasourceTreeviewPgLayer, DatasourceTreeviewPgPublication];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshData',
  }];
  route: string = DatasourceTreeviewPgTable.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgtable/info';
  }

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.name = datas.name;
    if (this.vitisCheck.isDefAndNotNull(datas.icon)) {
      this.icon = datas.icon;
    }
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
  }


  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      this.aChildrenObjects.push(new DatasourceTreeviewPgLayerList(this.vitisCheck, this.vitisRequest, this.injector));
      this.aChildrenObjects.push(new DatasourceTreeviewPgPublicationList(this.vitisCheck, this.vitisRequest, this.injector));
      resolve(this.aChildrenObjects);
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgTableForm {
    let sParentType: string = 'table';
    if (this.oParent.constructor.name == 'DatasourceTreeviewPgViewList') {
      sParentType = 'view';
    }
    if (this.oParent.constructor.name == 'DatasourceTreeviewPgMaterializedViewList') {
      sParentType = 'matview';
    }

    const oFormValues: DatasourcePgTableForm = {
      name: this.name,
      parentType: sParentType,
      count_data: this.vitisCheck.isDefAndNotNull(this.iCountData) ? this.iCountData : 0,
      srid: this.vitisCheck.isDefAndNotNull(this.srid) ? this.srid : null,
      geom_type: this.vitisCheck.isDefAndNotNull(this.geomType) ? this.geomType : null,
      geom_column_finded: this.vitisCheck.isDefAndNotNull(this.geomColumnFinded) ? this.geomColumnFinded : null,
      isGeomTable: false
    };

    this.vitisRequest.ajaxRequest({
      method: 'GET',
      url: `${environment.properties.app_api_url}/vmap/datasourcetreeview/tableinfo`,
      params: {
        database: this.oParent.oParent.oParent.name,
        schema: this.oParent.oParent.name,
        pg_table: this.name,
        datasource_id: this.oParent.oParent.oParent.oParent.datasourceId,
        type_table: sParentType
      },
      async: false,
      responseType: '',
      displayErrorMessage: true,
      success: async (resp: any) => {
        if (this.vitisCheck.isString(resp.data)) {
          const oResult = JSON.parse(resp.data);
          if (oResult.status == 1 && oResult.data) {
            if (this.vitisCheck.isString(oResult.data)) {
              oResult.data = JSON.parse(oResult.data);
            }

            if (this.vitisCheck.isDefAndNotNull(oResult.data.count_data)) {
              oFormValues.count_data = oResult.data.count_data;
              this.iCountData = oResult.data.count_data;
            }

            if (this.vitisCheck.isDefAndNotNull(oResult.data.srid)) {
              oFormValues.srid = oResult.data.srid;
              this.srid = oResult.data.srid;
            }

            if (this.vitisCheck.isDefAndNotNull(oResult.data.geom_type)) {
              oFormValues.geom_type = oResult.data.geom_type;
              this.geomType = oResult.data.geom_type;
            }

            if (this.vitisCheck.isDefAndNotNull(oResult.data.geom_column_finded)) {
              oFormValues.geom_column_finded = oResult.data.geom_column_finded;
              this.geomColumnFinded = oResult.data.geom_column_finded;
            }
          }
        }
      }
    });

    if (this.vitisCheck.isDefAndNotNull(oFormValues.srid) &&
      this.vitisCheck.isDefAndNotNull(oFormValues.geom_type) &&
      this.vitisCheck.isDefAndNotNull(oFormValues.geom_column_finded)) {
      oFormValues.isGeomTable = true;
    }

    this.isGeomTable = oFormValues.isGeomTable;

    return oFormValues;
  }

  /**
   * Lance le formulaire de création de l'objet enfant
   * @param {AdminDatasourceComponent} scope
   */
  public createPgLayer(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewPgLayer.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }

  /**
   * Initialise les valeurs du formulaire insert enfant
   */
  public getChildrenInitialFormValues(): any {
    const oFormValues: any = {
      database: this.oParent.oParent.oParent.name,
      schema: this.oParent.oParent.name,
      pg_table: this.name,
      datasource_id: this.oParent.oParent.oParent.oParent.datasourceId
    };

    return oFormValues;
  }

  /**
   * Rafraichit les données de l'objet et la liste des enfants
   * @param scope
   */
  public refreshData(scope: AdminDatasourceComponent) {
    this.iCountData = null; // set null pour qu'au prochaine getFormValue les données soit recherché en base
    this.aChildrenObjects = null; // set null pour que scope.getChildrens recharge les données
    scope.getChildrens(this); // recharge les enfants à partir du scope pour que l'arbre se mette à jour
    this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
  }

  /**
   * Récupère l'url et les paramètres pour afficher les 100 premières données de la table dans la grille du second onglet
   */
  public getDataGridUrlAndParams(): any {
    return {
      url: `${environment.properties.app_api_url}/vmap/datasourcetreeview/tablevalues`,
      params: {
        database: this.oParent.oParent.oParent.name,
        schema: this.oParent.oParent.name,
        pg_table: this.name,
        datasource_id: this.oParent.oParent.oParent.oParent.datasourceId
      }
    };
  }

  /**
   * Récupère en base les 100 premières valeurs d'une table
   */
  public getTableDatas(): Promise<any> {
    return new Promise((resolve, reject) => {
      const oUrlAndParams = this.getDataGridUrlAndParams();
      const params = oUrlAndParams.params;
      params.get_geom_column = true;
      params.limit = 100;
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: oUrlAndParams.url,
        params: params,
        success: async (resp: any) => {
          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (resp.data.status == 1) {
              if (this.vitisCheck.isDefAndNotNull(resp.data.data) && this.vitisCheck.isDefAndNotNull(resp.data.tableinfo)) {
                this.aValues = resp.data.data;
                this.oTableInfo = resp.data.tableinfo;

                const aGeoms = [];

                for (let i = 0; i < this.aValues.length; i++) {
                  const oValue = this.aValues[i];
                  const sGeom = oValue[this.oTableInfo.geom_column_finded];
                  if (this.vitisCheck.isDefAndNotNull(sGeom)) {
                    aGeoms.push(sGeom);
                  }
                }
                resolve({ table_map_layers: aGeoms });
              }
            }
          }
        }
      });
    });
  }
}


export class DatasourcePgTableForm {
  name: string;
  parentType: string; // table | view | matview
  count_data?: number;
  srid?: number;
  geom_type?: string;
  geom_column_finded?: string;
  isGeomTable: boolean;
}