import { Injector } from '@angular/core';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from '../../admin-datasource.component';
import { DatasourceTreeviewWMSConnection } from './DatasourceTreeviewWMSConnection';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { datasourceTreeviewWMSForm } from '../../forms/DatasourceTreeviewWMS/DatasourceTreeviewWMS.form';

export class DatasourceTreeviewWMS {
  uId: number;
  name: string;
  id: number;
  icon: string = 'icon-vmap-arrow-redo';
  collapsed: boolean = false;
  aChildrenTypes = [DatasourceTreeviewWMSConnection];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: '',
    label: 'ADMIN_DATASOURCE_BTN_ADD_WMS_CONNECTION',
    event: 'insertConnection',
  },
  {
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshData',
  }];
  route: string = DatasourceTreeviewWMS.getRoute();

  private adminDatasourceService: AdminDatasourceService;

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.name = datas.label;
    this.id = datas.datasourcetype_id;
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
  }

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'wms';
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasources`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'name',
          attributs: 'datasource_id|name|description|flux_url|flux_version|flux_login|flux_password',
          filter: { 'relation': 'AND', 'operators': [{ 'column': 'datasourcetype_id', 'compare_operator': '=', 'value': this.id }] },
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              for (let i = 0; i < oResult.data.length; i++) {
                const datasource = oResult.data[i];
                const child = new DatasourceTreeviewWMSConnection(this.vitisCheck, this.vitisRequest, this.injector, datasource);
                this.aChildrenObjects.push(child);
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Lance le formulaire de création de l'objet enfant
   * @param {AdminDatasourceComponent} scope
   */
  public insertConnection(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewWMSConnection.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }

  /**
   * Rafraichit les données de l'objet
   * @param scope
   */
  public refreshData(scope: AdminDatasourceComponent) {
    // Rechargement de la source de données WMS.
    this.aChildrenObjects = null;
    scope.getChildrens(this);
    // Page d'accueil de la source de données WMS.
    scope.openRoute({
      route: this.route,
      formName: 'display',
      uId: this.uId
    });
  }
}