import { LocalStorage, VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from 'src/modules/vmap/components/admin-datasource/admin-datasource.component';
import { DatasourceTreeviewWMSConnection } from 'src/modules/vmap/components/admin-datasource/models/DatasourceTreeviewWMS/DatasourceTreeviewWMSConnection';
import { Injector } from '@angular/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';

export class DatasourceTreeviewWMSLayer {
  uId: number;
  oParent: DatasourceTreeviewWMSConnection;
  icon: string = 'icon-vmap-layers-half';

  layer_id: number;
  name: string;
  datasource: any;
  datasource_id: number;
  datemaj: Date;
  source_id: number;
  source_name: string;
  extent: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  is_background_layer: boolean;
  theme_label: string;
  theme_id: number;
  layer_name: string;
  crs_list: string;
  layer_style: string;
  layer_format: string;
  layer_options: string;
  is_tiled: boolean;

  // Services
  private adminDatasourceService: AdminDatasourceService;
  private localStorage: LocalStorage;

  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_WMS_LAYER',
    event: 'updateWMSLayer',
  }, {
    icon: 'fa fa-clone',
    label: 'ADMIN_DATASOURCE_BTN_CLONE_LAYER',
    event: 'cloneWMSLayer'
  }, {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_WMS_LAYER',
    event: 'deleteWMSLayer',
  }];
  route: string = DatasourceTreeviewWMSLayer.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'wmslayer/info';
  }

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.setData_(datas);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
    this.localStorage = this.injector.get(LocalStorage);
  }

  /**
   * Rafraichit les données de l'objet
   */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/layers/${this.layer_id}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData_(oResult.data);
          if (this.vitisCheck.isDefAndNotNull(this.adminDatasourceService)) {
            this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
          }
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData_(datas: any) {
    this.layer_id = datas.layer_id;
    this.name = datas.name;
    this.datasource = datas.datasource;
    this.datasource_id = datas.datasource_id;
    this.datemaj = datas.datemaj;
    this.source_id = datas.source ? datas.source.source_id : null;
    this.source_name = datas.source ? datas.source.name : null;
    this.extent = datas.extent;
    this.min_layer_scale = datas.min_layer_scale;
    this.max_layer_scale = datas.max_layer_scale;
    this.active = datas.active;
    this.is_background_layer = datas.is_background_layer;
    this.theme_label = datas.theme ? datas.theme.label : null;
    this.theme_id = datas.theme ? datas.theme.theme_id : null;
    this.layer_name = datas.layer_name;
    this.crs_list = datas.crs_list;
    this.layer_style = datas.layer_style;
    this.layer_format = datas.layer_format;
    this.layer_options = JSON.parse(datas.layer_options);
    this.is_tiled = datas.is_tiled;
  }


  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): Promise<DatasourceWMSLayerForm> {
    return new Promise((resolve, reject) => {
      const oFormValues: DatasourceWMSLayerForm = {
        layer_id: this.layer_id,
        name: this.name,
        datasource: this.datasource,
        datasource_id: this.datasource_id,
        datemaj: this.datemaj,
        source_id: this.source_id,
        source_name: this.source_name,
        extent: this.extent,
        min_layer_scale: this.min_layer_scale,
        max_layer_scale: this.max_layer_scale,
        active: this.active,
        is_background_layer: this.is_background_layer,
        theme_label: this.theme_label,
        theme_id: this.theme_id,
        layer_name: this.layer_name,
        crs_list: this.crs_list,
        layer_style: this.layer_style,
        layer_format: this.layer_format,
        layer_options: this.layer_options,
        is_tiled: this.is_tiled
      };

      if ((this.vitisCheck.isDefAndNotNull(oFormValues.layer_options) && oFormValues.layer_options != '')
        || !this.vitisCheck.isDefAndNotNull(this.oParent) || !this.vitisCheck.isFunction(this.oParent.getCapabilitiesWMS)) {
        resolve(oFormValues);
      } else { // Par sécurité si le layer_options n'existe pas on charge le oCapabilities
        this.oParent.getCapabilitiesWMS().then((oCapabilities) => {
          oFormValues.layer_options = oCapabilities.json_capabilities;
          resolve(oFormValues);
        });
      }
    });
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public updateWMSLayer(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewWMSLayer.getRoute(),
      formName: 'update',
      uId: this.uId
    });
  }

  /**
   * Sauvegarde le uId et renvoie vers le formulaire d'ajout
   */
   public cloneWMSLayer(scope: AdminDatasourceComponent) {
    this.localStorage.set('idClone', this.uId);
    const oObject = this.adminDatasourceService.getObjectById(this.uId);
    scope.openRoute({
      route: DatasourceTreeviewWMSLayer.getRoute(),
      formName: 'insert',
      uId: oObject.oParent.uId
    });
  }

  /**
   * Supprimer une couche WMS
   * @param scope
   */
  public deleteWMSLayer(scope: AdminDatasourceComponent) {
    this.adminDatasourceService.deleteLayer(this.layer_id, this.uId);
  }

}

export class DatasourceWMSLayerForm {
  layer_id: number;
  name: string;
  datasource: any;
  datasource_id: number;
  datemaj: Date;
  source_id: number;
  source_name: string;
  extent: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  is_background_layer: boolean;
  theme_label: string;
  theme_id: number;
  layer_name: string;
  crs_list: string;
  layer_style: string;
  layer_format: string;
  layer_options: string;
  is_tiled: boolean;
}