import { VitisCheck, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from '../../admin-datasource.component';
import { DatasourceTreeviewWMTSLayer } from './DatasourceTreeviewWMTSLayer';
import { Injector } from '@angular/core';
import { WmtsService } from 'src/modules/vmap/components/admin-datasource/services/wmts.service';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { TranslateService } from '@ngx-translate/core';

export class DatasourceTreeviewWMTSConnection {

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.wmtsService = this.injector.get(WmtsService);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
    this.translate = this.injector.get(TranslateService);
    this.setData_(datas);
  }

  uId: number;
  icon: string = 'icon-vmap-wms';
  collapsed: boolean = false;
  // Services
  wmtsService: WmtsService;
  adminDatasourceService: AdminDatasourceService;
  translate: TranslateService;

  public datasourceId: number;
  public datasource: any;
  public name: string;
  public description: string;
  public flux_url: string;
  public flux_version: string;
  public flux_type: string;
  public flux_options: string;
  public flux_login: string;
  public flux_password: string;

  public oCapabilities: any;
  aChildrenTypes = [DatasourceTreeviewWMTSLayer];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: 'icon-vmap-wms',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_WMTS_CONNECTION',
    event: 'updateWMTSConnection',
  }, {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_WMTS_CONNECTION',
    event: 'deleteWMTSConnection',
  }, {
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_ADD_WMTS_LAYER',
    event: 'insertWMTSLayer',
  }];
  route: string = DatasourceTreeviewWMTSConnection.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'wmtsconnection/info';
  }

  /**
   * Rafraichit les données de l'objet
   */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/datasources/${this.datasourceId}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      params: {
        attributs: 'datasource_id|name|description|flux_url|flux_type|flux_version|flux_login|flux_password|flux_options'
      },
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData_(oResult.data);
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData_(datas: any) {
    this.datasourceId = datas.datasource_id;
    this.datasource = datas.datasource;
    this.name = datas.name;
    this.description = datas.description;
    this.flux_url = datas.flux_url;
    this.flux_type = datas.flux_type;
    this.flux_version = datas.flux_version;
    this.flux_options = datas.flux_options;
    this.flux_login = datas.flux_login;
    this.flux_password = datas.flux_password;
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/wmtslayers`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          filter: { 'relation': 'AND', 'operators': [{ 'column': 'datasource_id', 'compare_operator': '=', 'value': this.datasourceId }] },
          order_by: 'name'
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              for (let i = 0; i < oResult.data.length; i++) {
                const service = oResult.data[i];
                this.aChildrenObjects.push(new DatasourceTreeviewWMTSLayer(this.vitisCheck, this.vitisRequest, this.injector, service));
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourceWMTSConnectionForm {
    const oFormValues: DatasourceWMTSConnectionForm = {
      name: this.name,
      description: this.description,
      flux_url: this.flux_url,
      flux_type: this.flux_type,
      flux_version: this.flux_version,
      flux_options: this.flux_options,
      flux_login: this.flux_login,
      flux_password: this.flux_password,
      datasource_id: this.datasourceId,
      datasource: this.datasource
    };

    return oFormValues;
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public updateWMTSConnection(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewWMTSConnection.getRoute(),
      formName: 'update',
      uId: this.uId
    });
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public deleteWMTSConnection(scope: AdminDatasourceComponent) {
    this.loadChildrens().then(resp => {
      if (resp.length) {
        this.translate.get('ADMIN_DATASOURCE_BTN_DELETE_CONNECTION_WARNING').subscribe((translation) => {
          vitisAlert.alert({
            type: 'error',
            text: translation,
            showCancelButton: true
          });
        });
      } else {
        this.adminDatasourceService.deleteDatasource(this.datasourceId, this.uId);
      }
    });
  }

  /**
   * Lance le formulaire d'ajout d'une couche
   * @param {AdminDatasourceComponent} scope
   */
  public insertWMTSLayer(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewWMTSLayer.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }

  /**
   * Utilisé comme getFormValues pour l'onglet avec la carte, pour attendre que le oCapabilities soit chargé avant d'ouvrir le formulaire
   */
  public beforeInitForm(): Promise<any> {
    return new Promise((resolve, reject) => {
      if (this.vitisCheck.isDefAndNotNull(this.getChildrens()) && this.getChildrens().length > 0) {
        this.getCapabilitiesWMTS().then((oXmlCapabilities: any) => {
          resolve(oXmlCapabilities);
        });
      } else {
        // Si besoin charge les enfants pour pouvoir vérifier si les couches sont déjà chargé
        this.loadChildrens().then((aChildrens) => {
          this.adminDatasourceService.setChildrensMetadata(aChildrens, this);
          // Récupère le oCapabilities
          this.getCapabilitiesWMTS().then((oXmlCapabilities: any) => {
            resolve(oXmlCapabilities);
          });
        });
      }
    });
  }

  /**
   * Utilisé comme getFormValues pour l'onglet avec la carte, pour attendre que le oCapabilities soit chargé avant d'ouvrir le formulaire
   */
  public getCapabilitiesWMTS(): Promise<any> {
    return new Promise((resolve, reject) => {
      // Récupère le oCapabilities
      if (this.vitisCheck.isDefAndNotNull(this.oCapabilities)) {
        resolve({
          get_capabilities: JSON.stringify(this.oCapabilities, null, '\t'),
          json_capabilities: this.oCapabilities
        });
      } else {
        this.wmtsService.getAjaxCapabilitiesWMTS(this).then((oCapabilities) => {
          if (oCapabilities !== false) {
            this.oCapabilities = oCapabilities;
            resolve({
              get_capabilities: JSON.stringify(this.oCapabilities, null, '\t'),
              json_capabilities: this.oCapabilities
            });
          } else {
            resolve(false);
          }
        });
      }
    });
  }

  /**
   * Initialise les valeurs du formulaire insert enfant
   */
  public getChildrenInitialFormValues(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.getCapabilitiesWMTS().then((oCapabilities) => {
        if (oCapabilities !== false) {
          const oFormValues = {
            layer_options: oCapabilities.json_capabilities,
            datasource_id: this.datasourceId,
            datasource: this.getFormValues(),
          };
          resolve(oFormValues);
        } else {
          resolve(false);
        }
      });
    });
  }
}

export class DatasourceWMTSConnectionForm {
  name: string;
  description: string;
  flux_url: string;
  flux_type: string;
  flux_version: string;
  flux_options: string;
  flux_login: string;
  flux_password: string;
  datasource_id: number;
  datasource: number;
}