import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { MapCoreSmallScreenService } from 'src/app/vitis/map-core/services/map-core-small-screen/map-core-small-screen.service';

@Component({
  selector: 'app-vmap-controls-left-buttons',
  templateUrl: './vmap-controls-left-buttons.component.html',
  styleUrls: ['./vmap-controls-left-buttons.component.scss']
})
export class VmapControlsLeftButtonsComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() isWidget: boolean;
  @Input() haveLayersPanel: boolean;

  layers_tool: boolean = false;

  public isSmallScreen;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    private route: ActivatedRoute,
    private mapCoreSmallScreenService: MapCoreSmallScreenService
  ) { }

  ngOnInit(): void {

    this.isSmallScreen = window.innerWidth <= 768;
    this.subscriptions.push(this.mapCoreSmallScreenService.smallScreenObservable.subscribe(isSmallScreen => {
      this.isSmallScreen = isSmallScreen;
    }));

    this.subscriptions.push(this.route.queryParams.subscribe(oParams => {
      // Affiche les outils de couches si layers_tool === 'true'
      if (oParams.layers_tool === 'true') {
        this.layers_tool = true;
      }
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }
}
