import { Component, Input, OnInit } from '@angular/core';
import { Map } from 'ol';
import { OverviewMap } from 'ol/control';
import TileLayer from 'ol/layer/Tile';
import ImageLayer from 'ol/layer/Image';
import { MapCoreJSONService } from 'src/app/vitis/map-core/services/map-core-json/map-core-json.service';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';

@Component({
  selector: 'app-vmap-controls-overview',
  templateUrl: './vmap-controls-overview.component.html',
  styleUrls: ['./vmap-controls-overview.component.scss']
})
export class VmapControlsOverviewComponent implements OnInit {

  @Input() olMap: Map;
  @Input() isSmallScreen: boolean;

  private overviewMapControl: OverviewMap;
  public bCollapsed: boolean = false;

  constructor(
    private mapCoreJSONService: MapCoreJSONService,
    private mapCoreManagerService: MapCoreManagerService,
  ) { }

  ngOnInit(): void {
    this.olMap.once('rendercomplete', () => {
      this.initOverview();
    });

    this.olMap.on('change', () => {
      this.initOverview();
    });
  }

  /**
   * Initialise l'overview
   */
  public initOverview() {

    if (this.overviewMapControl) {
      this.olMap.removeControl(this.overviewMapControl);
      this.olMap.set('overview', null);
    }

    this.overviewMapControl = new OverviewMap({
      className: 'ol-overviewmap ol-custom-overviewmap',
      layers: this.getOverviewLayers_(),
      collapsed: false,
      target: document.getElementById('mapOverviewElement'),
    });

    this.olMap.addControl(this.overviewMapControl);
    this.olMap.set('overview', this.overviewMapControl);
  }

  /**
   * Toogle
   */
  public toggleCollapse() {
    const collapsed = this.overviewMapControl.getCollapsed();
    this.overviewMapControl.setCollapsed(!collapsed);
    this.bCollapsed = !collapsed;
  }

  /**
   * Retourne les couches à afficher sur l'overview
   * @returns
   */
  private getOverviewLayers_() {
    const oMapDef = this.mapCoreJSONService.getMapDef(this.olMap);
    let aLayers = this.mapCoreJSONService.getLayersFromDef(oMapDef, {
      size: this.olMap.getSize(),
      tileSize: [250, 250]
    }, this.olMap);

    // Change le DPI des couches Mapserver
    aLayers = this.mapCoreManagerService.changeLayersDPI([...aLayers], 288);

    // Optimisaiton des couches
    if (oMapDef.options && oMapDef.options.optimized) {
      aLayers = this.mapCoreManagerService.optimiseLayers([...aLayers]);
    }

    return aLayers;
  }
}
