import { Component, Input, OnInit, Output, EventEmitter } from '@angular/core';
import { Map } from 'ol';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { LayerLegend, LegendObject } from './vmap-layers-legend.model';

@Component({
  selector: 'app-vmap-layers-legend',
  templateUrl: './vmap-layers-legend.component.html',
  styleUrls: ['./vmap-layers-legend.component.scss']
})
export class VmapLayersLegendComponent implements OnInit {

  public bSecondPanelOpen: boolean = false;

  @Input() public olMap: Map;
  @Output() editLayerLegend = new EventEmitter();

  public aLayersLegends: Array<LayerLegend> = [];

  constructor(
    private vitisCheck: VitisCheck,
    private vmapToolsService: VmapToolsService,
  ) { }

  ngOnInit(): void {
    this.aLayersLegends = this.vmapToolsService.getLayersLegends(this.olMap);

    this.olMap.on('change', () => {
      this.aLayersLegends = this.vmapToolsService.getLayersLegends(this.olMap);
    });
  }

  /**
   * Ouvre le second panneau
   */
  public toggleSecondPanel(bForceValue?: boolean) {
    if (this.vitisCheck.isBoolean(bForceValue)) {
      this.bSecondPanelOpen = bForceValue;
    } else {
      this.bSecondPanelOpen = !this.bSecondPanelOpen;
    }

    // Panneau de gauche ouvert à la taille 3 ou 1 si fermé
    if (this.bSecondPanelOpen) {
      this.vmapToolsService.setLeftPanel(3, 'control-layers-legend');
    } else {
      this.vmapToolsService.setLeftPanel(1, 'control-layers');
    }
  }

  /**
   * Édite la légende
   * @param oLegendObject
   */
  public editLegend(oLegendObject: LegendObject) {
    this.toggleSecondPanel(true);
    if (this.bSecondPanelOpen) {
      this.editLayerLegend.next(oLegendObject);
    } else {
      this.editLayerLegend.next(null);
    }
  }
}
