import { Component, Input, OnInit } from '@angular/core';
import { Map } from 'ol';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';

@Component({
  selector: 'app-vmap-layers-settings',
  templateUrl: './vmap-layers-settings.component.html',
  styleUrls: ['./vmap-layers-settings.component.scss']
})
export class VmapLayersSettingsComponent implements OnInit {

  @Input() olMap: Map;
  @Input() oLayerElement: any;

  public bIsLayerVisible: boolean = true;
  public iLayerOpacity: number = 100;

  constructor(
    private mapCoreManagerService: MapCoreManagerService,
  ) { }

  ngOnInit(): void {
    this.iLayerOpacity = this.getLayerOpacity(this.oLayerElement);
    this.bIsLayerVisible = this.getLayerVisibility(this.oLayerElement);
  }

  /**
   * Retourne l'opacité de la couche dans l'arbre
   * @param layerElement couche dans l'arbre
   * @returns
   */
  private getLayerOpacity(layerElement: any): number {
    if (layerElement.layer_id) {
      return layerElement.olLayer.get('opacity') * 100;
    }

    if (layerElement.children) {
      let greaterOpacity = 0;

      for (const children of layerElement.children) {
        const opacity = this.getLayerOpacity(children);

        if (opacity > greaterOpacity) {
          greaterOpacity = opacity;
        }
      }

      return greaterOpacity;
    }
  }

  /**
   * Retourne la visibilité de la couche dans l'arbre
   * @param layerElement couche dans l'arbre
   * @returns
   */
  private getLayerVisibility(layerElement: any): boolean {
    if (layerElement.layer_id) {
      return layerElement.olLayer.get('visible');
    }

    if (layerElement.children) {
      let visible = false;

      for (const children of layerElement.children) {
        const childrenVisibility = this.getLayerVisibility(children);

        if (childrenVisibility) {
          visible = true;
        }
      }

      return visible;
    }
  }

  /**
   * Set l'opacité d'une couhe
   * @param layerElement
   * @param opacity
   */
  private setLayerOpacity(layerElement: any, opacity: number) {
    if (layerElement.layer_id) {
      this.mapCoreManagerService.setLayerOpacity(layerElement, this.olMap, opacity / 100);
    }

    if (layerElement.children) {
      for (const children of layerElement.children) {
        this.setLayerOpacity(children, opacity);
      }
    }
  }

  /**
   * Set la visibilité d'une couche
   * @param layerElement
   * @param visibility
   */
  private setLayerVisibility(layerElement: any, visibility: boolean) {
    if (layerElement.layer_id) {
      this.mapCoreManagerService.setLayerVisible(layerElement, this.olMap, visibility);
    }

    if (layerElement.children) {
      for (const children of layerElement.children) {
        this.setLayerVisibility(children, visibility);
      }
    }
  }

  /**
   * Appelé lors du changement d'opacité depuis le curseur
   * @param value
   */
  public opacityChanged(value: number) {
    this.iLayerOpacity = value;
    this.setLayerOpacity(this.oLayerElement, this.iLayerOpacity);
  }

  /**
   * Appelé lors du changement de visibilité depuis le bouton
   * @param value
   */
  public toggleVisibility() {
    this.bIsLayerVisible = !this.bIsLayerVisible;
    this.setLayerVisibility(this.oLayerElement, this.bIsLayerVisible);
  }

}

