import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { Map } from 'ol';
import { LegendObject } from 'src/modules/vmap/components/carto/components/vmap-layers-panel/components/vmap-layers-legend/vmap-layers-legend.model';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { MapCoreSmallScreenService } from 'src/app/vitis/map-core/services/map-core-small-screen/map-core-small-screen.service';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-vmap-layers-panel',
  templateUrl: './vmap-layers-panel.component.html',
  styleUrls: ['./vmap-layers-panel.component.scss']
})
export class VmapLayersPanelComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;

  public bPanelOpen: boolean = false;
  public selectedTab = 0;
  public iNbCols = 0;
  public oLegendObject: LegendObject;
  public bEditLayerLegendOpen: boolean = false;
  public sMapName: string;

  // Liste des Contrôles de la carte.
  public oEnabledControls: any;

  public mapCompare: boolean;
  public sMapCompareName: string;
  public olMapCompare: Map;
  public selectedCompareTab = 0;

  public isSmallScreen;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    private vmapToolsService: VmapToolsService,
    private mapCoreSmallScreenService: MapCoreSmallScreenService,
  ) { }

  ngOnInit(): void {

    this.isSmallScreen = window.innerWidth <= 768;
    this.subscriptions.push(this.mapCoreSmallScreenService.smallScreenObservable.subscribe(isSmallScreen => {
      this.isSmallScreen = isSmallScreen;
    }));

    // Nom de la carte
    this.sMapName = this.olMap.get('name');

    this.olMap.on('change', (event: any) => {
      setTimeout(() => {
        if (event.target.get('map_compare')) {
          this.mapCompare = true;
          this.olMapCompare = event.target.get('map_compare');
          // Nom de la carte de comparaison
          this.sMapCompareName = this.olMapCompare.get('name');
        } else {
          this.mapCompare = false;
        }
        this.sMapName = event.target.get('name');
      });
    });

    // Écoute l'ouverture des menus
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'control-layers');
      this.bEditLayerLegendOpen = (sTool === 'control-layers-legend');
    }));
    this.subscriptions.push(this.vmapToolsService.getLeftPanelEmiter().subscribe((iNbCols: number) => {
      this.iNbCols = iNbCols;
      // Sauve la liste des Contrôles de la carte.
      this.oEnabledControls = this.olMap.get('enabled_controls');
      // Activation du 1er contrôle de la carte principale.
      const aControls = ['layers_tree_tab', 'layers_list_tab', 'layers_legend_tab'];
      for (let i = 0; i < aControls.length; i++) {
        if (this.oEnabledControls[aControls[i]] === true) {
          this.selectedTab = i;
          break;
        }
      }
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Modification d'une légende
   * @param oLegendObject
   */
  public editLayerLegend(oLegendObject: LegendObject) {
    this.oLegendObject = oLegendObject;
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.bPanelOpen = false;
    this.vmapToolsService.setLeftPanel(0, '');
  }

  /**
   * Sélection d'un onglet pour la carte principale
   * @param index
   */
  public selectTab(index) {
    this.selectedTab = index;
  }

  /**
   * Sélection d'un onglet pour la carte de comparaison
   * @param index
   */
  public selectCompareTab(index) {
    this.selectedCompareTab = index;
  }
}
