import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { MapsTheme, Theme } from './vmap-maps-panel.model';

@Component({
  selector: 'app-vmap-maps-panel',
  templateUrl: './vmap-maps-panel.component.html',
  styleUrls: ['./vmap-maps-panel.component.scss']
})
export class VmapMapsPanelComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;

  public bPanelOpen: boolean = false;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  get mapId() {
    return this.route.snapshot.queryParams['map_id'];
  }

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private vmapToolsService: VmapToolsService,
  ) { }

  ngOnInit(): void {
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'control-maps');
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.bPanelOpen = false;

    this.vmapToolsService.setLeftPanel(0, '');
  }

  /**
   * Load une carte
   */
  public loadMap(map_id) {

    // Change le map_id dans l'URL
    this.router.navigate(['/dashboad/map'], { queryParams: { map_id: map_id } });

    // Ferme l'outil
    this.closePanel();
  }
}
