import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { Map } from 'ol';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import usersForm from 'src/assets/forms/user_infos/user_infos.json';
import { VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-vmap-tools-menu-mobile',
  templateUrl: './vmap-tools-menu-mobile.component.html',
  styleUrls: ['./vmap-tools-menu-mobile.component.scss']
})
export class VmapToolsMenuMobileComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;

  public bPanelOpen: boolean;

  public openUserInfos = false;
  public sDefinitionUser: any;
  public sDatasUser: any = {};

  public dropDownUser = false;
  public dropdownUserId = `dropdown-${new Date().getTime()}`;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    private vmapToolsService: VmapToolsService,
    private authService: AuthService,
    private vitisRequest: VitisRequest,
    private formreaderSrvc: FormreaderService
  ) { }

  ngOnInit() {
    this.sDefinitionUser = usersForm;
    this.sDefinitionUser.update.eventContainer = this;
    this.sDatasUser = this.authService.getUserInfos();

    // Écoute l'ouverture des menus
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'menu-mobile');
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.bPanelOpen = false;
    this.vmapToolsService.setLeftPanel(0, '');
  }

  /**
   * Déconnectes l'utilisateur en cours
   */
  public disconnect() {
    this.authService.disconnect();
  }

  /**
   * Evenement appelé avant la validation du formulaire de modification de l'utilisateur
   * @returns
   */
  public beforeEvent() {
    return true;
  }

  /**
   * Evenement appelé lors de la validation du formulaire de modification de l'utilisateur
   */
  public sendSimpleForm() {
    this.vitisRequest.ajaxRequestPromise({
      method: 'PUT',
      url: `${environment.properties.app_api_url}/vitis/users/${this.sDatasUser.user_id}`,
      data: this.formreaderSrvc.transformToDataForm(this.sDatasUser)
    }).catch(e => {
      console.error(e);
    });
  }

  /**
   * Evement appelé après la validation du formulaire de modification de l'utilisateur
   */
  public afterEvent() {
    this.authService.setUserInfos(this.sDatasUser);
  }

  /**
   * Listener lors de la modification du formulaire de l'utilisateur
   * @param form
   */
  public formUsersChanges(form) {
    this.sDatasUser = form;
  }
}
