import { Component, OnInit, Input, Output, EventEmitter, OnDestroy } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { Subscription } from 'rxjs';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';

@Component({
  selector: 'app-vmap-tools-map-list',
  templateUrl: './vmap-tools-map-list.component.html',
  styleUrls: ['./vmap-tools-map-list.component.scss']
})
export class VmapToolsMapListComponent implements OnInit, OnDestroy {

  @Input() mapSelected: number;
  @Output() selectMap = new EventEmitter();

  private maps;
  public themes;
  public mapsPerThemes;

  public openThemes: Array<string> = [];

  public filterMapName;
  public filterTheme = 'all';

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    private mapCoreManagerService: MapCoreManagerService,
    private translate: TranslateService,
  ) { }

  ngOnInit(): void {
    this.subscriptions.push(this.mapCoreManagerService.getMapCatalog().subscribe(mapCatalog => {
      this.maps = mapCatalog.maps;
      this.groupMapsPerThemes(this.maps);
      this.themes = Object.keys(this.mapsPerThemes);
      this.openThemes = this.themes;
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Regroupe les cartes par thèmes
   * @param maps
   */
  public groupMapsPerThemes(maps) {
    this.mapsPerThemes = maps.reduce((r, a) => {
      if (a.theme === null) { a.theme = this.translate.instant('COMPONENT_MAP_MAP_LIST_OTHER_THEME'); }
      const key = a.theme;
      r[key] = r[key] || [];
      r[key].push(a);
      return r;
    }, Object.create(null));
  }

  /**
   * Ouvre / Ferme un thème
   * @param theme
   */
  public collapseTheme(theme) {
    if (this.openThemes.includes(theme)) {
      this.openThemes = this.openThemes.filter(openTheme => openTheme !== theme);
    } else {
      this.openThemes.push(theme);
    }
  }

  /**
   * Update le filtre
   */
  public onChangeFilter() {
    const mapsFiltred = this.filterTheme !== 'all' ?
      this.maps.filter(map => map.theme.toLowerCase().search(this.filterTheme.toLowerCase()) !== -1 &&
        map.name.toLowerCase().search(this.filterMapName.toLowerCase()) !== -1) :
      this.maps.filter(map => map.name.toLowerCase().search(this.filterMapName.toLowerCase()) !== -1);
    this.groupMapsPerThemes(mapsFiltred);
  }
}
