import { Component, OnInit, Input, EventEmitter, ViewChild, ElementRef, ViewChildren, QueryList, OnDestroy } from '@angular/core';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { MapToolDef } from '../../map-tools.model';
@Component({
  selector: 'app-vmap-tools-tabs',
  templateUrl: './vmap-tools-tabs.component.html',
  styleUrls: ['./vmap-tools-tabs.component.scss']
})
export class VmapToolsTabsComponent implements OnInit, OnDestroy {

  @ViewChild('leftBtn') public leftBtn: ElementRef;
  @ViewChild('rightBtn') public rightBtn: ElementRef;
  @ViewChild('scrollArea') public scrollArea: ElementRef;
  @ViewChildren('tab') public tabs: QueryList<ElementRef>;

  @Input() olMap: Map;
  @Input() oTool: MapToolDef;

  @Input() public oTabsEmitter: EventEmitter<Array<any>>;
  @Input() public oSelectedTabEmitter: EventEmitter<number>;

  public aTabs: Array<any> = [];
  public iSelectedTabIndex = 0;

  // Nombre de pixels à faire défiler au moment du défilement
  private iScrollOffset = 370;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    public vmapToolsService: VmapToolsService,
  ) { }

  ngOnInit(): void {
    this.subscriptions.push(this.oTabsEmitter.subscribe((next: Array<any>) => {
      this.aTabs = next;

      if (this.aTabs.length > 0) {
        this.iSelectedTabIndex = 0;
        this.aTabs[this.iSelectedTabIndex].selected = true;
      }

      // Une fois les tabs changés, rafraîchir l'état des boutons de défilement
      setTimeout(() => {
        this.refreshScrollBar();
      });
    }));

    this.subscriptions.push(this.oSelectedTabEmitter.subscribe((next: number) => {
      if (this.aTabs[this.iSelectedTabIndex]) {
        this.aTabs[this.iSelectedTabIndex].selected = false;
      }

      if (this.aTabs[next]) {
        this.aTabs[next].selected = true;
      }

      this.iSelectedTabIndex = next;

      // Une fois l'index changé, rafraîchir l'état des boutons de défilement
      setTimeout(() => {
        this.refreshScrollBar();
      });
    }));

    this.subscriptions.push(this.vmapToolsService.getLeftPanelEmiter().subscribe(() => {
      setTimeout(() => {
        this.refreshScrollBar();
      }, 500);
    }));

    this.subscriptions.push(this.vmapToolsService.getRightPanelEmiter().subscribe(() => {
      setTimeout(() => {
        this.refreshScrollBar();
      }, 500);
    }));

    this.subscriptions.push(this.vmapToolsService.getBottomPanelEmiter().subscribe(() => {
      setTimeout(() => {
        this.refreshScrollBar();
      }, 500);
    }));

    setTimeout(() => {
      this.refreshScrollBar();
    });
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Fonction déclenchée au moment de la sélection de tab
   * @param tabIndex
   */
  public onTabSelect(tabIndex: number) {
    if (this.iSelectedTabIndex !== tabIndex) {
      this.oSelectedTabEmitter.next(tabIndex);
    }
  }

  /**
   * Permet de rafraîchir l'état des boutons de défilement des tabs
   */
  public refreshScrollBar() {
    const containerWidth = this.scrollArea.nativeElement.clientWidth;

    let tabsWidth = 0;
    const tabs = this.tabs.toArray();
    for (let i = 0; i < tabs.length; i++) {
      const oTabElement = tabs[i].nativeElement;
      tabsWidth += oTabElement.clientWidth;
    }

    if (containerWidth - tabsWidth < 0) {
      const scrollDiff = this.scrollArea.nativeElement.scrollWidth - this.scrollArea.nativeElement.clientWidth;
      const percentage = Math.floor((this.scrollArea.nativeElement.scrollLeft / scrollDiff) * 100);

      if (percentage === 0) {
        this.leftBtn.nativeElement.classList.add('hidden');
        this.rightBtn.nativeElement.classList.remove('hidden');
      } else if (percentage === 100) {
        this.leftBtn.nativeElement.classList.remove('hidden');
        this.rightBtn.nativeElement.classList.add('hidden');
      } else {
        this.leftBtn.nativeElement.classList.remove('hidden');
        this.rightBtn.nativeElement.classList.remove('hidden');
      }
    } else {
      this.leftBtn.nativeElement.classList.add('hidden');
      this.rightBtn.nativeElement.classList.add('hidden');
    }
  }

  /**
   * Fonction déclenchée au clic du bouton de défilement de la barre d'onglets
   * @param direction
   */
  public onScrollClick(direction: string) {
    let scrollPosition = this.scrollArea.nativeElement.scrollLeft;

    if (direction === 'left') {
      scrollPosition = scrollPosition - this.iScrollOffset;
    } else if (direction === 'right') {
      scrollPosition = scrollPosition + this.iScrollOffset;
    }

    this.scrollArea.nativeElement.scrollTo(scrollPosition, 0);
  }
}