import { Component, EventEmitter, Input, OnChanges, OnInit, Output } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { VitisCheck, vitisAlert, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { LayerSymbologyService } from 'src/modules/vmap/components/layer-symbology/services/layer-symbology/layer-symbology.service';
import { JsonSymbologyClass } from 'src/modules/vmap/components/models/json-symbology/json-symbology-class.model';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';
import { JsonSymbologyStyle } from 'src/modules/vmap/components/models/json-symbology/json-symbology-style.model';

@Component({
  selector: 'app-layer-symbology-class',
  templateUrl: './layer-symbology-class.component.html',
  styleUrls: ['./layer-symbology-class.component.scss']
})
export class LayerSymbologyClassComponent implements OnInit, OnChanges {

  constructor(
    public vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    private translate: TranslateService,
    public symbologyService: LayerSymbologyService,
  ) {
    this.formatExpression = this.symbologyService.expressionTaductionArrToString.bind(this);
  }

  @Input() oDefinitionSymJson;
  @Input() oLayer: VmapLayer;
  @Input() columnsAvancedInput;
  @Input() isClassOpen;
  @Input() multipleClass; // Index de la classe
  @Input() headerTitle = true;
  @Input() currGeomType: string;
  @Input() isSymbolscaledenom: boolean;

  public bClassOpen: boolean = true;

  public legendName: string;
  public expression: string;
  public columns: Array<any>;

  public isMultipleClass: boolean;

  @Output() deleteVal = new EventEmitter<any>();
  @Output() updateEdit = new EventEmitter<any>();
  @Output() cloneClassEvt = new EventEmitter<any>();
  @Output() moveUpClassEvt = new EventEmitter<any>();
  @Output() moveDownClassEvt = new EventEmitter<any>();
  @Output() isValidField = new EventEmitter<any>();
  public formatExpression: Function;

  isObject = (value): boolean => typeof value == 'object';

  ngOnInit(): void {
    if (!this.isClassOpen) {
      this.bClassOpen = this.isClassOpen;
    }

    this.isMultipleClass = this.vitisCheck.isDefAndNotNull(this.multipleClass);
  }

  ngOnChanges(changes) {
    if (changes.oDefinitionSymJson && changes.oDefinitionSymJson.currentValue) {
      this.legendName = changes.oDefinitionSymJson.currentValue.legendName;
      this.expression = changes.oDefinitionSymJson.currentValue.expression;
      if (this.vitisCheck.isArray(this.expression)) {
        this.expression = this.symbologyService.expressionTaductionArrToString(this.expression);
      }
    }
    if (changes.columnsAvancedInput && changes.columnsAvancedInput.currentValue) {
      this.columns = [...changes.columnsAvancedInput.currentValue];
    }
  }

  /**
   * Suppression d'une classe
   * @param e
   */
  public deleteClass = () => {

    this.translate.get([
      'COMPONENT_MAP_DELETE_SYMBOLOGY_CLASS',
      'FORM_YES',
      'FORM_NO',
    ]).subscribe((translation) => {
      vitisAlert.alert({
        title: `${translation['COMPONENT_MAP_DELETE_SYMBOLOGY_CLASS']} "${this.legendName}" ?`,
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#3085d6',
        cancelButtonColor: '#d33',
        confirmButtonText: translation['FORM_YES'],
        cancelButtonText: translation['FORM_NO']
      }).then((result: any) => {
        if (result.value) {
          this.deleteVal.next({ deleteClass: true, indexClass: this.multipleClass });
        }
      });
    });
  }

  /**
   * Suppression d'une symbologie
   * @param e
   */
  public deleteSym = (e: any) => {
    this.deleteVal.next({ e, indexClass: this.multipleClass });
  }

  /**
   * Édition d'une classe
   * @returns
   */
  public edit = () => this.updateEdit.next({ indexClass: this.multipleClass, class: new JsonSymbologyClass(this.oDefinitionSymJson, 'expression') });

  /**
   * Clonne une classe
   * @param e
   */
  public cloneClass(e: any) {
    this.cloneClassEvt.next({ cloneClass: true, indexClass: this.multipleClass });
  }

  /**
   * Remonce une classe
   * @param e
   */
  public moveUpClass(e: any) {
    this.moveUpClassEvt.next({ moveUpClass: true, indexClass: this.multipleClass });
  }

  /**
   * Descend une classe
   * @param e
   */
  public moveDownClass(e: any) {
    this.moveDownClassEvt.next({ moveDownClass: true, indexClass: this.multipleClass });
  }

  /**
   * AJoute un style au table odefsymjson.styles
   * @param style type stype à ajouter
   */
  public addStyle() {
    let id = 1;
    for (const key in this.oDefinitionSymJson.styles) {
      if (Object.prototype.hasOwnProperty.call(this.oDefinitionSymJson.styles, key)) {
        const oStyle = this.oDefinitionSymJson.styles[key];
        if (oStyle.id > id) {
          id = oStyle.id;
        }
      }
    }
    const obj = this.symbologyService.setDefaultValue(this.currGeomType);
    const jsonSymbology = new JsonSymbologyStyle({...{ id: id + 1, hasStyle: true}, ...obj});
    this.oDefinitionSymJson.styles = [...this.oDefinitionSymJson.styles, jsonSymbology];
  }

  /**
   * Supprime le style dans l'object odefsymjson.styles
   * @param e position à supprimer
   */
  public trash = (e): void => {
    this.deleteVal.next({ e: this.getAttribut(e), indexClass: this.multipleClass });
  }

  /**
   * Clone le style dans l'object odefsymjson.styles
   * @param e objet à cloner
   */
  public cloneStyle = (e): void => {
    const id = this.oDefinitionSymJson.styles.length + 1;
    const jsonSymbology = this.vitisClone.clone(e);
    jsonSymbology.id = id;
    this.oDefinitionSymJson.styles = [...this.oDefinitionSymJson.styles, jsonSymbology];
  }

  /**
   * Déplace le style vers le haut dans l'object odefsymjson.styles
   * @param index index de l'objet à déplacer
   */
  public moveUpStyle = (index): void => {
    if (index > 0) {
      this.oDefinitionSymJson.styles.splice(index - 1, 0, this.oDefinitionSymJson.styles.splice(index, 1)[0]);
    }
  }

  /**
   * Déplace le style vers le bas dans l'object odefsymjson.styles
   * @param index index de l'objet à déplacer
   */
  public moveDownStyle = (index): void => {
    if (index < this.oDefinitionSymJson.styles.length) {
    this.oDefinitionSymJson.styles.splice(index + 1, 0, this.oDefinitionSymJson.styles.splice(index, 1)[0]);
    }
  }

  /**
   * Récupère le chemin du style dans l'objet
   * @param indexStyle
   */
  public getAttribut = (indexStyle) => {
    return this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJson.id) ?
      `class.${this.oDefinitionSymJson.id}.styles.${indexStyle}` : `styles.${indexStyle}`;
  }
}
