import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { LayerSymbologyService } from 'src/modules/vmap/components/layer-symbology/services/layer-symbology/layer-symbology.service';
import { JsonSymbologyStyle, JsonSymbologyStylesLineDash, JsonSymbologyStylesLineSymbol, JsonSymbologyStylesSymbolCenter, JsonSymbologyStylesSymbolRepeat, StyleSubFields } from 'src/modules/vmap/components/models/json-symbology/json-symbology-style.model';
import { JsonSymbologyStylesOpacity, JsonSymbologyStylesSymbolOffset, JsonSymbologyStylesSymbolOutline, JsonSymbologyStylesSymbolSizeVariation, JsonSymbologyStylesVisibility } from '../../../models/json-symbology/json-symbology-style.model';

@Component({
  selector: 'app-layer-symbology-style',
  templateUrl: './layer-symbology-style.component.html',
  styleUrls: ['./layer-symbology-style.component.scss']
})
export class LayerSymbologyStyleComponent implements OnInit {

  @Input() oStyle: JsonSymbologyStyle;
  @Input() index: number;

  @Output() isValidField = new EventEmitter<any>();
  @Output() trash = new EventEmitter<any>();
  @Output() cloneStyle = new EventEmitter<any>();
  @Output() moveUpStyle = new EventEmitter<any>();
  @Output() moveDownStyle = new EventEmitter<any>();
  @Input() isSymbolscaledenom: boolean;
  @Input() currGeomType: string;
  @Input() bOpen: boolean = false;

  public oStyleType: any;
  public keysStyleType: any;
  public aReadOnlyList: Array<string> = [];
  public oIcons = this.service.getIcons();

  public isOpenDropDown: boolean = false;
  public isOpenDropDownAction: boolean = false;

  public sAttribut: string;

  get symbols() {
    return this.service.symbols['_value'];
  }

  constructor(
    public service: LayerSymbologyService,
    private vitisCheck: VitisCheck,
    private translate: TranslateService,
    ) { }

  ngOnInit(): void {
    this.oStyleType = this.service.getLabelsList(this.currGeomType);
    this.keysStyleType = Object.keys(this.oStyleType);
    this.sAttribut = 'styles.' + this.index;
  }

  /**
   * Ouvre la liste des styles pour l'ajout
   */
  public openAddSubStyle() {
    document.getElementById('stylesDropdown' + this.oStyle.id).classList.toggle('show');
    // Filtre la liste par rapport à des conditions
    this.aReadOnlyList = Object.keys(this.oStyleType).filter((key) => {
      const isPolygonSymbol = (this.currGeomType == 'polygon' && !(this.oStyle.otherFields['polygonSymbolCenter'] || this.oStyle.otherFields['polygonSymbolRepeat']));
      const isLineSymbol = (this.currGeomType == 'line' && !(this.oStyle.otherFields['lineSymbol']));
      return this.service.isReadOnly(key, {
        'symbolSizeVariation': ((this.isSymbolscaledenom !== true) || isPolygonSymbol || isLineSymbol),
        'polygonSymbolRepeat': this.oStyle.otherFields['polygonSymbolCenter'],
        'polygonSymbolCenter': this.oStyle.otherFields['polygonSymbolRepeat'],
        'symbolOffset': (isPolygonSymbol || isLineSymbol),
        'lineDash': !isLineSymbol
      });
    });
    this.isOpenDropDown = !this.isOpenDropDown;
  }

  /**
   * Ouvre la liste actions possibles
   */
  public openStyleActions() {
    document.getElementById('stylesDropdownActions' + this.oStyle.id).classList.toggle('show');
    this.isOpenDropDownAction = !this.isOpenDropDownAction;
  }

  /**
   * Ferme la liste des styles pour l'ajout
   */
  public closeAddSubStyle() {
    document.getElementById('stylesDropdown' + this.oStyle.id).classList.remove('show');
    this.isOpenDropDown = false;
  }

  /**
   * Ferme la liste actions possibles
   */
  public closeStyleActions() {
    document.getElementById('stylesDropdownActions' + this.oStyle.id).classList.remove('show');
    this.isOpenDropDownAction = false;
  }

  /**
   * Active un élément du style
   * @param key Element à activé
   */
  public addSubStyle(key: string) {
    let jsonSymbology;
    if (this.currGeomType == 'polygon' && (key == 'polygonSymbolRepeat' || key == 'polygonSymbolCenter')) {
      this.localAlertWarning(key, 'COMPONENT_MAP_FORM_ASK_ADD_POLYGON_SYMBOL_REPEAT_OR_CENTER');
    } else if (this.currGeomType == 'line' && key == 'lineSymbol') {
      this.localAlertWarning(key, 'COMPONENT_MAP_FORM_ASK_ADD_LINE_SYMBOL');
    } else {
      switch (key) {
        case 'opacity':
          jsonSymbology = { 'opacity': new JsonSymbologyStylesOpacity() };
          break;
        case 'visibility':
          jsonSymbology = { 'visibility': new JsonSymbologyStylesVisibility() };
          break;
        case 'symbolOutline':
          jsonSymbology = { 'symbolOutline': new JsonSymbologyStylesSymbolOutline() };
          break;
        case 'symbolOffset':
          jsonSymbology = { 'symbolOffset': new JsonSymbologyStylesSymbolOffset() };
          break;
        case 'symbolSizeVariation':
          jsonSymbology = { 'symbolSizeVariation': new JsonSymbologyStylesSymbolSizeVariation() };
          break;
        case 'lineDash':
          jsonSymbology = { 'lineDash': new JsonSymbologyStylesLineDash() };
          break;
      }

      document.getElementById('stylesDropdown' + this.oStyle.id).classList.remove('show');
      this.oStyle.otherFields = { ...this.oStyle.otherFields, ...jsonSymbology };
    }
  }

  /**
   * Set la valeur dans otherFields en fonction du chemin dans event
   */
  public setJsonSymboValue(event: StyleSubFields) {
    this.oStyle = this.service.setOtherFieldValue(event, this.oStyle);
  }

  /**
   * Suppression de la valeur
   * @param event
   */
  public deleteVal(event) {
    this.translate.get([
      this.oStyleType[event],
      'COMPONENT_MAP_FORM_ASK_DELETE',
      'FORM_YES',
      'FORM_NO',
    ]).subscribe((translation) => {
      vitisAlert.alert({
        title: `${translation['COMPONENT_MAP_FORM_ASK_DELETE']} "${translation[this.oStyleType[event]]}" ?`,
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#3085d6',
        cancelButtonColor: '#d33',
        confirmButtonText: translation['FORM_YES'],
        cancelButtonText: translation['FORM_NO']
      }).then((result: any) => {
        if (result.value) {
          for (const key in this.oStyle.otherFields) {
            if (Object.prototype.hasOwnProperty.call(this.oStyle.otherFields, key)) {
              if (key === event) {
                delete this.oStyle.otherFields[key];
              }
              // Si la demande de suppression concerne Symbole au centroïde de chaque polygone
              // ou Symbole répété dans tout le polygone ou symbole sur une ligne
              // On supprime aussi les clés symbolOffset et symbolSizeVariation
              if ((event === 'polygonSymbolRepeat' || event == 'polygonSymbolCenter' || event == 'lineSymbol')
                && (key === 'symbolOffset' || key == 'symbolSizeVariation')) {
                delete this.oStyle.otherFields[key];
              }
            }
          }
        }
      });
    });
  }

  /**
   * Affiche la modale d'avertissement
   * @param key
   * @param sText
   */
  private localAlertWarning(key: string, sText: string) {
    let jsonSymbology;
    this.translate.get([
      this.oStyleType[key],
      sText,
      'FORM_YES',
      'FORM_NO',
    ], { name_section: this.translate.instant(this.oStyleType[key]) }).subscribe((translation) => {
      vitisAlert.alert({
        title: translation[this.oStyleType[key]],
        html: translation[sText],
        width: 800,
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#3085d6',
        cancelButtonColor: '#d33',
        confirmButtonText: translation['FORM_YES'],
        cancelButtonText: translation['FORM_NO']
      }).then((result: any) => {
        if (result.value) {
          switch (key) {
            case 'polygonSymbolRepeat':
              jsonSymbology = { 'polygonSymbolRepeat': new JsonSymbologyStylesSymbolRepeat };
              break;
            case 'polygonSymbolCenter':
              jsonSymbology = { 'polygonSymbolCenter': new JsonSymbologyStylesSymbolCenter };
              break;
            case 'lineSymbol':
              delete this.oStyle.otherFields.lineDash;
              jsonSymbology = { 'lineSymbol': new JsonSymbologyStylesLineSymbol };
              break;
          }

          document.getElementById('stylesDropdown' + this.oStyle.id).classList.remove('show');
          this.oStyle.otherFields = { ...this.oStyle.otherFields, ...jsonSymbology };
        }
      });
    });
  }
}
