/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

    const aVmapColumnTypeList = [{
            key: 'string',
            label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_TEXT_LABEL'
        },
        {
            key: 'number',
            label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_NUMBER_LABEL'
        },
        {
            key: 'bool',
            label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_BOOLEAN_LABEL'
        },
        // {
        //     key: 'image',
        //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_IMAGE_LABEL'
        // },
        // {
        //     key: 'document',
        //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_DOCUMENT_LABEL'
        // },
        // {
        //     key: 'link',
        //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_LINK_LABEL'
        // },
        // {
        //     key: 'date',
        //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_DATE_LABEL'
        // }
    ];

    scope.initFormLayerPG = function () {
        // Gère le message pour les colonnes du requeteur
        const oJsonColumnsField = scope.getFormField('json_columns');
        if (services.vitisCheck.isDefAndNotNull(oJsonColumnsField) && services.vitisCheck.isDefAndNotNull(oJsonColumnsField.oninit)) {
            oJsonColumnsField.oninit.subscribe(() => {
                setTimeout(() => {
                    setJsonColumnsMessage();
                }, 500);
            });
        }
    }

    scope.openJsonColumnsModal = function () {
        let sForm = '';
        let oFormValues = scope.oFormValues.value;

        const params = {
            database: oFormValues.database,
            schema: oFormValues.schema,
            table: oFormValues.pg_table,
            datasource_id: oFormValues.datasource_id
        };

        // Si une table specifique est utilisé pour le requeteur
        if (oFormValues.is_specific_table_requestor) {
            if (!services.vitisCheck.isDefAndNotNull(oFormValues.schema_requestor) ||
                oFormValues.schema_requestor == '' ||
                !services.vitisCheck.isDefAndNotNull(oFormValues.table_requestor) ||
                oFormValues.table_requestor == ''
            ) {
                services.translate.get('PG_LAYER_JSON_COLUMNS_MISSING_PARAMS').subscribe(async (translation) => {
                    services.vitisAlert.alert({
                        text: translation,
                        showCancelButton: true
                    });
                });
                return;
            } else {
                params.schema = oFormValues.schema_requestor;
                params.table = oFormValues.table_requestor;
            }
        }

        services.vitisRequest.ajaxRequest({
            method: 'GET',
            url: `${env.properties.app_api_url}/vmap/pglayers/columns`,
            params: params,
            success: async (resp) => {
                if (services.vitisCheck.isDefAndNotNull(resp.data)) {
                    const oResult = resp.data;
                    // $(document).off('click', '#table_field_list_selectAll');
                    if (oResult.status == 1 && oResult.data && oResult.data.length >= 0) {
                        // Déclaration function pour select all
                        // $(document).on('click', '#table_field_list_selectAll', function () {
                        //     if (document.getElementById("table_field_list_selectAll").checked === true) {
                        //         for (let i = 0; i < oResult.data.length; i++) {
                        //             if ((document.getElementById('table_field_list_' + oResult.data[i].column_name))) {
                        //                 (document.getElementById('table_field_list_' + oResult.data[i].column_name)).checked = true;
                        //             } else {
                        //                 console.error('Erreur dans le chargement de la checbox: ', oResult.data[i].column_name);
                        //             }
                        //         }
                        //     } else {
                        //         for (let i = 0; i < oResult.data.length; i++) {
                        //             if ((document.getElementById('table_field_list_' + oResult.data[i].column_name))) {
                        //                 (document.getElementById('table_field_list_' + oResult.data[i].column_name)).checked = false;
                        //             } else {
                        //                 console.error('Erreur dans le chargement de la checbox: ', oResult.data[i].column_name);
                        //             }
                        //         }
                        //     }
                        // });

                        let disabled = '';
                        if (scope.sFormDefinitionName == 'display') {
                            disabled = 'disabled';
                        }

                        const needTranslates = [
                            'DATASOURCE_FORM_PG_LAYER_JSON_COLUMNS_MODAL_TITLE',
                            'PG_LAYER_JSON_COLUMNS_LABEL',
                            'PG_LAYER_JSON_COLUMNS_ALL_LIST',
                            'CONFIRM_CONFIRM',
                            'CONFIRM_CANCEL',
                            'STUDIO_COMPONENT_LABEL_PLACEHOLDER',
                            'PG_LAYER_JSON_COLUMNS_INDEX',
                            'PG_LAYER_JSON_COLUMNS_TYPE'
                        ];
                        for (let i = 0; i < aVmapColumnTypeList.length; i++) {
                            needTranslates.push(aVmapColumnTypeList[i].label);
                        }

                        services.translate.get(needTranslates).subscribe(async (translates) => {
                            // Préparation du contenu de la modale
                            let sForm = `
                                <div style="text-align: left; height: 100%">
                                    <!--Sélectionner tous-->
                                    <!--
                                    <div class="row">
                                        <div class="checkbox checkbox-info margin-2 col-12" style="position: relative">
                                            <input id="table_field_list_selectAll" type="checkbox" style="cursor:pointer; position: absolute;" ${disabled} />
                                            <label for="table_field_list_selectAll" style="cursor:pointer"> ${translates['PG_LAYER_JSON_COLUMNS_ALL_LIST']} </label>
                                        </div>
                                    </div>
                                    -->
                                    <div class="row">
                                        <div class="mb-2 col-2" style="position: relative">
                                        </div>
                                        <div class="mb-2 col-1" style="position: relative; padding: 0px; text-align: center;">
                                        Requêteur
                                        </div>
                                        <div class="mb-2 col-1" style="position: relative; padding: 0px; text-align: center;">
                                        Filtre
                                        </div>
                                        <div class="mb-2 col-2" style="position: relative">
                                        </div>
                                        <div class="mb-2 col-3" style="position: relative">
                                        </div>
                                        <div class="mb-2 col-2" style="position: relative">
                                        </div>
                                    </div>
                                    <!--Liste-->
                                    <div class="scrollbar-container">
                            `;

                            for (let i = 0; i < oResult.data.length; i++) {
                                const column = oResult.data[i];
                                let index = -1;
                                let json_columns = [];
                                if (oFormValues.json_columns && oFormValues.json_columns.length > 0) {
                                    json_columns = [...JSON.parse(oFormValues.json_columns)];
                                    for (let j = 0; j < json_columns.length; j++) {
                                        if (json_columns[j].column == column.column_name) {
                                            index = j;
                                        }
                                    }
                                }

                                sForm += `
                                    <div class="row">
                                        <!-- Nom colonne -->
                                        <div class="checkbox checkbox-info mb-2 col-2" style="position: relative; text-align: right;">
                                            <label> ${column.column_name} </label>
                                        </div>
                                        <!-- Checkbox requêteur -->
                                        <div class="checkbox checkbox-info mb-2 col-1" style="position: relative; text-align: center;">
                                            <input id="table_field_list_${column.column_name}" 
                                                type="checkbox" 
                                                style="cursor:pointer; position: absolute"
                                                ${((index !== -1) ? json_columns[index].requestor === true ? 'checked' : '' : '')}
                                                ${disabled}>
                                            <label for="table_field_list_${column.column_name}"></label>
                                        </div>
                                        <!-- Checkbox filtre -->
                                        <div class="checkbox checkbox-info mb-2 col-1" style="position: relative; text-align: center;">
                                            <input id="table_field_filter_${column.column_name}" 
                                                type="checkbox" 
                                                style="cursor:pointer; position: absolute"
                                                ${((index !== -1) ? json_columns[index].filter === true ? 'checked' : '' : '')}
                                                ${disabled}>
                                            <label for="table_field_filter_${column.column_name}"></label>
                                        </div>
                                        <!-- Label -->
                                        <div class="input-group input-group-sm col-3 mb-2">
                                            <div class="input-group-prepend">
                                                <label class="input-group-text" id="table_field_label_label"> ${translates['PG_LAYER_JSON_COLUMNS_LABEL']} </label>
                                            </div>
                                            <input id="table_field_label_input_${column.column_name}" 
                                                type="text" 
                                                class="form-control" 
                                                placeholder="${translates['STUDIO_COMPONENT_LABEL_PLACEHOLDER']}" 
                                                required="" 
                                                value="${((index !== -1) ? json_columns[index].name : column.column_name)}"
                                                ${disabled}>
                                        </div>
                                        <!-- Type -->
                                        <div class="input-group input-group-sm col-3 mb-2">
                                            <div class="input-group-prepend">
                                                <label class="input-group-text" id="table_field_index_label"> ${translates['PG_LAYER_JSON_COLUMNS_TYPE']} </label>
                                            </div>
                                            <select class="custom-select custom-select-sm"
                                                id="table_field_type_input_${column.column_name}"
                                                ${disabled}>`;

                                // Contenu du select
                                for (let k = 0; k < aVmapColumnTypeList.length; k++) {
                                    const type = aVmapColumnTypeList[k];
                                    if (index !== -1 && json_columns[index].type == type.key) {
                                        sForm += '<option value="' + type.key + '" selected>' + translates[type.label] + '</option>';
                                    } else {
                                        sForm += '<option value="' + type.key + '">' + translates[type.label] + '</option>';
                                    }
                                }

                                sForm += `  </select>
                                        </div>
                                        <!-- Index -->
                                        <div class="input-group input-group-sm col-2 mb-2">
                                            <div class="input-group-prepend">
                                                <label class="input-group-text" id="table_field_index_label"> ${translates['PG_LAYER_JSON_COLUMNS_INDEX']} </label>
                                            </div>
                                            <input id="table_field_index_input_${column.column_name}" 
                                                type="text" 
                                                class="form-control" 
                                                required=""
                                                value="${((index !== -1) ? json_columns[index].index : i)}"
                                                ${disabled}>
                                        </div>
                                    </div>`;
                            }


                            sForm += `</div>
                                </div>`;

                            // Si + de 13 éléments on ajoute le style pour la scrollbar
                            if (oResult.data.length > 13) {
                                sForm += '<style>';
                                sForm += '.swal2-popup.swal2-modal.vitis-show { height: 80% }';
                                sForm += '.swal2-popup.swal2-modal.vitis-show .swal2-content { height: 85% }';
                                sForm += '.swal2-popup.swal2-modal.vitis-show .swal2-content .swal2-html-container { height: 100% }';
                                sForm += '.scrollbar-container { overflow-x: scroll; height: 95% }';
                                sForm += '.scrollbar-container .row { width: 101% }';
                                sForm += '</style>';
                            }

                            // Ouverture de la modale
                            const {
                                value: aAlertFormValues
                            } = await services.vitisAlert.alert({
                                title: translates['DATASOURCE_FORM_PG_LAYER_JSON_COLUMNS_MODAL_TITLE'],
                                width: 1200,
                                html: sForm,
                                focusConfirm: false,
                                showCancelButton: true,
                                confirmButtonText: translates['CONFIRM_CONFIRM'],
                                cancelButtonText: translates['CONFIRM_CANCEL'],
                                preConfirm: () => {
                                    const preConfirm = [];
                                    for (let i = 0; i < oResult.data.length; i++) {
                                        preConfirm.push({
                                            column: oResult.data[i],
                                            requestor: (document.getElementById('table_field_list_' + oResult.data[i].column_name)).checked,
                                            filter: (document.getElementById('table_field_filter_' + oResult.data[i].column_name)).checked,
                                            value: (document.getElementById('table_field_label_input_' + oResult.data[i].column_name)).value,
                                            index: (document.getElementById('table_field_index_input_' + oResult.data[i].column_name)).value,
                                            type: (document.getElementById('table_field_type_input_' + oResult.data[i].column_name)).value,
                                        });
                                    }
                                    return preConfirm;
                                }
                            });

                            // Résultat de la modal
                            if (aAlertFormValues) {
                                const json_columns = [];
                                for (let i = 0; i < aAlertFormValues.length; i++) {
                                    const data = aAlertFormValues[i];
                                    json_columns.push({
                                        requestor: data.requestor,
                                        filter: data.filter,
                                        column: data.column.column_name,
                                        name: data.value,
                                        index: data.index,
                                        type: data.type
                                    });
                                }

                                scope.oFormValues.patchValue({
                                    json_columns: JSON.stringify(json_columns)
                                });

                                // Vérifie que json_columns à une valeur sinon affiche un message
                                setJsonColumnsMessage();
                            }
                        });
                    } else {
                        console.error('Impossible de récupérer les champs de la table');
                        services.translate.get('PG_LAYER_JSON_COLUMNS_RESPONSE_NO_DATA').subscribe(async (translation) => {
                            services.vitisAlert.alert({
                                text: translation,
                                showCancelButton: true
                            });
                        });
                    }
                } else {
                    console.error('Impossible de récupérer les champs de la table - resp', resp);
                }
            }
        });
    }

    const setJsonColumnsMessage = () => {
        // Vérifie que json_columns à une valeur sinon affiche un message
        if (!scope.oFormValues.value.json_columns || scope.oFormValues.value.json_columns == '' || scope.oFormValues.value.json_columns == '[]') {
            services.translate.get('DATASOURCE_FORM_PG_LAYER_JSON_COLUMNS_MESSAGE_MISSING').subscribe((translate) => {
                scope.oFormValues.patchValue({
                    info_json_columns_message: translate
                });
            });
        } else {
            scope.oFormValues.patchValue({
                info_json_columns_message: ''
            });
        }
    }

    /**
     * Chargement des infos WMS
     */
    scope.wmsPrepareData = function () {
        setTimeout(() => {
            const oDatasource = scope.getFormField('datasource').getValue();

            if (oDatasource &&
                oDatasource.flux_url &&
                oDatasource.flux_version
            ) {
                scope.wmsGetCapabilities(oDatasource).then((oCapabilities) => {
                    if (oCapabilities &&
                        oCapabilities.json &&
                        oCapabilities.json.Capability &&
                        oCapabilities.json.Capability.Layer) {

                        const oLayerOptionsField = scope.getFormField('layer_options');
                        oLayerOptionsField.setValue(JSON.stringify(oCapabilities.json));

                        // Remplit les listes déroulantes avec le contenu du getCapabilities
                        scope.wmsFillSelectFields(oCapabilities.json);

                    } else {
                        console.error('Capabilities not valid', oCapabilities)
                    }
                })
            } else {
                console.error('datasource not valid', oDatasource);
            }
            // Nom et url de la source.
            scope.setSourceFormField(oDatasource, 'wms');
        }, 500);
    }

    /**
     * Recharge les listes déroulantes lorsque l'utilisateur change de couche
     */
    scope.wmsLayerNameChange = function () {
        const oLayerNameVal = scope.getFormField('layer_name').getValue();
        const oLayerOptionsVal = scope.getFormField('layer_options').getValue();

        if (this.vitisCheck.isDefAndNotNull(oLayerNameVal) && oLayerNameVal != 'undefined') {
            let oJSONCapabilities;
            try {
                oJSONCapabilities = JSON.parse(oLayerOptionsVal)
            } catch (error) {}

            if (oJSONCapabilities) {
                // Ajout setTimeout pour que le champ du formulaire ai eu le temps de changer de valeurs
                setTimeout(() => scope.wmsFillSelectFields(oJSONCapabilities), 100);
            }
        }
    }

    /**
     * Remplit les listes déroulantes avec le contenu du getCapabilities
     * @param {*} oJSONCapabilities 
     */
    scope.wmsFillSelectFields = function (oJSONCapabilities) {
        const oLayerNameField = scope.getFormField('layer_name');
        const aLayers = services.WmsService.searchLayers(oJSONCapabilities.Capability.Layer);

        // Remplit la liste des couches
        services.AdminDatasourceFormService.prepareSelectForm(aLayers, oLayerNameField, ['Title'], 'Name');

        // Couche sélectionnée
        const oSelectedLayer = aLayers.find((oLayer) => oLayer.Name === oLayerNameField.getValue());

        if (this.vitisCheck.isDefAndNotNull(oSelectedLayer)) {
            // Remplit la liste des styles
            if (oSelectedLayer.Style) {
                services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Style, scope.getFormField('layer_style'), ['Title'], 'Name');
            }

            // Remplit la liste des formats
            if (oSelectedLayer.Format) {
                services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Format, scope.getFormField('layer_format'));
            }

            // Remplit la liste des CRS
            if (oSelectedLayer.CRS || oSelectedLayer.SRS) {
                const aAvailableCoordsys = oSelectedLayer.CRS ? oSelectedLayer.CRS : oSelectedLayer.SRS
                scope.getFormField('crs_list').setValue(aAvailableCoordsys.join('|'));
            }
        } else {
            console.error('La couche ' + oLayerNameField.getValue() + ' n\'a pas été trouvée');
        }
    }

    /**
     * Effecture la requête getCapabilities
     * @param {*} oDatasource 
     * @returns 
     */
    scope.wmsGetCapabilities = function (oDatasource) {
        return new Promise((resolve, reject) => {
            services.WmsService.getAjaxCapabilitiesWMS({
                flux_url: oDatasource.flux_url,
                flux_version: oDatasource.flux_version,
                flux_login: oDatasource.flux_login,
                flux_password: oDatasource.flux_password
            }).then((oCapabilities) => {
                if (oCapabilities !== false) {
                    resolve(oCapabilities);
                } else {
                    services.translate.get('CANT_REFRESH_CAPABILITIES').subscribe((sTranslate) => {
                        vitisAlert.fireToast({
                            type: 'warning',
                            title: sTranslate
                        });
                    });
                    reject();
                }
            });

        });
    }

    /**
     * Chargement des infos WMTS
     */
    scope.wmtsPrepareData = function () {
        setTimeout(() => {
            const oDatasource = scope.getFormField('datasource').getValue();

            if (oDatasource &&
                oDatasource.flux_url &&
                oDatasource.flux_version
            ) {
                scope.wmtsGetCapabilities(oDatasource).then((oCapabilities) => {
                    if (oCapabilities &&
                        oCapabilities.Contents &&
                        oCapabilities.Contents.Layer) {

                        const oLayerOptionsField = scope.getFormField('layer_options');
                        oLayerOptionsField.setValue(JSON.stringify(oCapabilities));

                        // Remplit les listes déroulantes avec le contenu du getCapabilities
                        scope.wmtsFillSelectFields(oCapabilities);

                    } else {
                        console.error('Capabilities not valid', oCapabilities)
                    }
                })
            } else {
                console.error('datasource not valid', oDatasource);
            }
            // Nom et url de la source.
            scope.setSourceFormField(oDatasource, 'wmts');
        }, 500);
    }

    /**
     * Recharge les listes déroulantes lorsque l'utilisateur change de couche
     */
    scope.wmtsLayerNameChange = function () {
        const oLayerNameVal = scope.getFormField('layer_name').getValue();
        const oLayerOptionsVal = scope.getFormField('layer_options').getValue();

        if (this.vitisCheck.isDefAndNotNull(oLayerNameVal) && oLayerNameVal != 'undefined') {
            let oJSONCapabilities;
            try {
                oJSONCapabilities = JSON.parse(oLayerOptionsVal)
            } catch (error) {}

            if (oJSONCapabilities) {
                // Ajout setTimeout pour que le champ du formulaire ai eu le temps de changer de valeurs
                setTimeout(() => scope.wmtsFillSelectFields(oJSONCapabilities), 100);
            }
        }
    }

    /**
     * Remplit les listes déroulantes avec le contenu du getCapabilities
     * @param {*} oJSONCapabilities 
     */
    scope.wmtsFillSelectFields = function (oJSONCapabilities) {
        const oLayerNameField = scope.getFormField('layer_name');
        const aLayers = services.WmtsService.searchLayers(oJSONCapabilities.Contents.Layer);

        // Remplit la liste des couches
        services.AdminDatasourceFormService.prepareSelectForm(aLayers, oLayerNameField, ['Title'], 'Identifier');

        // Couche sélectionnée
        const oSelectedLayer = aLayers.find((oLayer) => oLayer.Identifier === oLayerNameField.getValue());

        if (this.vitisCheck.isDefAndNotNull(oSelectedLayer)) {
            // Remplit la liste des styles
            if (oSelectedLayer.Style) {
                services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Style, scope.getFormField('layer_style'), ['Title'], 'Identifier');
            }

            // Remplit la liste des formats
            if (oSelectedLayer.Format) {
                services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Format, scope.getFormField('layer_format'));
            }

            // Remplit la liste des matrices
            if (oSelectedLayer.TileMatrixSetLink) {
                services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.TileMatrixSetLink, scope.getFormField('matrix_set'), ['TileMatrixSet'], 'TileMatrixSet');
            }
        } else {
            console.error('La couche ' + oLayerNameField.getValue() + ' n\'a pas été trouvée');
        }
    }

    /**
     * Effecture la requête getCapabilities
     * @param {*} oDatasource 
     * @returns 
     */
    scope.wmtsGetCapabilities = function (oDatasource) {
        return new Promise((resolve, reject) => {
            services.WmtsService.getAjaxCapabilitiesWMTS({
                flux_type: oDatasource.flux_type,
                flux_url: oDatasource.flux_url,
                flux_version: oDatasource.flux_version,
                flux_login: oDatasource.flux_login,
                flux_password: oDatasource.flux_password
            }).then((oCapabilities) => {
                if (oCapabilities !== false) {
                    resolve(oCapabilities);
                } else {
                    services.translate.get('CANT_REFRESH_CAPABILITIES').subscribe((sTranslate) => {
                        vitisAlert.fireToast({
                            type: 'warning',
                            title: sTranslate
                        });
                    });
                    reject();
                }
            });

        });
    }

    /**
     * Initialisation des champs de form. de la source.
     * @param {object} oDatasource 
     */
    scope.initFormLayerOsm = function (oFormValues) {
        // Nom et url de la source.
        scope.setSourceFormField(oFormValues.datasource, 'osm');
    }

    /**
     * Initialisation des champs de form. de la source.
     * @param {object} oDatasource 
     */
     scope.initFormLayerXyz = function (oFormValues) {
        // Nom et url de la source.
        scope.setSourceFormField(oFormValues.datasource, 'xyz');
    }
            
    /**
     * Initialisation des champs de form. de la source.
     * @param {object} oDatasource 
     * @param {string} sLayerType
     */
    scope.setSourceFormField = function (oDatasource, sLayerType) {
        // Nom et url de la source.
        if (this.vitisCheck.isDefAndNotNull(oDatasource)) {
            const oSourceUrl = scope.getFormField('source_url');
            const oSourceName = scope.getFormField('source_name');
            if (sLayerType == 'wms' || sLayerType == 'wmts') {
                oSourceUrl.setValue(oDatasource['flux_url']);
                oSourceName.setValue(oDatasource['name']);
            }
            else {
                var subscriptionSourceUrl = oSourceUrl.oninit.subscribe(() => {
                    oSourceUrl.setValue(oDatasource['flux_url']);
                    // Supprime la subscription à l'observable.
                    subscriptionSourceUrl.unsubscribe();
                });
                var subscriptionSourceName = oSourceName.oninit.subscribe(() => {
                    oSourceName.setValue(oDatasource['name']);
                    // Supprime la subscription à l'observable.
                    subscriptionSourceName.unsubscribe();
                });
            }
        }
    }
}