/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

    /**
     * Génère le code du widget
     */
    scope.generateCodeWidget = function () {
        if (scope.isFormValid()) {

            const sWidgetUrl = scope.getWidgetURL();
            if (this.vitisCheck.isDefAndNotNull(sWidgetUrl)) {

                // Set le lien vers le widget
                const oLinkField = scope.getFormField('url_widget');
                oLinkField.visible = true;
                setTimeout(() => {
                    oLinkField.setValue(sWidgetUrl);
                });

                // Set le champ monacoeditor
                const sHtml = scope.getWidgetIframe(sWidgetUrl);
                const oCodeField = scope.getFormField('html_code');
                oCodeField.visible = true;
                setTimeout(() => {
                    oCodeField.setValue(sHtml);
                });
            }
        }
    }

    /**
     * Retourne l'URL du widget
     */
    scope.getWidgetURL = function () {

        const sClientUrl = env.properties.app_client_url;
        const sLoginToken = scope.getFormField('logintoken').getValue();
        const sMapId = scope.getFormField('map_id').getValue();
        const sTitle = scope.getFormField('widget_title').getValue();
        const sLayerTool = scope.getFormField('layers_tool').getValue();

        if ((this.vitisCheck.isDefAndNotNull(sLoginToken) && sLoginToken !== '') &&
            (this.vitisCheck.isDefAndNotNull(sMapId) && sMapId !== '') &&
            (this.vitisCheck.isDefAndNotNull(sTitle) && sTitle !== '') &&
            (this.vitisCheck.isDefAndNotNull(sClientUrl)) && sClientUrl !== '') {
            let sUrl = `${sClientUrl}/widget/vmap?map_id=${sMapId}&token=${sLoginToken}&widget_title=${sTitle}`;
            if (sLayerTool) {
                sUrl += `&layers_tool=${sLayerTool}`
            }
            return sUrl;
        }
    }

    /**
     * Retourne le code HTML pour utiliser le widget dans une iframe
     * @param {*} sWidgetUrl 
     */
    scope.getWidgetIframe = function (sWidgetUrl) {

        const iHeight = scope.getFormField('iframe_height').getValue();
        const iWidth = scope.getFormField('iframe_width').getValue();
        const sDivId = scope.getFormField('div_id').getValue();
        const sTitle = scope.getFormField('widget_title').getValue();
        const sHtml = `<iframe id="${sDivId}" title="${sTitle}" width="${iWidth}" height="${iHeight}"
    style="border: 1px solid #39468A; border-radius: 5px 5px 0px 0px;"
    src = "${sWidgetUrl}" >
</iframe>`;

        return sHtml;
    }
}