/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

    /**
     * Initialise l'outil de gestion de l'étendue
     */
    scope.initOLMapExtent = function () {

        // Dernière fois qu'un carte a été chargée
        scope.lastMapLoad_ = Date.now();

        var lastMapChange = Date.now();
        var oMapFiled = services.oFormReader.getFormField('map_extent');

        /**
         * Change l'étendue de la carte
         * @param {*} sExtent 
         * @param {*} sCoordsys 
         * @param {*} olMap 
         */
        var setOlMapExtent = function (sExtent, sCoordsys, olMap) {
            if (sExtent) {
                var olExtent = sExtent.split('|').map(coord => Number(coord));
                var currMapProjection = olMap.getView().getProjection().getCode();
                var aNewExtent = oMapFiled.transformExtent(olExtent, `EPSG:${sCoordsys}`, currMapProjection);
                oMapFiled.setExtent(aNewExtent);
            }
        }

        /**
         * Calcule l'étendue de la carte, la reprojette et la place sur le champ texte
         */
        var setFormExtentFromMap = function (olMap) {
            var currMapExtent = olMap.getView().calculateExtent();
            var currMapProjection = olMap.getView().getProjection().getCode();
            var formProjection = 'EPSG:' + scope.oFormValues.get('coordsys_id').value;
            var aNewExtent = oMapFiled.transformExtent(currMapExtent, currMapProjection, formProjection);
            lastMapChange = Date.now();
            scope.oFormValues.patchValue({
                extent: aNewExtent.join('|')
            });
        }

        /**
         * Écoute les changements de la carte et met à jour le champ étendue
         */
        var listenMapChanges = function () {
            services.formreaderSrvc.mapChangeEventObservable.subscribe(olMap => {
                if (scope.lastMapLoad_ + 1000 < Date.now()) {
                    setFormExtentFromMap(olMap);
                }
            });
        }

        var firstLoad = true;
        services.formreaderSrvc.mapChangeEventObservable.subscribe(olMap => {
            if (firstLoad) {

                // Set l'étendue du composant carto
                setOlMapExtent(scope.oFormValues.get('extent').value, scope.oFormValues.get('coordsys_id').value, olMap);

                if (scope.mapListened_ !== true) {

                    // Écoute les changements du composant carto
                    setTimeout(() => {
                        listenMapChanges();
                    }, 500);

                    // Met à jour l'extent et la carte par rapport au champ coordsys_id
                    scope.oFormValues.controls['coordsys_id'].valueChanges.subscribe(value => {
                        setFormExtentFromMap(olMap);
                    });

                    // Met à jour l'extent de la carte par rapport au champ extent
                    scope.oFormValues.controls['extent'].valueChanges.subscribe(value => {
                        if (lastMapChange + 1000 < Date.now()) {
                            setOlMapExtent(scope.oFormValues.get('extent').value, scope.oFormValues.get('coordsys_id').value, olMap);
                        }
                    });
                }
            }
            firstLoad = false;
            scope.mapListened_ = true;
        });
    }
}