<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vitis\WebService\Service\Normalizer\VitisNormalizer;
use App\Module\Vmap\WebService\Entity\AbstractLayer;
use App\Module\Vmap\WebService\Entity\LayerMap;
use App\Module\Vmap\WebService\Entity\Map;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Serializer\Normalizer\AbstractNormalizer;

/**
 * Controller pour gérer les requêtes de la table layer_map
 *
 */
class LayerMapController extends GenericController {
  /**
   * Récupère la liste des jointure couche/carte et les metadatas
   * @return Response réponse à retourner par l'API
   */
  public function getLayersSourcesForMap($id): Response {
    // récupère les layers de map
    $oMapRepository = $this->oDatabaseConnection->getUserEntityManager()->getRepository(Map::class);

    if (!$oMapRepository instanceof GenericRepository) {
      return $this->formatGenericResponseForErrorOnSelect('bad repository');
    }

    $oMap = $oMapRepository->findById($id)[0];

    if (empty($oMap)) {
      return $this->formatGenericResponseForErrorOnSelect('cannot find map');
    }

    $aLayers = $oMap->getLayersSource(true);

    $aReturn = [
      'status' => 1,
      'total_row_number' => count($aLayers),
      'list_count' => count($aLayers),
      'data' => $aLayers,
    ];

    // transforme le tableau de réponse dans le résultat souhaité
    return $this->formatResponse($aReturn);
  }

  public function getLayersSourcesNotIncluded($id): Response {
    $aData = $this->requestToArray();
    $oLayersRepository = $this->oDatabaseConnection->getUserEntityManager()->getRepository(AbstractLayer::class);

    if (!$oLayersRepository instanceof GenericRepository) {
      return $this->formatGenericResponseForErrorOnSelect('bad repository');
    }

    $aLayers = $oLayersRepository->findAllNotJoinedToMap($id, $aData);

    $aSerializerContext = [];
    if (isset($aData['attributs'])) {
      $aSerializerContext[AbstractNormalizer::ATTRIBUTES] = explode('|', $aData['attributs']);
    }
    if (isset($aData['distinct'])) {
      $aSerializerContext[VitisNormalizer::DISTINCT] = ($aData['distinct'] === 'true');
    }

    return $this->formatResponse($aLayers, Response::HTTP_OK, $aSerializerContext);
  }


  /**
   * Récupère une couche d'un carte avec son $layer_map_id
   * @return Response réponse à retourner par l'API
   */
  public function getLayer($idMap, $id): Response {
    if (isset($idMap)) {
      // récupère les layers de map
      $oMap = $this->oDatabaseConnection->getUserEntityManager()->getReference(Map::class, $idMap);
      $aLayers = $oMap->getLayersSource();

      $aReturn = [
        'data' => [],
        'status' => 1,
      ];

      foreach ($aLayers as $oLayer) {
        if ($oLayer['layer_map_id'] == $id) {
          // Récupère la couche
          $aReturn['data'] = $oLayer;
        }
      }

      $this->formatResponse($aReturn, Response::HTTP_OK);
    } else {
      $this->formatResponse(['status' => 0], Response::HTTP_UNAUTHORIZED);
    }

    return $this->oResponse;
  }

  /**
   * Ajoute une liste de layersID dans la table layer_map
   * @return Response réponse à retourner par l'API
   */
  public function postDatas(): Response {
    $routeParameters = $this->getRequestParameters();
    $aData = $this->requestToArray();

    try {
      $aEntities = [];
      for ($i = 0; $i < count($aData['data']); $i++) {
        $sData = $aData['data'][$i];
        $layerMap = new LayerMap();
        $layerMap->setLayerId($sData['layer_id']);
        $layerMap->setLayer($this->oDatabaseConnection->getAdminEntityManager()->find(AbstractLayer::class, $sData['layer_id']));
        $layerMap->setMapId($sData['map_id']);
        $layerMap->setMap($this->oDatabaseConnection->getAdminEntityManager()->find(Map::class, $sData['map_id']));
        $layerMap->setIsVisible(true);
        if (array_key_exists('layer_index', $sData)) {
          $layerMap->setLayerIndex($sData['layer_index']);
        }

        $serialisation = $layerMap->jsonSerialize($routeParameters);
        $this->oDatabaseConnection->getAdminEntityManager()->persist($layerMap);

        $serialisation['layer_map_id'] = $layerMap->getLayerMapId();
        $aEntities[] = $serialisation;
      }
      $this->oDatabaseConnection->getAdminEntityManager()->flush();

      $aReturn = [
        'data' => $aEntities,
        'status' => 1,
      ];
      $this->formatResponse($aReturn, Response::HTTP_OK);
    } catch (\Exception $e) {
      $this->formatResponse(['status' => 0], Response::HTTP_UNAUTHORIZED);
    }
    return $this->oResponse;
  }

  /**
   * Modifie des entité LayerMap
   * @return Response réponse à retourner par l'API
   */
  public function updateDatas(): Response {
    $aData = $this->requestToArray();
    $oDataAccessor = $this->getDataAccessor();

    try {
      $aEntities = [];
      foreach ($aData['data'] as $data) {
        $sData = json_encode($data);
        $oEntity = $oDataAccessor->updateEntity($data['layer_map_id'], $sData);
        $aEntities[] = $oEntity;
      }

      $aReturn = [
        'data' => $aEntities,
        'status' => 1,
      ];
      $this->formatResponse($aReturn, Response::HTTP_OK);
    } catch (\Exception $e) {
      $this->formatResponse(['status' => 0], Response::HTTP_UNAUTHORIZED);
    }

    return $this->oResponse;
  }
}
