<?php

namespace App\Module\Vmap\WebService\Controller;

use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour le catalog de vMap.
 *
 */
class MapCatalogController extends DatasourceTreeviewController {
  /**
   * Permet de récupérer la catalogue de cartes de l'utilisateur au format suivant
   * {
   *    maps: []
   *    defaultMap: int|null
   * }
   * @return Response réponse à retourner par l'API
   */
  public function getDatas(): Response {
    $aData = $this->requestToArray();

    // Ordonancement par catalog_index
    $aData['order_by'] = 'catalog_index';

    // chargement de la connexion à la base de donnée
    $oDataAccessor = $this->getDataAccessor();
    $aEntities = $oDataAccessor->getEntities($aData);

    $aMaps = $aEntities['data'];
    $aDefaultData = $this->getDefaultMapFields();

    $aEntities['data'] = [
      'maps' => $aMaps,
      'defaultMap' => $aDefaultData['map_id'] ?? null,
      'defaultExtent' => $aDefaultData['extent'] ?? null,
    ];

    // transforme le tableau de réponse dans le résultat souhaité
    $this->formatResponse($aEntities, Response::HTTP_OK);

    return $this->oResponse;
  }

  /**
   * Fait le lien avec la table v_user_defaultmap
   * Trouve les données pour cet utilisateur
   * @return array|null
   */
  private function getDefaultMapFields(): ?array{
    $qb = $this->oDatabaseConnection->getQueryAccessor()->createQueryBuilder();
    $qb->select('*')
      ->from('[SCHEMA].v_user_defaultmap', 'a');
    $qb->setMaxResults(1);

    $aParams = [
      'SCHEMA' => ['value' => $this->oProperties->get('schema_vmap'), 'type' => 'schema_name'],
    ];

    try {
      $aDefaultMaps = $qb->executeWithParams($aParams)->fetchAll();
      if (!empty($aDefaultMaps[0])) {
        return $aDefaultMaps[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('v_user_defaultmap : can\'t get user default map with log_url: ' . $e->getMessage());
    }
    return null;
  }
}
