<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vmap\WebService\Entity\AbstractPublication;
use App\Module\Vmap\WebService\Entity\Flux;
use App\Module\Vmap\WebService\Entity\PublicationFlux;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour gérer les requêtes de la table publication_flux
 *
 */
class PublicationFluxController extends GenericController {
  /**
   * Récupère la liste des jointure couche/carte et les metadatas
   * @return Response réponse à retourner par l'API
   */
  public function getPublicationsFlux($id): Response {
    // récupère les publications du flux
    $oFluxRepository = $this->oDatabaseConnection->getUserEntityManager()->getRepository(Flux::class);

    if (!$oFluxRepository instanceof GenericRepository) {
      return $this->formatGenericResponseForErrorOnSelect('bad repository');
    }

    $oFlux = $oFluxRepository->findById($id)[0];

    if (empty($oFlux)) {
      return $this->formatGenericResponseForErrorOnSelect('cannot find flux');
    }

    $aPublications = $oFlux->getPublications();

    $aReturn = [
      'status' => 1,
      'total_row_number' => count($aPublications),
      'list_count' => count($aPublications),
      'data' => $aPublications,
    ];

    // transforme le tableau de réponse dans le résultat souhaité
    return $this->formatResponse($aReturn);
  }
  /**
   * Ajoute une liste de layersID dans la table layer_map
   * @return Response réponse à retourner par l'API
   */
  public function postDatas(): Response {
    $routeParameters = $this->getRequestParameters();
    $aData = $this->requestToArray();

    try {
      $aEntities = [];
      for ($i = 0; $i < count($aData['data']); $i++) {
        $sData = $aData['data'][$i];
        $publicationFlux = new PublicationFlux();
        $publicationFlux->setPublication($this->oDatabaseConnection->getAdminEntityManager()->find(AbstractPublication::class, $sData['publication_id']));
        $publicationFlux->setPublicationId($sData['publication_id']);
        $publicationFlux->setFlux($this->oDatabaseConnection->getAdminEntityManager()->find(Flux::class, $sData['flux_id']));
        $publicationFlux->setFluxId($sData['flux_id']);

        $serialisation = $publicationFlux->jsonSerialize($routeParameters);
        $this->oDatabaseConnection->getAdminEntityManager()->persist($publicationFlux);

        $serialisation['flux_layer_id'] = $publicationFlux->getFluxLayerId();
        $aEntities[] = $serialisation;
      }
      $this->oDatabaseConnection->getAdminEntityManager()->flush();

      $aReturn = [
        'data' => $aEntities,
        'status' => 1,
      ];
      $this->formatResponse($aReturn, Response::HTTP_OK);
    } catch (\Exception $e) {
      $this->formatResponse(['status' => 0], Response::HTTP_UNAUTHORIZED);
    }
    return $this->oResponse;
  }
}
