<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\Common\Persistence\ObjectManagerAware;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;
use App\Module\Vitis\WebService\Entity\AbstractEntity;

/**
 * Objet ORM pour fluxdebuglevel
 *
 * @author Laurie Ambroise <laurie.ambroise@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="fluxdebuglevel")
 */
class FluxDebugLevel extends AbstractEntity implements ObjectManagerAware {

  /**
   * @var EntityManager
   */
  private $oEntityManager;
  /**
   * @var LoggerManager
   */
  private $oLoggerManager;

  /**
   * @var int
   * @Column(type="integer", name="fluxdebuglevel_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).fluxdebuglevel_fluxdebuglevel_id_seq")
   */
  private $fluxdebuglevel_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $label;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\Flux>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\Flux", mappedBy="flux")
   */
  private $aFlux;

  public function __construct() {
    $this->aFlux = new ArrayCollection();
  }

  public function __toString(): ?string {
    return $this->getLabel();
  }

  /**
   * @return int identifiant du theme
   */
  public function getFluxDebugLevelId(): int {
    return $this->fluxdebuglevel_id;
  }

  /**
   * @return string|null label du theme
   */
  public function getLabel(): ?string {
    return $this->label;
  }

  /**
   * @param string $label label du theme
   * @return self
   */
  public function setLabel(?string $label): self {
    $this->label = $label;
    return $this;
  }

  /**
   * retourne la liste des couches postgres
   * @return array
   */
  public function getFlux(): array{
    return $this->aFlux->toArray();
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base (ObjectManagerAware)
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
