<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vmap\WebService\Service\MapServerService;
use Doctrine\Common\Persistence\ObjectManagerAware;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour layerpostgres
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="layerpostgres")
 */
class LayerPostgres extends AbstractLayer implements ObjectManagerAware {
  /**
   * @var string
   */
  public const TYPE = 'postgres';

  /**
   * @var int
   * @Id
   * @Column(type="integer", name="layer_id", nullable=false)
   */
  protected $layer_id;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $definition_sym_json;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $definition_sym_mapserver;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_sym_definition_json;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $database;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $schema;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $pg_table;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $idfield;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $geomfield;

  /**
   * @var Coordsys
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Coordsys", inversedBy="layers_postgres")
   * @JoinColumn(name="coordsys_id", referencedColumnName="coordsys_id")
   */
  private $coordsys;

  /**
   * @var int
   * @column(type="integer")
   */
  private $coordsys_id;

  /**
   * @var MsGeomType
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\MsGeomType", inversedBy="layers_postgres")
   * @JoinColumn(name="ms_geomtype_id", referencedColumnName="ms_geomtype_id")
   */
  private $ms_geomtype;

  /**
   * @var int
   * @column(type="integer")
   */
  private $ms_geomtype_id;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $json_columns;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_location;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $location_search_field;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $location_placeholder;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $location_result_field;

  /**
   * @var SearchMode
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\SearchMode", inversedBy="layers_postgres")
   * @JoinColumn(name="location_search_mode_id", referencedColumnName="searchmode_id")
   */
  private $location_search_mode;

  /**
   * @var int
   * @column(type="integer")
   */
  private $location_search_mode_id;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_map_selection;

  /**
   * @var float
   * @Column(type="float", nullable=true)
   */
  private $map_selection_buffer;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_data_grid;

  /**
   * @var int|null
   * @Column(type="integer", nullable=true)
   */
  private $min_edition_scale;

  /**
   * @var int|null
   * @Column(type="integer", nullable=true)
   */
  private $max_edition_scale;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $form_title;

  /**
   * @var FormSize
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\FormSize", inversedBy="layers_postgres")
   * @JoinColumn(name="form_display_size_id", referencedColumnName="formsize_id")
   */
  private $form_display_size;

  /**
   * @var int
   * @column(type="integer")
   */
  private $form_display_size_id;

  /**
   * @var FormSize
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\FormSize", inversedBy="layers_postgres")
   * @JoinColumn(name="form_insert_size_id", referencedColumnName="formsize_id")
   */
  private $form_insert_size;

  /**
   * @var int
   * @column(type="integer")
   */
  private $form_insert_size_id;

  /**
   * @var FormSize
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\FormSize", inversedBy="layers_postgres")
   * @JoinColumn(name="form_update_size_id", referencedColumnName="formsize_id")
   */
  private $form_update_size;

  /**
   * @var int
   * @column(type="integer")
   */
  private $form_update_size_id;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_clone;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_automatically_edit;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_multi_edit;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_filter;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $select_field;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $default_form_json_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $default_form_js_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $published_form_json_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $published_form_js_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $custom_form_json_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $custom_form_js_def;

  /**
   * @var bool
   * @Column(type="boolean", nullable=true)
   */
  private $is_geom_selection;

  /**
   * @var bool
   * @Column(type="boolean", nullable=true)
   */
  private $is_geom_updatable;

  /**
   * @var bool
   * @Column(type="boolean", nullable=true)
   */
  private $is_geometry_layer;

  /**
   * @var bool
   * @Column(type="boolean", nullable=true)
   */
  private $is_requestor_layer;

  /**
   * @var bool
   * @Column(type="boolean", nullable=true)
   */
  private $is_specific_table_form;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $schema_form;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $table_form;

  /**
   * @var bool
   * @Column(type="boolean", nullable=true)
   */
  private $is_specific_table_requestor;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $schema_requestor;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $table_requestor;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $google_street_view;

  /**
   * @return string|null Définition de la symbologie de la couche postgres au format JSON
   */
  public function getDefinitionSymJson(): ?string {
    return $this->definition_sym_json;
  }

  /**
   * @param string $definition_sym_json Définition de la symbologie de la couche postgres au format JSON
   * @return self
   */
  public function setDefinitionSymJson(?string $definition_sym_json): self {
    $this->definition_sym_json = $definition_sym_json;
    return $this;
  }

  /**
   * @return string|null Définition de la symbologie de la couche postgres au format map server
   */
  public function getDefinitionSymMapserver(): ?string {
    return $this->definition_sym_mapserver;
  }

  /**
   * @param string $definition_sym_mapserver Définition de la symbologie de la couche postgres au format map server
   * @return self
   */
  public function setDefinitionSymMapserver(?string $definition_sym_mapserver): self {
    $this->definition_sym_mapserver = $definition_sym_mapserver;
    return $this;
  }

  /**
   * @return bool|null Si c'est Oui : on utilise le studio Si c'est Non : l'utilisateur a basculé sur l'édition manuelle du mapfile
   */
  public function getIsSymDefinitionJson(): ?bool {
    return $this->is_sym_definition_json;
  }

  /**
   * @param bool $is_sym_definition_json Si c'est Oui : on utilise le studio Si c'est Non : l'utilisateur a basculé sur l'édition manuelle du mapfile
   * @return self
   */
  public function setIsSymDefinitionJson(?bool $is_sym_definition_json): self {
    $this->is_sym_definition_json = $is_sym_definition_json;
    return $this;
  }

  /**
   * @return string|null Base de données
   */
  public function getDatabase(): ?string {
    return $this->database;
  }

  /**
   * @param string $database Base de données
   * @return self
   */
  public function setDatabase(?string $database): self {
    $this->database = $database;
    return $this;
  }

  /**
   * @return string|null Schema
   */
  public function getSchema(): ?string {
    return $this->schema;
  }

  /**
   * @param string $schema Schema
   * @return self
   */
  public function setSchema(?string $schema): self {
    $this->schema = $schema;
    return $this;
  }

  /**
   * @return string|null Nom de la table à utiliser
   */
  public function getPgTable(): ?string {
    return $this->pg_table;
  }

  /**
   * @param string $pg_table Table
   * @return self
   */
  public function setPgTable(?string $pg_table): self {
    $this->pg_table = $pg_table;
    return $this;
  }

  /**
   * @return string|null Champ/colonne identifiant de la table
   */
  public function getIdfield(): ?string {
    return $this->idfield;
  }

  /**
   * @param string $idfield Champ/colonne identifiant de la table
   * @return self
   */
  public function setIdfield(?string $idfield): self {
    $this->idfield = $idfield;
    return $this;
  }

  /**
   * @return string|null Champ/colonne contenant la géométrie
   */
  public function getGeomfield(): ?string {
    return $this->geomfield;
  }

  /**
   * @param string $geomfield Champ/colonne contenant la géométrie
   * @return self
   */
  public function setGeomfield(?string $geomfield): self {
    $this->geomfield = $geomfield;
    return $this;
  }

  /**
   * @return array|null Système de coordonnées de la couche postgres
   */
  public function getCoordsys(): ?array{
    if (!isset($this->coordsys)) {
      return null;
    }
    // return context
    return [
      'coordsys_id' => $this->coordsys->getCoordsysId(),
      'label' => $this->coordsys->getLabel(),
    ];
  }

  /**
   * @param int $coordsys_id Système de coordonnées de la couche postgres
   * @return self
   */
  public function setCoordsysId(?int $coordsys_id): self {
    $this->coordsys_id = $coordsys_id;
    if (isset($this->oEntityManager)) {
      if (empty($coordsys_id)) {
        $this->setCoordsys(null);
      } else {
        $coordsys = $this->oEntityManager->getReference(Coordsys::class, $coordsys_id);
        $this->setCoordsys($coordsys);
      }

    }
    return $this;
  }

  /**
   * @return int
   */
  public function getCoordsysId(): ?int {
    return $this->coordsys_id;
  }

  /**
   * @param Coordsys $coordsys Système de coordonnées de la couche postgres
   * @return self
   */
  public function setCoordsys(?Coordsys $coordsys): self {
    $this->coordsys = $coordsys;
    return $this;
  }

  /**
   * @return array|null Type de géométrie de la couche postgres
   */
  public function getMsGeomType(): ?array{
    if (!isset($this->ms_geomtype)) {
      return null;
    }
    // return context
    return [
      'ms_geomtype_id' => $this->ms_geomtype->getMsGeomTypeId(),
      'label' => $this->ms_geomtype->getLabel(),
    ];
  }

  /**
   * @param int $ms_geomtype_id Type de géométrie de la couche postgres
   * @return self
   */
  public function setMsGeomTypeId(?int $ms_geomtype_id): self {
    $this->ms_geomtype_id = $ms_geomtype_id;
    if (isset($this->oEntityManager)) {
      if (empty($ms_geomtype_id)) {
        $this->setMsGeomType(null);
      } else {
        $ms_geomtype = $this->oEntityManager->getReference(MsGeomType::class, $ms_geomtype_id);
        $this->setMsGeomType($ms_geomtype);
      }
    }
    return $this;
  }

  /**
   * @return int
   */
  public function getMsGeomTypeId(): ?int {
    return $this->ms_geomtype_id;
  }

  /**
   * @param MsGeomType $ms_geomtype Type de géométrie de la couche postgres
   * @return self
   */
  public function setMsGeomType(?MsGeomType $ms_geomtype): self {
    $this->ms_geomtype = $ms_geomtype;
    return $this;
  }

  /**
   * @return string|null Mappage entre les colonnes, les libéllés, si elles sont disponibles en recherche etc.
   */
  public function getJsonColumns(): ?string {
    return $this->json_columns;
  }

  /**
   * @param string $json_columns Mappage entre les colonnes, les libéllés, si elles sont disponibles en recherche etc.
   * @return self
   */
  public function setJsonColumns(?string $json_columns): self {
    $this->json_columns = $json_columns;
    return $this;
  }

  /**
   * @return bool|null L'objet de la couche postgres est localisable
   */
  public function getIsLocation(): ?bool {
    return $this->is_location;
  }

  /**
   * @param bool $is_location L'objet de la couche postgres est localisable
   * @return self
   */
  public function setIsLocation(?bool $is_location): self {
    $this->is_location = $is_location;
    return $this;
  }

  /**
   * @return string|null Champ/colonne de la table sur laquel la recherche est faite
   */
  public function getLocationSearchField(): ?string {
    return $this->location_search_field;
  }

  /**
   * @param string $location_search_field Champ/colonne de la table sur laquel la recherche est faite
   * @return self
   */
  public function setLocationSearchField(?string $location_search_field): self {
    $this->location_search_field = $location_search_field;
    return $this;
  }

  /**
   * @return string|null Placeholder pour la recherche
   */
  public function getLocationPlaceholder(): ?string {
    return $this->location_placeholder;
  }

  /**
   * @param string $location_placeholder Placeholder pour la recherche
   * @return self
   */
  public function setLocationPlaceholder(?string $location_placeholder): self {
    $this->location_placeholder = $location_placeholder;
    return $this;
  }

  /**
   * @return string|null Champ/colonne à afficher dans la liste des resultats
   */
  public function getLocationResultField(): ?string {
    return $this->location_result_field;
  }

  /**
   * @param string $location_result_field Champ/colonne à afficher dans la liste des resultats
   * @return self
   */
  public function setLocationResultField(?string $location_result_field): self {
    $this->location_result_field = $location_result_field;
    return $this;
  }

  /**
   * @return array|null Mode de recherche de la couche postgres
   */
  public function getLocationSearchMode(): ?array{
    if (!isset($this->location_search_mode)) {
      return null;
    }
    // return context
    return [
      'search_mode_id' => $this->location_search_mode->getSearchModeId(),
      'label' => $this->location_search_mode->getLabel(),
    ];
  }

  /**
   * @param SearchMode $location_search_mode Mode de recherche de la couche postgres
   * @return self
   */
  public function setLocationSearchMode(?SearchMode $location_search_mode): self {
    $this->location_search_mode = $location_search_mode;
    return $this;
  }

  /**
   * @param int|null $location_search_mode_id Mode de recherche de la couche postgres
   * @return self
   */
  public function setLocationSearchModeId(?int $location_search_mode_id): self {
    $this->location_search_mode_id = $location_search_mode_id;
    if (isset($this->oEntityManager)) {
      if (empty($location_search_mode_id)) {
        $this->setLocationSearchMode(null);
      } else {
        $location_search_mode = $this->oEntityManager->getReference(SearchMode::class, $location_search_mode_id);
        $this->setLocationSearchMode($location_search_mode);
      }
    }
    return $this;
  }

  /**
   * @return int
   */
  public function getLocationSearchModeId(): ?int {
    return $this->location_search_mode_id;
  }

  /**
   * @return bool|null La couche postgres est sélectionnable sur la carte
   */
  public function getIsMapSelection(): ?bool {
    return $this->is_map_selection;
  }

  /**
   * @param bool $is_map_selection La couche postgres est sélectionnable sur la carte
   * @return self
   */
  public function setIsMapSelection(?bool $is_map_selection): self {
    $this->is_map_selection = $is_map_selection;
    return $this;
  }

  /**
   * @return float|null Buffer de selection
   */
  public function getMapSelectionBuffer(): ?float {
    return $this->map_selection_buffer;
  }

  /**
   * @param float $map_selection_buffer Buffer de selection
   * @return self
   */
  public function setMapSelectionBuffer(?float $map_selection_buffer): self {
    $this->map_selection_buffer = $map_selection_buffer;
    return $this;
  }

  /**
   * @return bool|null L'objet de la couche postgres est disponible dans le requeteur
   */
  public function getIsDataGrid(): ?bool {
    return $this->is_data_grid;
  }

  /**
   * @param bool $is_data_grid L'objet de la couche postgres est disponible dans le requeteur
   * @return self
   */
  public function setIsDataGrid(?bool $is_data_grid): self {
    $this->is_data_grid = $is_data_grid;
    return $this;
  }

  /**
   * @return int|null Echelle minimum en édition
   */
  public function getMinEditionScale(): ?int {
    return $this->min_edition_scale;
  }

  /**
   * @param int $min_edition_scale Echelle minimum en édition
   * @return self
   */
  public function setMinEditionScale(?int $min_edition_scale): self {
    $this->min_edition_scale = $min_edition_scale;
    return $this;
  }

  /**
   * @return int|null Echelle maximum en édition
   */
  public function getMaxEditionScale(): ?int {
    return $this->max_edition_scale;
  }

  /**
   * @param int $max_edition_scale Echelle maximum en édition
   * @return self
   */
  public function setMaxEditionScale(?int $max_edition_scale): self {
    $this->max_edition_scale = $max_edition_scale;
    return $this;
  }

  /**
   * @return string|null Titre du formulaire de l'objet de la couche postgres
   */
  public function getFormTitle(): ?string {
    return $this->form_title;
  }

  /**
   * @param string $form_title Titre du formulaire de l'objet de la couche postgres
   * @return self
   */
  public function setFormTitle(?string $form_title): self {
    $this->form_title = $form_title;
    return $this;
  }

  /**
   * @return array|null Taille du formulaire Display
   */
  public function getDisplayFormSize(): ?array{
    if (!isset($this->form_display_size)) {
      return null;
    }
    // return context
    return [
      'form_display_size_id' => $this->form_display_size->getFormSizeId(),
      'label' => $this->form_display_size->getLabel(),
    ];
  }

  /**
   * @param FormSize $form_display_size Taille du formulaire Display
   * @return self
   */
  public function setDisplayFormSize(?FormSize $form_display_size): self {
    $this->form_display_size = $form_display_size;
    return $this;
  }

  /**
   * @param int $form_display_size_id Taille du formulaire Display
   * @return self
   */
  public function setDisplayFormSizeId(?int $form_display_size_id): self {
    $this->findAndSetFormSizeEntity($form_display_size_id, 'form_display_size_id', 'setDisplayFormSize');
    return $this;
  }

  /**
   * @return int
   */
  public function getDisplayFormSizeId(): ?int {
    return $this->form_display_size_id;
  }

  /**
   * @return array|null Taille du formulaire Insert
   */
  public function getInsertFormSize(): ?array{
    if (!isset($this->form_insert_size)) {
      return null;
    }
    // return context
    return [
      'form_insert_size_id' => $this->form_insert_size->getFormSizeId(),
      'label' => $this->form_insert_size->getLabel(),
    ];
  }

  /**
   * @param FormSize $form_insert_size Taille du formulaire Insert
   * @return self
   */
  public function setInsertFormSize(?FormSize $form_insert_size): self {
    $this->form_insert_size = $form_insert_size;
    return $this;
  }

  /**
   * @param int $form_insert_size_id Taille du formulaire Insert
   * @return self
   */
  public function setInsertFormSizeId(?int $form_insert_size_id): self {
    $this->findAndSetFormSizeEntity($form_insert_size_id, 'form_insert_size_id', 'setInsertFormSize');
    return $this;
  }

  /**
   * @return int
   */
  public function getInsertFormSizeId(): ?int {
    return $this->form_insert_size_id;
  }

  /**
   * @return array|null Taille du formulaire Update
   */
  public function getUpdateFormSize(): ?array{
    if (!isset($this->form_update_size)) {
      return null;
    }
    // return context
    return [
      'form_update_size_id' => $this->form_update_size->getFormSizeId(),
      'label' => $this->form_update_size->getLabel(),
    ];
  }

  /**
   * @param FormSize $form_update_size Taille du formulaire Update
   * @return self
   */
  public function setUpdateFormSize(?FormSize $form_update_size): self {
    $this->form_update_size = $form_update_size;
    return $this;
  }

  /**
   * @param int $form_update_size_id Taille du formulaire Update
   * @return self
   */
  public function setUpdateFormSizeId(?int $form_update_size_id): self {
    $this->findAndSetFormSizeEntity($form_update_size_id, 'form_update_size_id', 'setUpdateFormSize');
    return $this;
  }

  /**
   * récupère et set une formsize
   * @param int $iFormSizeId
   * @param string $sAttrName nom du champ à set
   * @param string $sMethodName nom du setter à appelé pour l'entité
   * @return void
   */
  private function findAndSetFormSizeEntity(?int $iFormSizeId, string $sAttrName, string $sMethodName): void {
    $this->$sAttrName = $iFormSizeId;
    if (isset($this->oEntityManager)) {
      if (empty($iFormSizeId)) {
        $this->$sMethodName(null);
      } else {
        $form_size = $this->oEntityManager->getReference(FormSize::class, $iFormSizeId);
        $this->$sMethodName($form_size);
      }
    }
  }

  /**
   * @return int
   */
  public function getUpdateFormSizeId(): ?int {
    return $this->form_update_size_id;
  }

  /**
   * @return bool|null L'objet de la couche postgres est clonnable
   */
  public function getIsClone(): ?bool {
    return $this->is_clone;
  }

  /**
   * @param bool $is_clone L'objet de la couche postgres est clonnable
   * @return self
   */
  public function setIsClone(?bool $is_clone): self {
    $this->is_clone = $is_clone;
    return $this;
  }

  /**
   * @return bool|null L'objet de la couche postgres s'ouvre automatiquement après l'edition
   */
  public function getIsAutomaticallyEdit(): ?bool {
    return $this->is_automatically_edit;
  }

  /**
   * @param bool $is_automatically_edit L'objet de la couche postgres s'ouvre automatiquement après l'edition
   * @return self
   */
  public function setIsAutomaticallyEdit(?bool $is_automatically_edit): self {
    $this->is_automatically_edit = $is_automatically_edit;
    return $this;
  }

  /**
   * @return bool|null L'objet de la couche postgres est disponible en édition multiple
   */
  public function getIsMultiEdit(): ?bool {
    return $this->is_multi_edit;
  }

  /**
   * @param bool $is_multi_edit L'objet de la couche postgres est disponible en édition multiple
   * @return self
   */
  public function setIsMultiEdit(?bool $is_multi_edit): self {
    $this->is_multi_edit = $is_multi_edit;
    return $this;
  }

  /**
   * @return bool|null L'objet de la couche postgres est disponible en filter
   */
  public function getIsFilter(): ?bool {
    return $this->is_filter;
  }

  /**
   * @param bool $is_filter L'objet de la couche postgres est disponible en filter
   * @return self
   */
  public function setIsFilter(?bool $is_filter): self {
    $this->is_filter = $is_filter;
    return $this;
  }

  /**
   * @return string|null Champ à afficher pour les infobulles multiples
   */
  public function getSelectField(): ?string {
    return $this->select_field;
  }

  /**
   * @param string $select_field Champ à afficher pour les infobulles multiples
   * @return self
   */
  public function setSelectField(?string $select_field): self {
    $this->select_field = $select_field;
    return $this;
  }

  /**
   * @return string|null Définition JSON par défaut du formulaire
   */
  public function getDefaultFormJsonDef(): ?string {
    return $this->default_form_json_def;
  }

  /**
   * @param string $default_form_json_def Définition JSON par défaut du formulaire
   * @return self
   */
  public function setDefaultFormJsonDef(?string $default_form_json_def): self {
    $this->default_form_json_def = $default_form_json_def;
    return $this;
  }

  /**
   * @return string|null Code JS par défaut du formulaire
   */
  public function getDefaultFormJsDef(): ?string {
    return $this->default_form_js_def;
  }

  /**
   * @param string $default_form_js_def Code JS par défaut du formulaire
   * @return self
   */
  public function setDefaultFormJsDef(?string $default_form_js_def): self {
    $this->default_form_js_def = $default_form_js_def;
    return $this;
  }

  /**
   * @return string|null Définition JSON du formulaire publié
   */
  public function getPublishedFormJsonDef(): ?string {
    return $this->published_form_json_def;
  }

  /**
   * @param string $published_form_json_def Définition JSON du formulaire publié
   * @return self
   */
  public function setPublishedFormJsonDef(?string $published_form_json_def): self {
    $this->published_form_json_def = $published_form_json_def;
    return $this;
  }

  /**
   * @return string|null Code JS du formulaire publié
   */
  public function getPublishedFormJsDef(): ?string {
    return $this->published_form_js_def;
  }

  /**
   * @param string $published_form_js_def Code JS du formulaire publié
   * @return self
   */
  public function setPublishedFormJsDef(?string $published_form_js_def): self {
    $this->published_form_js_def = $published_form_js_def;
    return $this;
  }

  /**
   * @return string|null Définition JSON du formulaire custom
   */
  public function getCustomFormJsonDef(): ?string {
    return $this->custom_form_json_def;
  }

  /**
   * @param string $custom_form_json_def Définition JSON du formulaire custom
   * @return self
   */
  public function setCustomFormJsonDef(?string $custom_form_json_def): self {
    $this->custom_form_json_def = $custom_form_json_def;
    return $this;
  }

  /**
   * @return string|null Code JS du formulaire custom
   */
  public function getCustomFormJsDef(): ?string {
    return $this->custom_form_js_def;
  }

  /**
   * @param string $custom_form_js_def Code JS du formulaire custom
   * @return self
   */
  public function setCustomFormJsDef(?string $custom_form_js_def): self {
    $this->custom_form_js_def = $custom_form_js_def;
    return $this;
  }

  /**
   * @return bool|null Si c'est Oui : la géométrie peut être saisie en insertion.
   */
  public function getIsGeomSelection(): ?bool {
    return $this->is_geom_selection;
  }

  /**
   * @param bool $is_geom_selection Si c'est Oui : la géométrie peut être saisie en insertion.
   * @return self
   */
  public function setIsGeomSelection(?bool $is_geom_selection): self {
    $this->is_geom_selection = $is_geom_selection;
    return $this;
  }

  /**
   * @return bool|null Si c'est Oui : la géométrie peut être saisie en édition.
   */
  public function getIsGeomUpdatable(): ?bool {
    return $this->is_geom_updatable;
  }

  /**
   * @param bool $is_geom_updatable Si c'est Oui : la géométrie peut être saisie en édition.
   * @return self
   */
  public function setIsGeomUpdatable(?bool $is_geom_updatable): self {
    $this->is_geom_updatable = $is_geom_updatable;
    return $this;
  }

  /**
   * @return bool|null Utilisé pour afficher/caché la liste des colonnes utile dans le cas d'une couche géométrique
   */
  public function getIsGeometryLayer(): ?bool {
    return $this->is_geometry_layer;
  }

  /**
   * @param bool $is_geometry_layer Utilisé pour afficher/caché la liste des colonnes utile dans le cas d'une couche géométrique
   * @return self
   */
  public function setIsGeometryLayer(?bool $is_geometry_layer): self {
    $this->is_geometry_layer = $is_geometry_layer;
    return $this;
  }

  /**
   * @return bool|null Utilisé pour afficher/caché la liste des colonnes utile dans le cas d'une couche interrogeable
   */
  public function getIsRequestorLayer(): ?bool {
    return $this->is_requestor_layer;
  }

  /**
   * @param bool $is_requestor_layer Utilisé pour afficher/caché la liste des colonnes utile dans le cas d'une couche interrogeable
   * @return self
   */
  public function setIsRequestorLayer(?bool $is_requestor_layer): self {
    $this->is_requestor_layer = $is_requestor_layer;
    return $this;
  }

  /**
   * @return bool|null Si une table spécifique est utilisé pour les formulaires
   */
  public function getIsSpecificTableForm(): ?bool {
    return $this->is_specific_table_form;
  }

  /**
   * @param bool $is_specific_table_form Si une table spécifique est utilisé pour les formulaires
   * @return self
   */
  public function setIsSpecificTableForm(?bool $is_specific_table_form): self {
    $this->is_specific_table_form = $is_specific_table_form;
    return $this;
  }

  /**
   * @return string|null Schema pour le formulaire
   */
  public function getSchemaForm(): ?string {
    return $this->schema_form;
  }

  /**
   * @param string $schema_form Schema pour le formulaire
   * @return self
   */
  public function setSchemaForm(?string $schema_form): self {
    $this->schema_form = $schema_form;
    return $this;
  }

  /**
   * @return string|null Nom de la table à utiliser pour le formulaire
   */
  public function getTableForm(): ?string {
    return $this->table_form;
  }

  /**
   * @param string $table_form Table pour le formulaire
   * @return self
   */
  public function setTableForm(?string $table_form): self {
    $this->table_form = $table_form;
    return $this;
  }

  /**
   * @return bool|null Si une table spécifique est utilisé pour le requêteur
   */
  public function getIsSpecificTableRequestor(): ?bool {
    return $this->is_specific_table_requestor;
  }

  /**
   * @param bool $is_specific_table_requestor Si une table spécifique est utilisé pour le requêteur
   * @return self
   */
  public function setIsSpecificTableRequestor(?bool $is_specific_table_requestor): self {
    $this->is_specific_table_requestor = $is_specific_table_requestor;
    return $this;
  }

  /**
   * @return string|null Schema pour le requêteur
   */
  public function getSchemaRequestor(): ?string {
    return $this->schema_requestor;
  }

  /**
   * @param string $schema_requestor Schema pour le requêteur
   * @return self
   */
  public function setSchemaRequestor(?string $schema_requestor): self {
    $this->schema_requestor = $schema_requestor;
    return $this;
  }

  /**
   * @return string|null Nom de la table à utiliser pour le requêteur
   */
  public function getTableRequestor(): ?string {
    return $this->table_requestor;
  }

  /**
   * @param string $table_requestor Table pour le requêteur
   * @return self
   */
  public function setTableRequestor(?string $table_requestor): self {
    $this->table_requestor = $table_requestor;
    return $this;
  }

  /**
   * @return bool|null 
   */
  public function getGoogleStreetView(): ?bool {
    return $this->google_street_view;
  }

  /**
   * @param bool $google_street_view 
   * @return self
   */
  public function setGoogleStreetView(?bool $google_street_view): self {
    $this->google_street_view = $google_street_view;
    return $this;
  }

  /**
   * @param bool $bAllowAdminLoginPwd autorise la récupération du login/pwd depuis les properties
   * @param string|null $sSchemaType
   * @return array|null
   */
  public function getDataConnectionFromLayer(bool $bAllowAdminLoginPwd = true, ?string $sSchemaType = null): ?array{
    // récupère la connexion d'après la datasource
    $aData = $this->datasource->getPostgresConnexionData($bAllowAdminLoginPwd, $this->database);

    $aData['field_id'] = $this->idfield;

    $aData['schema'] = $this->schema;
    $aData['table'] = $this->pg_table;

    if ($sSchemaType == 'form' && $this->is_specific_table_form === true) {
      $aData['schema'] = $this->schema_form;
      $aData['table'] = $this->table_form;
    }

    if ($sSchemaType == 'requestor' && $this->is_specific_table_requestor === true) {
      $aData['schema'] = $this->schema_requestor;
      $aData['table'] = $this->table_requestor;
    }

    if (empty($aData['schema']) || empty($aData['table']) || empty($aData['database'])) {
      return null;
    }

    return $aData;
  }

  /**
   * {@inheritdoc}
   */
  public static function getLayerParams(array $aLayer): array{
    $layerName = MapServerService::cleanLayerName($aLayer['name']);

    $paramsLayers = [];
    // Si la couche est de type géométrie alors params Layers est une suite de trois couche point, ligne, polygone
    if (isset($aLayer['ms_geomtype']) && $aLayer['ms_geomtype']['label'] === "GEOMETRY") {
      if (is_string($aLayer['definition_sym_json'])) {
        $aLayer['definition_sym_json'] = json_decode($aLayer['definition_sym_json'], true);
      } else if (is_array($aLayer['definition_sym_json'])) {
        $aLayer['definition_sym_json'] = $aLayer['definition_sym_json'];
      }
      foreach (['point', 'line', 'polygon'] as $geomtype) {
        if (array_key_exists($geomtype, $aLayer['definition_sym_json'])) {
          $paramsLayers[] = "{$layerName}_{$geomtype}";
        }
      }
    }
    if (empty($paramsLayers)) {
      $paramsLayers = [$layerName];
    }

    $aParamsLayer = [
      'layerType' => 'imagewms',
      'url' => $aLayer['url'],
      'version' => $aLayer['datasource']['flux_version'],
      'params' => [
        'LAYERS' => implode(',', $paramsLayers),
        'VERSION' => '1.3.0',
      ],
      'coordsys_id' => $aLayer['coordsys_id'],
      'datemaj' => $aLayer['datemaj'],
      'definition_sym_json' => $aLayer['definition_sym_json'],
      'form_display_size_id' => $aLayer['form_display_size_id'],
      'form_insert_size_id' => $aLayer['form_insert_size_id'],
      'form_title' => $aLayer['form_title'],
      'form_update_size_id' => $aLayer['form_update_size_id'],
      'geomfield' => $aLayer['geomfield'],
      'idfield' => $aLayer['idfield'],
      'is_automatically_edit' => $aLayer['is_automatically_edit'],
      'is_clone' => $aLayer['is_clone'],
      'is_geom_selection' => $aLayer['is_geom_selection'],
      'is_geom_updatable' => $aLayer['is_geom_updatable'],
      'is_geometry_layer' => $aLayer['is_geometry_layer'],
      'is_multi_edit' => $aLayer['is_multi_edit'],
      'is_filter' => $aLayer['is_filter'],
      'is_requestor_layer' => $aLayer['is_requestor_layer'],
      'is_specific_table_form' => $aLayer['is_specific_table_form'],
      'is_specific_table_requestor' => $aLayer['is_specific_table_requestor'],
      'is_sym_definition_json' => $aLayer['is_sym_definition_json'],
      'json_columns' => $aLayer['json_columns'],
      'location_placeholder' => $aLayer['location_placeholder'],
      'location_result_field' => $aLayer['location_result_field'],
      'location_search_field' => $aLayer['location_search_field'],
      'location_search_mode_id' => $aLayer['location_search_mode_id'],
      'map_selection_buffer' => $aLayer['map_selection_buffer'],
      'max_edition_scale' => $aLayer['max_edition_scale'],
      'min_edition_scale' => $aLayer['min_edition_scale'],
      'ms_geomtype' => $aLayer['ms_geomtype'],
      'ms_geomtype_id' => $aLayer['ms_geomtype_id'],
      'select_field' => $aLayer['select_field'],
      'published_form_js_def' => $aLayer['published_form_js_def'],
      'published_form_json_def' => $aLayer['published_form_json_def'],
      'schema_form' => $aLayer['schema_form'],
      'schema_requestor' => $aLayer['schema_requestor'],
      'table_form' => $aLayer['table_form'],
      'table_requestor' => $aLayer['table_requestor'],
      'database' => $aLayer['database'],
      'schema' => $aLayer['schema'],
      'table' => $aLayer['pg_table'],
      'datasource_id' => $aLayer['datasource_id'],
      'serverType' => 'mapserver',
      'hidpi' => false,
      'have_select_right' => $aLayer['have_select_right'],
      'have_insert_right' => $aLayer['have_insert_right'],
      'have_update_right' => $aLayer['have_update_right'],
      'have_delete_right' => $aLayer['have_delete_right'],
      'google_street_view' => $aLayer['google_street_view']
    ];

    return array_merge(parent::getLayerParams($aLayer), $aParamsLayer);
  }

  /**
   * {@inheritdoc}
   */
  public function jsonSerialize(?array $aRouteConfiguration = null): array{
    $aData = parent::jsonSerialize($aRouteConfiguration);
    $aData["definition_sym_json"] = $this->getDefinitionSymJson();
    $aData["definition_sym_mapserver"] = $this->getDefinitionSymMapserver();
    $aData["is_sym_definition_json"] = $this->getIsSymDefinitionJson();
    $aData["database"] = $this->getDatabase();
    $aData["schema"] = $this->getSchema();
    $aData["pg_table"] = $this->getPgTable();
    $aData["idfield"] = $this->getIdfield();
    $aData["geomfield"] = $this->getGeomfield();
    $aData["coordsys_id"] = $this->getCoordsysId();
    $aData["coordsys"] = $this->getCoordsys();
    $aData["ms_geomtype_id"] = $this->getMsGeomTypeId();
    $aData["ms_geomtype"] = $this->getMsGeomType();
    $aData["json_columns"] = $this->getJsonColumns();
    $aData["is_location"] = $this->getIsLocation();
    $aData["location_search_field"] = $this->getLocationSearchField();
    $aData["location_placeholder"] = $this->getLocationPlaceholder();
    $aData["location_result_field"] = $this->getLocationResultField();
    $aData["location_search_mode_id"] = $this->getLocationSearchModeId();
    $aData["location_search_mode"] = $this->getLocationSearchMode();
    $aData["is_map_selection"] = $this->getIsMapSelection();
    $aData["map_selection_buffer"] = $this->getMapSelectionBuffer();
    $aData["is_data_grid"] = $this->getIsDataGrid();
    $aData["min_edition_scale"] = $this->getMinEditionScale();
    $aData["max_edition_scale"] = $this->getMaxEditionScale();
    $aData["form_title"] = $this->getFormTitle();
    $aData["form_display_size_id"] = $this->getDisplayFormSizeId();
    $aData["form_display_size"] = $this->getDisplayFormSize();
    $aData["form_insert_size_id"] = $this->getInsertFormSizeId();
    $aData["form_insert_size"] = $this->getInsertFormSize();
    $aData["form_update_size_id"] = $this->getUpdateFormSizeId();
    $aData["form_update_size"] = $this->getUpdateFormSize();
    $aData["is_clone"] = $this->getIsClone();
    $aData["is_automatically_edit"] = $this->getIsAutomaticallyEdit();
    $aData["is_multi_edit"] = $this->getIsMultiEdit();
    $aData["is_filter"] = $this->getIsFilter();
    $aData["select_field"] = $this->getSelectField();
    $aData["default_form_json_def"] = $this->getDefaultFormJsonDef();
    $aData["default_form_js_def"] = $this->getDefaultFormJsDef();
    $aData["published_form_json_def"] = $this->getPublishedFormJsonDef();
    $aData["published_form_js_def"] = $this->getPublishedFormJsDef();
    $aData["custom_form_json_def"] = $this->getCustomFormJsonDef();
    $aData["custom_form_js_def"] = $this->getCustomFormJsDef();
    $aData["is_geom_selection"] = $this->getIsGeomSelection();
    $aData["is_geom_updatable"] = $this->getIsGeomUpdatable();
    $aData["is_geometry_layer"] = $this->getIsGeometryLayer();
    $aData["is_requestor_layer"] = $this->getIsRequestorLayer();
    $aData["is_specific_table_form"] = $this->getIsSpecificTableForm();
    $aData["schema_form"] = $this->getSchemaForm();
    $aData["table_form"] = $this->getTableForm();
    $aData["is_specific_table_requestor"] = $this->getIsSpecificTableRequestor();
    $aData["schema_requestor"] = $this->getSchemaRequestor();
    $aData["table_requestor"] = $this->getTableRequestor();
    $aData["google_street_view"] = $this->getGoogleStreetView();
    return $aData;
  }

  /**
   * {@inheritdoc}
   */
  public function getDeserializeMap(): array{
    $aData = parent::getDeserializeMap();
    $aData["definition_sym_json"] = "setDefinitionSymJson";
    $aData["definition_sym_mapserver"] = "setDefinitionSymMapserver";
    $aData["is_sym_definition_json"] = "setIsSymDefinitionJson";
    $aData["database"] = "setDatabase";
    $aData["schema"] = "setSchema";
    $aData["pg_table"] = "setPgTable";
    $aData["idfield"] = "setIdfield";
    $aData["geomfield"] = "setGeomfield";
    $aData["coordsys_id"] = "setCoordsysId";
    $aData["ms_geomtype_id"] = "setMsGeomTypeId";
    $aData["json_columns"] = "setJsonColumns";
    $aData["is_location"] = "setIsLocation";
    $aData["location_search_field"] = "setLocationSearchField";
    $aData["location_placeholder"] = "setLocationPlaceholder";
    $aData["location_result_field"] = "setLocationResultField";
    $aData["location_search_mode_id"] = "setLocationSearchModeId";
    $aData["is_map_selection"] = "setIsMapSelection";
    $aData["map_selection_buffer"] = "setMapSelectionBuffer";
    $aData["is_data_grid"] = "setIsDataGrid";
    $aData["min_edition_scale"] = "setMinEditionScale";
    $aData["max_edition_scale"] = "setMaxEditionScale";
    $aData["form_title"] = "setFormTitle";
    $aData["form_display_size_id"] = "setDisplayFormSizeId";
    $aData["form_insert_size_id"] = "setInsertFormSizeId";
    $aData["form_update_size_id"] = "setUpdateFormSizeId";
    $aData["is_clone"] = "setIsClone";
    $aData["is_automatically_edit"] = "setIsAutomaticallyEdit";
    $aData["is_multi_edit"] = "setIsMultiEdit";
    $aData["is_filter"] = "setIsFilter";
    $aData["select_field"] = "setSelectField";
    $aData["default_form_json_def"] = "setDefaultFormJsonDef";
    $aData["default_form_js_def"] = "setDefaultFormJsDef";
    $aData["published_form_json_def"] = "setPublishedFormJsonDef";
    $aData["published_form_js_def"] = "setPublishedFormJsDef";
    $aData["custom_form_json_def"] = "setCustomFormJsonDef";
    $aData["custom_form_js_def"] = "setCustomFormJsDef";
    $aData["is_geom_selection"] = "setIsGeomSelection";
    $aData["is_geom_updatable"] = "setIsGeomUpdatable";
    $aData["is_geometry_layer"] = "setIsGeometryLayer";
    $aData["is_requestor_layer"] = "setIsRequestorLayer";
    $aData["is_specific_table_form"] = "setIsSpecificTableForm";
    $aData["schema_form"] = "setSchemaForm";
    $aData["table_form"] = "setTableForm";
    $aData["is_specific_table_requestor"] = "setIsSpecificTableRequestor";
    $aData["schema_requestor"] = "setSchemaRequestor";
    $aData["table_requestor"] = "setTableRequestor";
    $aData["google_street_view"] = "setGoogleStreetView";
    return $aData;
  }

}
