<?php

namespace App\Module\Vmap\WebService\Entity;

use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour layerwms
 *
 * @Entity(repositoryClass="App\Module\Vmap\WebService\Repository\LayerRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="layerwms")
 */
class LayerWMS extends AbstractLayer {
  /**
   * @var string
   */
  public const TYPE = 'wms';
  /**
   * @var int
   * @Id
   * @Column(type="integer", name="layer_id", nullable=false)
   */
  protected $layer_id;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $layer_name;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $crs_list;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $layer_style;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $layer_format;

  /**
   * @var array
   * @Column(type="json", nullable=true)
   */
  private $layer_options;

  /**
   * @var bool
   * @Column(type="boolean", nullable=true)
   */
  private $is_tiled;

  /**
   * @return string|null
   */
  public function getLayerName(): ?string {
    return $this->layer_name;
  }

  /**
   * @param string $layer_name
   * @return self
   */
  public function setLayerName(?string $layer_name): self {
    $this->layer_name = $layer_name;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getCRSList(): ?string {
    return $this->crs_list;
  }

  /**
   * @param string $crs_list
   * @return self
   */
  public function setCRSList(?string $crs_list): self {
    $this->crs_list = $crs_list;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getLayerStyle(): ?string {
    return $this->layer_style;
  }

  /**
   * @param string $layer_style
   * @return self
   */
  public function setLayerStyle(?string $layer_style): self {
    $this->layer_style = $layer_style;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getLayerFormat(): ?string {
    return $this->layer_format;
  }

  /**
   * @param string $layer_format
   * @return self
   */
  public function setLayerFormat(?string $layer_format): self {
    $this->layer_format = $layer_format;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getLayerOptions(): ?string {
    if (isset($this->layer_options)) {
      return json_encode($this->layer_options);
    }
    return null;
  }

  /**
   * @param string $layer_options
   * @return self
   */
  public function setLayerOptions(?string $layer_options): self {
    $this->layer_options = json_decode($layer_options, true);
    return $this;
  }

  /**
   * @return bool|null
   */
  public function getIsTiled(): ?bool {
    return $this->is_tiled;
  }

  /**
   * @param bool $is_tiled
   * @return self
   */
  public function setIsTiled(?bool $is_tiled): self {
    $this->is_tiled = $is_tiled;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function getLayerParams(array $aLayer): array{
    $aParamsLayer = [
      'layerType' => $aLayer['is_tiled'] === true ? 'tilewms' : 'imagewms',
      'url' => $aLayer['datasource']['flux_url'],
      'version' => $aLayer['datasource']['flux_version'],
      'params' => [
        'LAYERS' => $aLayer['layer_name'],
        'FORMAT' => $aLayer['layer_format'],
        'VERSION' => $aLayer['datasource']['flux_version'],
      ],
    ];

    if (isset($aLayer['datasource']['flux_login'])) {
      $aParamsLayer['login'] = $aLayer['datasource']['flux_login'];
    }
    if (isset($aLayer['datasource']['flux_password'])) {
      $aParamsLayer['password'] = $aLayer['datasource']['flux_password'];
    }

    return array_merge(parent::getLayerParams($aLayer), $aParamsLayer);
  }

  /**
   * {@inheritdoc}
   */
  public function jsonSerialize(?array $aRouteConfiguration = null): array{
    $aData = parent::jsonSerialize($aRouteConfiguration);
    $aData['layer_name'] = $this->getLayerName();
    $aData['crs_list'] = $this->getCRSList();
    $aData['layer_style'] = $this->getLayerStyle();
    $aData['layer_format'] = $this->getLayerFormat();
    $aData['layer_options'] = $this->getLayerOptions();
    $aData['is_tiled'] = $this->getIsTiled();
    return $aData;
  }

  /**
   * {@inheritdoc}
   */
  public function getDeserializeMap(): array{
    $aData = parent::getDeserializeMap();
    $aData['layer_name'] = "setLayerName";
    $aData['crs_list'] = "setCRSList";
    $aData['layer_style'] = "setLayerStyle";
    $aData['layer_format'] = "setLayerFormat";
    $aData['layer_options'] = "setLayerOptions";
    $aData['is_tiled'] = "setIsTiled";
    return $aData;
  }
}
