<?php

namespace App\Module\Vmap\WebService\Entity;

use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour layerwmts
 *
 *
 * @Entity(repositoryClass="App\Module\Vmap\WebService\Repository\LayerRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="layerwmts")
 */
class LayerWMTS extends AbstractLayer {
  /**
   * @var string
   */
  public const TYPE = 'wmts';
  /**
   * @var int
   * @Id
   * @Column(type="integer", name="layer_id", nullable=false)
   */
  protected $layer_id;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $layer_name;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $matrix_set;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $layer_style;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $layer_format;

  /**
   * @var array
   * @Column(type="json", nullable=true)
   */
  private $layer_options;

  /**
   * @return string|null
   */
  public function getLayerName(): ?string {
    return $this->layer_name;
  }

  /**
   * @param string $layer_name
   * @return self
   */
  public function setLayerName(?string $layer_name): self {
    $this->layer_name = $layer_name;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getMatrixSet(): ?string {
    return $this->matrix_set;
  }

  /**
   * @param string $matrix_set
   * @return self
   */
  public function setMatrixSet(?string $matrix_set): self {
    $this->matrix_set = $matrix_set;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getLayerStyle(): ?string {
    return $this->layer_style;
  }

  /**
   * @param string $layer_style
   * @return self
   */
  public function setLayerStyle(?string $layer_style): self {
    $this->layer_style = $layer_style;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getLayerFormat(): ?string {
    return $this->layer_format;
  }

  /**
   * @param string $layer_format
   * @return self
   */
  public function setLayerFormat(?string $layer_format): self {
    $this->layer_format = $layer_format;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getLayerOptions(): ?string {
    if (isset($this->layer_options)) {
      if (is_string($this->layer_options)) {
        return $this->layer_options;
      } else {
        return json_encode($this->layer_options);
      }
    }
    return null;
  }

  /**
   * @param string $layer_options
   * @return self
   */
  public function setLayerOptions(?string $layer_options): self {
    // $this->layer_options = json_decode($layer_options, true); -> Dieu seul sait pourquoi ça ne fonctionne pas
    $this->layer_options = $layer_options;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function getLayerParams(array $aLayer): array{
    $aParamsLayer = [
      'layerType' => 'wmts',
      'url' => $aLayer['datasource']['flux_url'],
      'version' => $aLayer['datasource']['flux_version'],
      'service_options' => $aLayer['layer_options'],
      'layer' => $aLayer['layer_name'],
      'matrixSet' => $aLayer['matrix_set'],
      'style' => $aLayer['layer_style'],
      'format' => $aLayer['layer_format'],
      'requestEncoding' => $aLayer['datasource']['flux_type'],
    ];

    if (isset($aLayer['datasource']['flux_login'])) {
      $aParamsLayer['login'] = $aLayer['datasource']['flux_login'];
    }
    if (isset($aLayer['datasource']['flux_password'])) {
      $aParamsLayer['password'] = $aLayer['datasource']['flux_password'];
    }

    return array_merge(parent::getLayerParams($aLayer), $aParamsLayer);
  }

  public function jsonSerialize(?array $aRouteConfiguration = null): array
  {
    $aData = parent::jsonSerialize($aRouteConfiguration);
    $aData['layer_name'] = $this->getLayerName();
    $aData['matrix_set'] = $this->getMatrixSet();
    $aData['layer_style'] = $this->getLayerStyle();
    $aData['layer_format'] = $this->getLayerFormat();
    $aData['layer_options'] = $this->getLayerOptions();
    return $aData;
  }

  public function getDeserializeMap(): array
  {
    $aData = parent::getDeserializeMap();
    $aData['layer_name'] = "setLayerName";
    $aData['matrix_set'] = "setMatrixSet";
    $aData['layer_style'] = "setLayerStyle";
    $aData['layer_format'] = "setLayerFormat";
    $aData['layer_options'] = "setLayerOptions";
    return $aData;
  }
}
