<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour print template
 *
 * @author Mickaël Fernandez <mickael.fernandez@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="printtemplate")
 */
class PrintTemplate extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="printtemplate_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).printtemplate_printtemplate_id_seq")
   */
  private $printtemplate_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $name;

  /**
   * @var string
   * @Column(type="string", length=2, nullable=false)
   */
  private $rt_format_id;

  /**
   * @var string
   * @Column(type="string", length=8, nullable=false)
   */
  private $rt_orientation_id;

  /**
   * @var string
   * @Column(type="string", nullable=false)
   */
  private $definition;

  /**
   * @var string
   * @Column(type="string", length=255, nullable=false)
   */
  private $outputformats_id;

  /**
   * @var int
   * @Column(type="integer", nullable=true)
   */
  private $printtemplate_order;

  /**
   * @var ArrayCollection<\App\Module\Vitis\WebService\Entity\Group>
   * @ManyToMany(targetEntity="App\Module\Vitis\WebService\Entity\Group")
   * @JoinTable(
   *   name="@prop(schema_vmap).printtemplate_group",
   *   joinColumns={@JoinColumn(name="printtemplate_id", referencedColumnName="printtemplate_id")},
   *   inverseJoinColumns={@JoinColumn(name="group_id", referencedColumnName="group_id")}
   * )
   */
  private $aGroups;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $default_form_json_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $default_form_js_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $published_form_json_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $published_form_js_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $custom_form_json_def;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $custom_form_js_def;

  /**
   * @return string|null
   */
  public function getPrintTemplateId(): string {
    return $this->printtemplate_id;
  }

  /**
   * @return string|null
   */
  public function getName(): string {
    return $this->name;
  }

  /**
   * @param string $name
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getFormatId(): string {
    return $this->rt_format_id;
  }

  /**
   * @param string $rt_format_id
   * @return self
   */
  public function setFormatId(?string $rt_format_id): self {
    $this->rt_format_id = $rt_format_id;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getOrientationId(): string {
    return $this->rt_orientation_id;
  }

  /**
   * @param string $rt_orientation_id
   * @return self
   */
  public function setOrientationId(?string $rt_orientation_id): self {
    $this->rt_orientation_id = $rt_orientation_id;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getDefinition(): string {
    return $this->definition;
  }

  /**
   * @param string $definition
   * @return self
   */
  public function setDefinition(?string $definition): self {
    $this->definition = $definition;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getOutputFormatsId(): string {
    return $this->outputformats_id;
  }

  /**
   * @param string $outputformats_id
   * @return self
   */
  public function setOutputFormatsId(?string $outputformats_id): self {
    $this->outputformats_id = $outputformats_id;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getPrintTemplateOrder(): ?string {
    return $this->printtemplate_order;
  }

  /**
   * @param int $printtemplate_order
   * @return self
   */
  public function setPrintTemplateOrder(?int $printtemplate_order): self {
    $this->printtemplate_order = $printtemplate_order;
    return $this;
  }

  /**
   * @return int[] liste des user_id
   */
  public function getGroups(): array {
    $aGroupId = [];
    
    if ($this->aGroups) {
      foreach ($this->aGroups as $oGroup) {
        $aGroupId[] = $oGroup->getGroupId();
      }
    }
    
    return $aGroupId;
  }

  /**
   * @return string|null Définition JSON par défaut du formulaire
   */
  public function getDefaultFormJsonDef(): ?string {
    return $this->default_form_json_def;
  }

  /**
   * @param string $default_form_json_def Définition JSON par défaut du formulaire
   * @return self
   */
  public function setDefaultFormJsonDef(?string $default_form_json_def): self {
    $this->default_form_json_def = $default_form_json_def;
    return $this;
  }

  /**
   * @return string|null Code JS par défaut du formulaire
   */
  public function getDefaultFormJsDef(): ?string {
    return $this->default_form_js_def;
  }

  /**
   * @param string $default_form_js_def Code JS par défaut du formulaire
   * @return self
   */
  public function setDefaultFormJsDef(?string $default_form_js_def): self {
    $this->default_form_js_def = $default_form_js_def;
    return $this;
  }

  /**
   * @return string|null Définition JSON du formulaire publié
   */
  public function getPublishedFormJsonDef(): ?string {
    return $this->published_form_json_def;
  }

  /**
   * @param string $published_form_json_def Définition JSON du formulaire publié
   * @return self
   */
  public function setPublishedFormJsonDef(?string $published_form_json_def): self {
    $this->published_form_json_def = $published_form_json_def;
    return $this;
  }

  /**
   * @return string|null Code JS du formulaire publié
   */
  public function getPublishedFormJsDef(): ?string {
    return $this->published_form_js_def;
  }

  /**
   * @param string $published_form_js_def Code JS du formulaire publié
   * @return self
   */
  public function setPublishedFormJsDef(?string $published_form_js_def): self {
    $this->published_form_js_def = $published_form_js_def;
    return $this;
  }

  /**
   * @return string|null Définition JSON du formulaire custom
   */
  public function getCustomFormJsonDef(): ?string {
    return $this->custom_form_json_def;
  }

  /**
   * @param string $custom_form_json_def Définition JSON du formulaire custom
   * @return self
   */
  public function setCustomFormJsonDef(?string $custom_form_json_def): self {
    $this->custom_form_json_def = $custom_form_json_def;
    return $this;
  }

  /**
   * @return string|null Code JS du formulaire custom
   */
  public function getCustomFormJsDef(): ?string {
    return $this->custom_form_js_def;
  }

  /**
   * @param string $custom_form_js_def Code JS du formulaire custom
   * @return self
   */
  public function setCustomFormJsDef(?string $custom_form_js_def): self {
    $this->custom_form_js_def = $custom_form_js_def;
    return $this;
  }
}
