<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\Common\Persistence\ObjectManagerAware;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 * Objet ORM pour publication_flux
 *
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="publication_flux")
 */
class PublicationFlux extends AbstractEntity implements ObjectManagerAware {

  /**
   * @var EntityManager
   */
  protected $oEntityManager;

  /**
   * @var LoggerManager
   */
  protected $oLoggerManager;

  /**
   * @var int
   * @Column(type="integer", name="flux_layer_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).publication_flux_flux_layer_id_seq")
   */
  private $flux_layer_id;

  /**
   * @var int
   * @Column(type="integer", name="publication_id", nullable=false)
   */
  private $publication_id;

  /**
   * @var int
   * @Column(type="integer", name="flux_id", nullable=false)
   */
  private $flux_id;

  /**
   * @var AbstractPublication
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\AbstractPublication", inversedBy="aFlux")
   * @JoinColumn(name="publication_id", referencedColumnName="publication_id")
   */
  private $publication_name;

  /**
   * @var Flux|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Flux", cascade={"persist"}, inversedBy="publications")
   * @JoinColumn(name="flux_id", referencedColumnName="flux_id")
   */
  protected $flux;

  /**
   * @return int identifiant du flux_layer
   */
  public function getFluxLayerId(): ?int {
    return $this->flux_layer_id;
  }

  /**
   * @return int identifiant du layer_id
   */
  public function getPublicationId(): ?int {
    return $this->publication_id;
  }

  /**
   * @param int|null $publication_id
   * @return self
   */
  public function setPublicationId(?int $publication_id): self {
    $this->publication_id = $publication_id;

    if (!isset($this->oEntityManager) || !isset($this->publication_id)) {
      return $this;
    }
    try {
      $oPublicationRepository = $this->oEntityManager->getRepository(AbstractPublication::class);

      if (!$oPublicationRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oPublicationRepository->findById($this->publication_id);

      if (empty($oResult)) {
        return $this;
      }
      $this->publication = $oResult[0];
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find layer id ' . $this->publication_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @return array|null
   */
  public function getPublication(): ?string{
    if (!isset($this->publication_name)) {
      return null;
    }
    // return context
    return $this->publication_name->getName();
  }

  /**
   * @param AbstractPublication
   * @return self
   */
  public function setPublication(AbstractPublication $publication_name): self {
    $this->publication_name = $publication_name;
    return $this;
  }

  /**
   * @return Flux flux
   */
  public function getFlux(): ?Flux {
    return $this->flux;
  }

  /**
   * @return int identifiant du flux_id
   */
  public function getFluxId(): ?int {
    return $this->flux_id;
  }

  /**
   * @param int $flux_id
   * @return self
   */
  public function setFluxId(?int $flux_id): self {
    $this->flux_id = $flux_id;

    if (!isset($this->oEntityManager) || !isset($this->flux_id)) {
      return $this;
    }
    try {
      $oFluxRepository = $this->oEntityManager->getRepository(Flux::class);

      if (!$oFluxRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oFluxRepository->findById($this->flux_id);

      if (empty($oResult)) {
        return $this;
      }
      $this->flux = $oResult[0];
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find flux id ' . $this->flux_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @param Flux|null $flux
   * @return self
   */
  public function setFlux(?Flux $flux): self {
    $this->flux = $flux;
    return $this;
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base (ObjectManagerAware)
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
