<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vmap\WebService\Service\MapServerService;
use Doctrine\Common\Persistence\ObjectManagerAware;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour publicationpostgres
 *
 * @author Laurie Ambroise <laurie.ambroise@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="publicationpostgres")
 */
class PublicationPostgres extends AbstractPublication implements ObjectManagerAware {
  /**
   * @var string
   */
  public const TYPE = 'postgres';

  /**
   * @var int
   * @Id
   * @Column(type="integer", name="publication_id", nullable=false)
   */
  protected $publication_id;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $definition_sym_json;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $definition_sym_mapserver;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $is_sym_definition_json;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $database;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $schema;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $pg_table;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $idfield;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $geomfield;

  /**
   * @var Coordsys
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Coordsys", inversedBy="publications_postgres")
   * @JoinColumn(name="coordsys_id", referencedColumnName="coordsys_id")
   */
  private $coordsys;

  /**
   * @var int
   * @column(type="integer")
   */
  private $coordsys_id;

  /**
   * @var MsGeomType
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\MsGeomType", inversedBy="publications_postgres")
   * @JoinColumn(name="ms_geomtype_id", referencedColumnName="ms_geomtype_id")
   */
  private $ms_geomtype;

  /**
   * @var int
   * @column(type="integer")
   */
  private $ms_geomtype_id;

  /**
   * @return string|null Définition de la symbologie de la publication postgres au format JSON
   */
  public function getDefinitionSymJson(): ?string {
    return $this->definition_sym_json;
  }

  /**
   * @param string $definition_sym_json Définition de la symbologie de la publication postgres au format JSON
   * @return self
   */
  public function setDefinitionSymJson(?string $definition_sym_json): self {
    $this->definition_sym_json = $definition_sym_json;
    return $this;
  }

  /**
   * @return string|null Définition de la symbologie de la publication postgres au format map server
   */
  public function getDefinitionSymMapserver(): ?string {
    return $this->definition_sym_mapserver;
  }

  /**
   * @param string $definition_sym_mapserver Définition de la symbologie de la publication postgres au format map server
   * @return self
   */
  public function setDefinitionSymMapserver(?string $definition_sym_mapserver): self {
    $this->definition_sym_mapserver = $definition_sym_mapserver;
    return $this;
  }

  /**
   * @return bool|null Si c'est Oui : on utilise le studio Si c'est Non : l'utilisateur a basculé sur l'édition manuelle du mapfile
   */
  public function getIsSymDefinitionJson(): ?bool {
    return $this->is_sym_definition_json;
  }

  /**
   * @param bool $is_sym_definition_json Si c'est Oui : on utilise le studio Si c'est Non : l'utilisateur a basculé sur l'édition manuelle du mapfile
   * @return self
   */
  public function setIsSymDefinitionJson(?bool $is_sym_definition_json): self {
    $this->is_sym_definition_json = $is_sym_definition_json;
    return $this;
  }

  /**
   * @return string|null Base de données
   */
  public function getDatabase(): ?string {
    return $this->database;
  }

  /**
   * @param string $database Base de données
   * @return self
   */
  public function setDatabase(?string $database): self {
    $this->database = $database;
    return $this;
  }

  /**
   * @return string|null Schema
   */
  public function getSchema(): ?string {
    return $this->schema;
  }

  /**
   * @param string $schema Schema
   * @return self
   */
  public function setSchema(?string $schema): self {
    $this->schema = $schema;
    return $this;
  }

  /**
   * @return string|null Nom de la table à utiliser
   */
  public function getPgTable(): ?string {
    return $this->pg_table;
  }

  /**
   * @param string $pg_table Table
   * @return self
   */
  public function setPgTable(?string $pg_table): self {
    $this->pg_table = $pg_table;
    return $this;
  }

  /**
   * @return string|null Champ/colonne identifiant de la table
   */
  public function getIdfield(): ?string {
    return $this->idfield;
  }

  /**
   * @param string $idfield Champ/colonne identifiant de la table
   * @return self
   */
  public function setIdfield(?string $idfield): self {
    $this->idfield = $idfield;
    return $this;
  }

  /**
   * @return string|null Champ/colonne contenant la géométrie
   */
  public function getGeomfield(): ?string {
    return $this->geomfield;
  }

  /**
   * @param string $geomfield Champ/colonne contenant la géométrie
   * @return self
   */
  public function setGeomfield(?string $geomfield): self {
    $this->geomfield = $geomfield;
    return $this;
  }

  /**
   * @return array|null Système de coordonnées de la publication postgres
   */
  public function getCoordsys(): ?array{
    if (!isset($this->coordsys)) {
      return null;
    }
    // return context
    return [
      'coordsys_id' => $this->coordsys->getCoordsysId(),
      'label' => $this->coordsys->getLabel(),
    ];
  }

  /**
   * @param int $coordsys_id Système de coordonnées de la publication postgres
   * @return self
   */
  public function setCoordsysId(?int $coordsys_id): self {
    $this->coordsys_id = $coordsys_id;
    if (isset($this->oEntityManager)) {
      if (empty($coordsys_id)) {
        $this->setCoordsys(null);
      } else {
        $coordsys = $this->oEntityManager->getReference(Coordsys::class, $coordsys_id);
        $this->setCoordsys($coordsys);
      }

    }
    return $this;
  }

  /**
   * @return int
   */
  public function getCoordsysId(): ?int {
    return $this->coordsys_id;
  }

  /**
   * @param Coordsys $coordsys Système de coordonnées de la publication postgres
   * @return self
   */
  public function setCoordsys(?Coordsys $coordsys): self {
    $this->coordsys = $coordsys;
    return $this;
  }

  /**
   * @return array|null Type de géométrie de la publication postgres
   */
  public function getMsGeomType(): ?array{
    if (!isset($this->ms_geomtype)) {
      return null;
    }
    // return context
    return [
      'ms_geomtype_id' => $this->ms_geomtype->getMsGeomTypeId(),
      'label' => $this->ms_geomtype->getLabel(),
    ];
  }

  /**
   * @param int $ms_geomtype_id Type de géométrie de la publication postgres
   * @return self
   */
  public function setMsGeomTypeId(?int $ms_geomtype_id): self {
    $this->ms_geomtype_id = $ms_geomtype_id;
    if (isset($this->oEntityManager)) {
      if (empty($ms_geomtype_id)) {
        $this->setMsGeomType(null);
      } else {
        $ms_geomtype = $this->oEntityManager->getReference(MsGeomType::class, $ms_geomtype_id);
        $this->setMsGeomType($ms_geomtype);
      }
    }
    return $this;
  }

  /**
   * @return int
   */
  public function getMsGeomTypeId(): ?int {
    return $this->ms_geomtype_id;
  }

  /**
   * @param MsGeomType $ms_geomtype Type de géométrie de la publication postgres
   * @return self
   */
  public function setMsGeomType(?MsGeomType $ms_geomtype): self {
    $this->ms_geomtype = $ms_geomtype;
    return $this;
  }

  /**
   * @param bool $bAllowAdminLoginPwd autorise la récupération du login/pwd depuis les properties
   * @param string|null $sSchemaType
   * @return array|null
   */
  public function getDataConnectionFromPublication(bool $bAllowAdminLoginPwd = true, ?string $sSchemaType = null): ?array{
    // récupère la connexion d'après la datasource
    $aData = $this->datasource->getPostgresConnexionData($bAllowAdminLoginPwd, $this->database);

    $aData['field_id'] = $this->idfield;

    $aData['schema'] = $this->schema;
    $aData['table'] = $this->pg_table;

    if ($sSchemaType == 'form' && $this->is_specific_table_form === true) {
      $aData['schema'] = $this->schema_form;
      $aData['table'] = $this->table_form;
    }

    if ($sSchemaType == 'requestor' && $this->is_specific_table_requestor === true) {
      $aData['schema'] = $this->schema_requestor;
      $aData['table'] = $this->table_requestor;
    }

    if (empty($aData['schema']) || empty($aData['table']) || empty($aData['database'])) {
      return null;
    }

    return $aData;
  }

  /**
   * {@inheritdoc}
   */
  public static function getPublicationParams(array $aPublication): array{
    $layerName = MapServerService::cleanLayerName($aPublication['name']);

    $paramsPublications = [];
    // Si la publication est de type géométrie alors params Layers est une suite de trois publication point, ligne, polygone
    if ($aPublication['ms_geomtype']['label'] === "GEOMETRY") {
      if (is_string($aPublication['definition_sym_json'])) {
        $aPublication['definition_sym_json'] = json_decode($aPublication['definition_sym_json'], true);
      } else if (is_array($aPublication['definition_sym_json'])) {
        $aPublication['definition_sym_json'] = $aPublication['definition_sym_json'];
      }
      foreach (['point', 'line', 'polygon'] as $geomtype) {
        if (array_key_exists($geomtype, $aPublication['definition_sym_json'])) {
          $paramsPublications[] = "{$layerName}_{$geomtype}";
        }
      }
    }
    if (empty($paramsPublications)) {
      $paramsPublications = [$layerName];
    }

    $aParamsPublication = [
      'layerType' => 'imagewms',
      'url' => $aPublication['url'],
      'version' => $aPublication['datasource']['flux_version'],
      'params' => [
        'LAYERS' => implode(',', $paramsPublications),
        'VERSION' => '1.3.0',
      ],
      'coordsys_id' => $aPublication['coordsys_id'],
      'definition_sym_json' => $aPublication['definition_sym_json'],
      'definition_sym_mapserver' => $aPublication['definition_sym_mapserver'],
      'is_sym_definition_json' => $aPublication['is_sym_definition_json'],
      'geomfield' => $aPublication['geomfield'],
      'idfield' => $aPublication['idfield'],
      'ms_geomtype' => $aPublication['ms_geomtype'],
      'ms_geomtype_id' => $aPublication['ms_geomtype_id'],
      'database' => $aPublication['database'],
      'schema' => $aPublication['schema'],
      'table' => $aPublication['pg_table'],
      'serverType' => 'mapserver',
      'hidpi' => false,
      'have_select_right' => $aPublication['have_select_right'],
      'have_insert_right' => $aPublication['have_insert_right'],
      'have_update_right' => $aPublication['have_update_right'],
      'have_delete_right' => $aPublication['have_delete_right']
    ];

    return array_merge(parent::getPublicationParams($aPublication), $aParamsPublication);
  }

  /**
   * {@inheritdoc}
   */
  public function jsonSerialize(?array $aRouteConfiguration = null): array{
    $aData = parent::jsonSerialize($aRouteConfiguration);
    $aData["definition_sym_json"] = $this->getDefinitionSymJson();
    $aData["definition_sym_mapserver"] = $this->getDefinitionSymMapserver();
    $aData["is_sym_definition_json"] = $this->getIsSymDefinitionJson();
    $aData["database"] = $this->getDatabase();
    $aData["schema"] = $this->getSchema();
    $aData["pg_table"] = $this->getPgTable();
    $aData["idfield"] = $this->getIdfield();
    $aData["geomfield"] = $this->getGeomfield();
    $aData["coordsys_id"] = $this->getCoordsysId();
    $aData["coordsys"] = $this->getCoordsys();
    $aData["ms_geomtype_id"] = $this->getMsGeomTypeId();
    $aData["ms_geomtype"] = $this->getMsGeomType();
    return $aData;
  }

  /**
   * {@inheritdoc}
   */
  public function getDeserializeMap(): array{
    $aData = parent::getDeserializeMap();
    $aData["definition_sym_json"] = "setDefinitionSymJson";
    $aData["definition_sym_mapserver"] = "setDefinitionSymMapserver";
    $aData["is_sym_definition_json"] = "setIsSymDefinitionJson";
    $aData["database"] = "setDatabase";
    $aData["schema"] = "setSchema";
    $aData["pg_table"] = "setPgTable";
    $aData["idfield"] = "setIdfield";
    $aData["geomfield"] = "setGeomfield";
    $aData["coordsys_id"] = "setCoordsysId";
    $aData["ms_geomtype_id"] = "setMsGeomTypeId";
    return $aData;
  }

}
