import { Injectable } from '@angular/core';
import {
  Router,
  CanActivate,
  ActivatedRouteSnapshot
} from '@angular/router';
import { AuthService } from './auth.service';
import { VitisCheck } from '../../services/utils/vitis-utils';
import { environment } from 'src/environments/environment';

@Injectable({
  providedIn: 'root'
})
export class RoleGuardService implements CanActivate {

  constructor(
    public auth: AuthService,
    public router: Router,
    public vitisCheck: VitisCheck
  ) { }

  /**
   * Redirige l'utilisateur sur la page de login si il n'est pas authentifié
   * @return {boolean} true si l'utilisateur est authentifié, sinon false
   */
  canActivate(route: ActivatedRouteSnapshot): boolean {

    const mode = route.data;
    const userPrivileges = this.auth.getPrivileges();

    if (this.vitisCheck.isString(mode.privilege) || this.vitisCheck.isArray(mode.privilege)) {
      if (mode.privilege === 'public') {
        return true;
      }

      // conversion en array
      const aModePrivilege = this.vitisCheck.isArray(mode.privilege) ? mode.privilege : [mode.privilege];

      // cherche le match de privilège entre les deux tableaux
      const aMatch: Array<string> = userPrivileges.filter(item => aModePrivilege.indexOf(item) > -1);
      // si un privilege a match l'objet est visible
      let bHasRight: boolean = aMatch.length > 0;
      if (!bHasRight) {
        return false;
      }

      if (mode.visible) {
        if (this.vitisCheck.isString(mode.visible)) {
          // remplacement des properties
          const sVisible: string = environment.replacePropertiesInString(mode.visible);
          // évaluation de l'expression
          bHasRight = eval(sVisible);
        }
      }

      return bHasRight;
    } else {
      console.error('Privilege not defined for', mode);
    }

    this.router.navigate(['**'], { queryParamsHandling: 'merge' });
    return false;
  }
}
