import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { VitisCheck, VitisRequest, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';

@Injectable({
  providedIn: 'root'
})

export class CustomTranslationService {
  // Traductions de l'application (fichiers json).
  public oTranslations: any = {};
  // Id des traductions personnalisées.
  public aCustomTranslations: any = [];
  // Instance de la grille de données.
  public dataTableInstance: any;

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private vitisClone: VitisClone,
    private translate: TranslateService
  ) { }

  /**
   * Chargement et application des traductions personnalisées.
   */
   public loadCustomTranslations() {
    return new Promise<void>((resolve, reject) => {
      // Sauvegarde les traductions.
      if (Object.keys(this.oTranslations).length == 0) {
        this.oTranslations = this.vitisClone.clone(this.translate.translations);
      }
      // Chargement des traductions personnalisées.
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: `${environment.properties.app_api_url}/vitis/customtranslations`,
        success: (resp: any) => {
          if (this.vitisCheck.isDefAndNotNull(resp.data) &&
          this.vitisCheck.isDefAndNotNull(resp.data.data)) {
            // Remplace les traductions par les personnalisées.
            const oTranslations = this.vitisClone.clone(this.oTranslations);
            this.aCustomTranslations.length = 0;
            for (let i = 0; i < resp.data.data.length; i++) {
              if (!this.vitisCheck.isDefAndNotNull(oTranslations[resp.data.data[i].lang])) {
                oTranslations[resp.data.data[i].lang] = {};
              }
              oTranslations[resp.data.data[i].lang]
              [resp.data.data[i].translation_id] = resp.data.data[i].text;
              if (this.aCustomTranslations.indexOf(resp.data.data[i].translation_id) === -1) {
                this.aCustomTranslations.push(resp.data.data[i].translation_id);
              }
            }
            // Application des traductions.
            for (const sLang of Object.keys(oTranslations)) {
              this.translate.setTranslation(sLang, oTranslations[sLang]);
            }
            resolve();
          }
        },
        error: (resp: any) => {
          console.error('Error loading custom translations', resp);
        }
      });
    });
  }
}
