import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

/**
 * Type lien
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * text? : texte à afficher dans le lien, si vide vaut la valeur du lien
 * target? : cible du lien (_blank) par dáfaut
 *
 * Exemple:
 * {
 *   "type": "linkurl",
 *   "name": "url",
 *   "label": "URL",
 *   "nb_cols": 12,
 *   "text": "Mon url"
 *   "options": {
 *     "type": "inline"
 *   }
 * }
 */
@Component({
  selector: 'app-linkurl',
  templateUrl: './linkurl.component.html',
  styleUrls: ['./linkurl.component.scss']
})
export class LinkurlComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      return this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }

  /**
   * Return true if the element is a string not empty
   * @param {string} element
   * @returns {Boolean}
   */
  public isStringNotEmpty(element: String): boolean {
    if (!this.vitisCheck.isDefAndNotNull(element)) {
      return false;
    }
    if (!this.vitisCheck.isString(element)) {
      return false;
    }
    if (!(element.length > 0)) {
      return false;
    }
    return true;
  }

}
