import { Component, OnInit, Input, ViewChild, OnChanges, SimpleChanges } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck, VitisClone, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { MapFormreaderComponent } from 'src/app/vitis/formreader/components/map-formreader/map-formreader.component';
import { Feature } from 'ol';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { FormreaderComponent } from '../../formreader.component';
import { MapCoreFormreaderService } from 'src/app/vitis/map-core/services/map-core-fromreader/map-core-formreader.service';
import { Geometry } from 'ol/geom';

declare var $: any;

/**
 * Type map workbench
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * options?
 *
 * Exemple:
 * {
 *   "type": "label",
 *   "name": "mylabel",
 *   "label": "My label",
 *   "nb_cols": 12,
 *   "options": {
 *     "type": "inline"
 *   }
 * }
 */
@Component({
  selector: 'app-map-workbench',
  templateUrl: './map-workbench.component.html',
  styleUrls: ['./map-workbench.component.scss']
})
export class MapWorkbenchComponent extends FormreaderField implements OnInit, OnChanges {

  @ViewChild('map_workbench_subform') oSubFormReader: FormreaderComponent;
  @ViewChild('map_workbench_modal_form') oModalFormReader: FormreaderComponent;

  @ViewChild(MapFormreaderComponent, { static: false, read: MapFormreaderComponent }) set mapFormreaderComponent(mapFormreaderComponent: MapFormreaderComponent) {
    setTimeout(() => {
      this.oMap = mapFormreaderComponent.mapCoreComponent.olMap;
      // Paramètrage de la carte après son initialisation
      this.afterMapInit();
      // Initialisation du formulaire personnalisé de la carte.
      if (this.vitisCheck.isDefAndNotNull(this.oField.custom_form)) {
        if (this.vitisCheck.isDefAndNotNull(this.oField.custom_form.form)) {
          this.initSubform();
        }
      }
      // Evènènements des boutons de la liste.
      this.oFormReader.showAddFeatureModal = () => { this.showAddFeatureModal(); };
      this.oFormReader.deleteSelectedFeature = () => { this.deleteSelectedFeature(); };
    });
  }

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;
  @Input() oFormReader: any;

  public sMapWorkbenchFeatureTypeModalId: string = 'map_workbench_feature_type_modal_' + Date.now();
  public bFormDisabled: boolean = true;

  public aIcons: any = [];
  public sSelectedIcon: string;
  public olSelectedFeature: any = null;
  public sSelectedFeatureType: string = null;
  public aFeatures: any = [];
  public custom_form: any = null;
  public ol: any; // -> à corriger.
  public oMap: any = {};

  // Valeurs par défaut obligatoires
  public featureForm: any = {
    draw: {
      symbol: 'fa-home',
      color: 'red',
      outline_color: 'white',
      size: 4,
      rotation: 0,
      dash: 20
    },
    text: {
      text: '',
      size: 14,
      font: 'Arial',
      color: 'white',
      outline_color: 'black',
      outline_size: 1,
      offsetX: 0,
      offsetY: 10,
      rotation: 0
    },
    attributes: {}
  };

  // Paramètres de la liste.
  public oDataGridField: any = {
    options: {
      select: true,
      searching: false,
      refresh: false,
      paging: false,
      info: false,
      columns: [],
      buttons: [{
        type: 'button',
        name: 'delete-selected-feature',
        label: 'FORM_READER_MAP_WORKBENCH_DELETE_FEATURE',
        class: 'btn-success',
        icon: '',
        event: 'deleteSelectedFeature()'
      }, {
        type: 'button',
        name: 'show-add-feature-modal',
        label: 'FORM_READER_MAP_WORKBENCH_ADD_FEATURE',
        class: 'btn-success',
        icon: '',
        event: 'showAddFeatureModal()'
      }],
      data: []
    }
  };

  public featureFormLastUpdate: number = Date.now();
  public featureFormTimeout: number = 50;
  public lastRowSelected = null;
  public lastRowChange = Date.now();
  public lastRowChangeTimeout = 100;

  private deselectTimestamp = Date.now();

  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    private mapCoreFormreaderService: MapCoreFormreaderService,
    private mapCoreUtilsService: MapCoreUtilsService,
  ) {
    super(translate, vitisCheck);
  }

  ngOnInit() {
    if (!this.vitisCheck.isDefAndNotNull(this.oField.options)) {
      this.oField.options = {};
    }

    // Timeout car un composant map-formreader est utilisé et va surcharger la fonction s'il n'y a pas de timeout
    setTimeout(() => {
      // Fonction getValue
      this.oField.getValue = () => {
        return this.getValue();
      };
    });

    this.initMapWorkbench();
  }

  ngOnChanges(changes: SimpleChanges) {
    //
    if (this.vitisCheck.isDefAndNotNull(changes.featureForm)) {
      this.featureFormChange();
    }
    //
    if (this.vitisCheck.isDefAndNotNull(changes.olSelectedFeature)) {
      this.olSelectedFeatureChange();
    }
  }

  /**
   * Fonction getValue propre à MapWorkbench
   * @returns
   */
  public getValue() {

    // remplit this.aFeatures
    this.loadFeatures_();

    // Set le geojson
    const sValue = this.mapCoreUtilsService.getGeojsonFromFeatures(this.aFeatures, this.oMap);
    return sValue;
  }

  /**
   * .
   */
  public initMapWorkbench() {
    // Colonnes à ajouter à la liste.
    if (this.vitisCheck.isDefAndNotNull(this.oField.attributes_def)) {
      for (let i = 0; i < this.oField.attributes_def.length; i++) {
        this.oDataGridField.options.columns.push({
          data: this.oField.attributes_def[i].name,
          title: this.oField.attributes_def[i].label
        });
      }
    }

    // Custom form ?
    if (this.vitisCheck.isDefAndNotNull(this.oField.custom_form)) {
      // Désactive la possibilité d'ajouter des features
      this.unactiveDrawInteractions();
    }

  }

  /**
   * Paramètrage de la carte après son initialisation.
   */
  public afterMapInit() {

    // Sélection d'une ligne dans la grille de données.
    this.oDataGridField.dataTableInstance.on('select', (e, dt, type, indexes) => {
      if (type === 'row') {
        const data = this.oDataGridField.dataTableInstance.rows(indexes).data()[0];
        this.dataGridSelectionChange(data);
      }
    });
    this.oDataGridField.dataTableInstance.on('deselect', (e, dt, type, indexes) => {
      if (type === 'row') {
        if (this.deselectTimestamp < (Date.now() - 1000)) {
          this.clearSelectedFeature();
        }
      }
    });
  }

  /**
   * Initialisation du formulaire personnalisé de la carte.
   */
  public initSubform() {
    this.oSubFormReader.initForm(this.vitisClone.clone(this.oField.custom_form.form), 'update');
  }

  /**
   * Delete the selected feature from the map and the list
   */
  public deleteSelectedFeature() {
    if (!this.vitisCheck.isDefAndNotNull(this.olSelectedFeature)) {
      vitisAlert.fireToast({
        icon: 'error',
        title: this.translate.instant('FORM_READER_MAP_WORKBENCH_NO_GEOMETRY_SELECTED')
      });
      return null;
    }

    vitisAlert.alert({
      title: this.translate.instant('FORM_READER_MAP_WORKBENCH_GEOMETRY_REMOVAL_CONFIRM'),
      type: 'info',
      showCancelButton: false,
      confirmButtonText: 'OK'
    }).then((result) => {
      if (result.value) {
        this.removeFeature(this.olSelectedFeature);
      }
    });
  }

  /**
   * Affiche la modale d'ajout de feature
   */
  public showAddFeatureModal() {
    const sFieldName = this.oField.custom_form.featureStructure.field;
    const oFeatureTypeSubform = this.vitisClone.clone(this.oField.custom_form.form);

    // Supprime les champs autre que sFieldName
    let formOk = false;
    for (let i = 0; i < oFeatureTypeSubform.update.rows.length; i++) {
      for (let ii = 0; ii < oFeatureTypeSubform.update.rows[i].fields.length; ii++) {
        if (oFeatureTypeSubform.update.rows[i].fields[ii].name !== sFieldName) {
          oFeatureTypeSubform.update.rows[i].fields[ii].visible = false;
          formOk = true;
        }
      }
    }

    // Vérifie que le formulaire soit bon
    if (!formOk) {
      console.error('Field ' + sFieldName + ' not present in the form');
      return null;
    }

    // Exécution du FormReader.
    this.oModalFormReader.initForm(oFeatureTypeSubform, 'update');

    // Unselect la feature courrante
    this.clearSelectedFeature();

    // Affichage de la fenêtre modale du formulaire personnalisé.
    $('#' + this.sMapWorkbenchFeatureTypeModalId).modal('show');
  }

  /**
   * Lance l'interation de dessin sur la carte
   */
  public addFeature() {

    // Vérifie la validité du formulaire
    if (this.oModalFormReader.isFormValid()) {

      const sFieldName = this.oField.custom_form.featureStructure.field;
      const sFieldTypes = this.oField.custom_form.featureStructure.types;

      // Récupère le type de géométrie choisi dans la liste
      const oModalFormValues = this.oModalFormReader.getFormValues();
      let sGeometryType = null;
      if (this.vitisCheck.isDefAndNotNull(oModalFormValues[sFieldName])) {
        if (this.vitisCheck.isDefAndNotNull(sFieldTypes[oModalFormValues[sFieldName]])) {
          if (this.vitisCheck.isDefAndNotNull(sFieldTypes[oModalFormValues[sFieldName]].geometryType)) {
            sGeometryType = sFieldTypes[oModalFormValues[sFieldName]].geometryType;

            // Rétrocompatibilité
            if (sGeometryType === 'line') {
              sGeometryType = 'linestring';
            }
          }
        }
      }

      // Désactive la possibilité d'ajouter des features
      this.unactiveDrawInteractions_();

      // Ajoute la/les nouvelle interaction
      if (this.vitisCheck.isDefAndNotNull(sGeometryType)) {
        this.mapCoreFormreaderService.emitMapFormreaderEvent(this.oField.id, {
          action: 'activeDrawInteraction',
          geomType: sGeometryType,
          removeInteractionAfterDraw: true
        });

        // cache la modale
        $('#' + this.sMapWorkbenchFeatureTypeModalId).modal('hide');
      } else {
        this.oFormReader.vitisAlert.fireToast({
          icon: 'error',
          title: this.oFormReader.services.translate.instant('FORM_READER_MAP_WORKBENCH_MISSING_FEATURE_TYPE')
        });
      }

    }
  }

  /**
   * Supprime une feature
   * @param oFeature
   */
  public removeFeature(oFeature: Feature<Geometry>) {
    this.oField.getMapComponent().removeFeature(oFeature);
    this.clearSelectedFeature();
    setTimeout(() => {
      // remplit this.aFeatures
      this.loadFeatures_();

      // Re-remplit la grille
      this.fillGridFromFeatures_();
    });
  }

  /**
   * Allow to close a modal
   */
  public closeModal() {
    $('#' + this.sMapWorkbenchFeatureTypeModalId).modal('hide');
  }

  /**
   * Unselect the current feature
   * @param {boolean} bremoveInteraction
   */
  public clearSelectedFeature() {
    this.olSelectedFeature = null;
    this.bFormDisabled = true;
    this.oField.getMapComponent().selectDrawFeature(null);
    this.lastRowSelected = null;
    this.deselectTimestamp = Date.now();
    this.oDataGridField.dataTableInstance.rows().deselect();
  }

  /**
   * Unactive the draw interactions of the map
   */
  public unactiveDrawInteractions() {
    // Désactive la possibilité d'ajouter des features
    const aDrawInteractions = ['point', 'line', 'polygon', 'DP', 'DL', 'DPol', 'RA', 'remove_all', 'RO', 'remove'];
    for (let i = 0; i < aDrawInteractions.length; i++) {
      if (this.vitisCheck.isDefAndNotNull(this.oMap)) {
        if (this.vitisCheck.isDefAndNotNull(this.oMap.removeInteraction)) {
          this.oMap.removeInteraction(aDrawInteractions[i]);
        }
      }
      if (this.vitisCheck.isDefAndNotNull(this.oField.map_options.interactions[aDrawInteractions[i]])) {
        this.oField.map_options.interactions[aDrawInteractions[i]] = false;
      }
    }
  }

  /**
   * Set the style of a feature from his type
   * @param oFeature
   */
  public setStyleFromType_(oFeature: any) {

    if (!this.vitisCheck.isDefAndNotNull(oFeature)) {
      return null;
    }

    const sFieldName = this.oField.custom_form.featureStructure.field;
    const sFieldTypes = this.oField.custom_form.featureStructure.types;
    const oAttributes = oFeature.get('attributes');

    if (this.vitisCheck.isDefAndNotNull(oAttributes)) {
      if (this.vitisCheck.isDefAndNotNull(oAttributes[sFieldName])) {
        if (this.vitisCheck.isDefAndNotNull(sFieldTypes[oAttributes[sFieldName]])) {
          if (this.vitisCheck.isDefAndNotNull(sFieldTypes[oAttributes[sFieldName]].style)) {
            // Remplace les balises par leurs valeurs respectives
            const oStyle = this.vitisClone.clone(sFieldTypes[oAttributes[sFieldName]].style);
            for (const key1 in oStyle) {
              if (Object.prototype.hasOwnProperty.call(oStyle, key1)) {
                for (const key2 in oStyle[key1]) {
                  if (Object.prototype.hasOwnProperty.call(oStyle[key1], key2)) {
                    if (this.vitisCheck.isString(oStyle[key1][key2])) {
                      // La chaine commence et se finit par {{ et }} ?
                      if (oStyle[key1][key2].substr(0, 2) === '{{' && oStyle[key1][key2].substr(-2, 2) === '}}') {
                        // Le contenu des balises est un des champs du formulaire ?
                        const sTag = oStyle[key1][key2].substr(2, oStyle[key1][key2].length - 4);
                        if (this.vitisCheck.isDefAndNotNull(oAttributes[sTag])) {
                          oStyle[key1][key2] = oAttributes[sTag];
                        } else {
                          oStyle[key1][key2] = '';
                        }
                      }
                    }
                  }
                }
              }
            }

            // oFeature.style = oStyle;
            this.mapCoreFormreaderService.emitMapFormreaderEvent(this.oField.id, {
              action: 'setFeatureStyle',
              style: oStyle,
              feature: oFeature
            });
          }
        }
      }
    }
  }

  /**
   * Unactive the draw interactions of the map
   */
  public unactiveDrawInteractions_() {
    // Désactive la possibilité d'ajouter des features
    const aDrawInteractions = ['point', 'line', 'polygon', 'DP', 'DL', 'DPol', 'RA', 'remove_all', 'RO', 'remove'];
    for (let i = 0; i < aDrawInteractions.length; i++) {
      if (this.vitisCheck.isDefAndNotNull(this.oMap)) {
        if (this.vitisCheck.isDefAndNotNull(this.oMap.removeInteraction)) {
          this.oMap.removeInteraction(aDrawInteractions[i]);
        }
      }
      if (this.vitisCheck.isDefAndNotNull(this.oField.map_options.interactions[aDrawInteractions[i]])) {
        this.oField.map_options.interactions[aDrawInteractions[i]] = false;
      }
    }
  }

  /**
   * Remplit la grille
   *
   */
  public fillGridFromFeatures_() {
    const aFeatures = this.aFeatures;
    const aGridData = [];
    let oAttributes: any;

    if (this.vitisCheck.isArray(aFeatures)) {
      for (let i = 0; i < aFeatures.length; i++) {
        if (this.vitisCheck.isDefAndNotNull(aFeatures[i].get('attributes'))) {
          oAttributes = aFeatures[i].get('attributes');
        } else {
          oAttributes = {};
        }
        oAttributes.map_workbench_index = this.vitisClone.clone(i);
        aFeatures[i].set('attributes', oAttributes);

        // met à "" les attributs non remplis
        for (let ii = 0; ii < this.oField.attributes_def.length; ii++) {
          const attr = this.oField.attributes_def[ii];
          if (!oAttributes[attr.name]) {
            oAttributes[attr.name] = '';
          }
        }

        // Sélection
        oAttributes.selected = true;
        if (aFeatures[i] === this.olSelectedFeature) {
          oAttributes.selected = true;
        } else {
          oAttributes.selected = false;
        }

        aGridData.push(oAttributes);
      }
    }

    this.oDataGridField.setContent(aGridData);
  }

  /**
   * Update the feature attributes
   */
  public updateFeatureAttributes_() {
    this.olSelectedFeature.set('attributes', this.featureForm.attributes);
  }

  /**
   * Update the feature style
   */
  public updateFeatureStyleFromForm_() {
    this.mapCoreUtilsService.setStyleByJSON(this.olSelectedFeature, this.featureForm);
  }

  /**
   * À appeler lorsqu'une feature est sélectionnée
   */
  public olSelectedFeatureChange() {

    // bFormDisabled
    if (this.vitisCheck.isDefAndNotNull(this.olSelectedFeature)) {
      this.bFormDisabled = false;
    } else {
      this.bFormDisabled = true;
    }
    // sSelectedFeatureType
    if (this.vitisCheck.isDefAndNotNull(this.olSelectedFeature)) {
      switch (this.olSelectedFeature.getGeometry().getType()) {
        case 'Point':
        case 'MultiPoint':
          this.sSelectedFeatureType = 'Point';
          break;
        case 'LineString':
        case 'MultiLineString':
          this.sSelectedFeatureType = 'Line';
          break;
        case 'Polygon':
        case 'MultiPolygon':
          this.sSelectedFeatureType = 'Polygon';
          break;
        default:
          this.sSelectedFeatureType = null;
          break;
      }
    }

    if (this.vitisCheck.isDefAndNotNull(this.olSelectedFeature)) {
      const oAttributes = this.vitisCheck.isDefAndNotNull(this.olSelectedFeature.get('attributes')) ? this.olSelectedFeature.get('attributes') : {};
      this.featureForm.attributes = oAttributes;

      // Mode custom_form
      const sFieldName = this.oField.custom_form.featureStructure.field;
      const sFieldTypes = this.oField.custom_form.featureStructure.types;
      const oFeatureTypeSubform = this.vitisClone.clone(this.oField.custom_form.form);
      if (this.vitisCheck.isDefAndNotNull(oAttributes[sFieldName])) {
        if (this.vitisCheck.isDefAndNotNull(sFieldTypes[oAttributes[sFieldName]])) {
          if (this.vitisCheck.isArray(sFieldTypes[oAttributes[sFieldName]].disableAttrs)) {
            const aDisableAttrs = sFieldTypes[oAttributes[sFieldName]].disableAttrs;
            for (let i = 0; i < oFeatureTypeSubform.update.rows.length; i++) {
              for (let ii = 0; ii < oFeatureTypeSubform.update.rows[i].fields.length; ii++) {
                if (aDisableAttrs.indexOf(oFeatureTypeSubform.update.rows[i].fields[ii].name) !== -1) {
                  oFeatureTypeSubform.update.rows[i].fields[ii].visible = false;
                }
              }
            }
          }
        }
      }

      // Chargement formreader
      this.oSubFormReader.initForm(oFeatureTypeSubform, 'update', oAttributes);

      // Écoute les changements sur le formulaire de gauche pour les appliquer à la feature
      this.oSubFormReader.oFormValues.valueChanges.subscribe((values) => {
        const oFormValues = this.oSubFormReader.getFormValues();
        this.olSelectedFeature.set('attributes', oFormValues);
        this.featureForm.attributes = oFormValues;
        // Modifie le style de la feature
        this.setStyleFromType_(this.olSelectedFeature);
        // Re-remplit la grille
        this.fillGridFromFeatures_();
      });

      // Style
      const oStyle = this.mapCoreUtilsService.getStyleAsJSON(this.olSelectedFeature);
      if (this.vitisCheck.isDefAndNotNull(oStyle.draw)) {
        this.featureForm.draw = oStyle.draw;
      }
      if (this.vitisCheck.isDefAndNotNull(oStyle.text)) {
        this.featureForm.text = oStyle.text;
      }
    } else {
      // Mode dessin libre
      this.featureForm.attributes = {};
      // Mode custom_form
      this.oSubFormReader.clearValues();
    }

    // Grille
    if (!this.vitisCheck.isDefAndNotNull(this.olSelectedFeature)) {
      this.clearSelectedFeature();
    }
  }

  /**
   * À appeler lorsque le formulaire d'une feature change
   */
  public featureFormChange() {
    // Temps minimum entre deux mises à jour
    if ((this.featureFormLastUpdate + this.featureFormTimeout) > Date.now()) {
      return null;
    }
    this.featureFormLastUpdate = Date.now();
    if (!this.vitisCheck.isDefAndNotNull(this.oMap)) {
      return 0;
    }
    if (!this.vitisCheck.isDefAndNotNull(this.oMap.FeatureOverlay)) {
      return 0;
    }
    if (!this.vitisCheck.isDefAndNotNull(this.olSelectedFeature)) {
      return 0;
    }
    this.updateFeatureStyleFromForm_();
    this.updateFeatureAttributes_();

    setTimeout(function () {
      this.oMap.saveFeatures();
    });
  }

  /**
   * À appeler lorsque la sélection datagrid change
   */
  public dataGridSelectionChange(row: any) {
    let olGridSelectedFeature = null;

    // Récupère la feature dans la liste
    for (let i = 0; i < this.aFeatures.length; i++) {
      if (this.aFeatures[i].get('attributes').map_workbench_index === row.map_workbench_index) {
        olGridSelectedFeature = this.aFeatures[i];
        break;
      }
    }

    if (olGridSelectedFeature === this.olSelectedFeature) {
      return;
    }

    this.oField.getMapComponent().selectDrawFeature(olGridSelectedFeature);
  }

  /**
   * Fonction appelée lorsqu'une feature est ajoutée ou modifiée
   *
   * @param oFeature feature OL
   */
  public setFeatureReceived(oRecivedFeature: any) {
    if (oRecivedFeature) {

      const aRecivedFeatures: Array<Feature<Geometry>> = this.vitisCheck.isArray(oRecivedFeature) ? oRecivedFeature : [oRecivedFeature];
      const sFieldName = this.oField.custom_form.featureStructure.field;
      const oModalFormValues = this.oModalFormReader.getFormValues();

      // Renseigne les features
      for (let i = 0; i < aRecivedFeatures.length; i++) {
        const oFeature = aRecivedFeatures[i];

        // Ajout de l'attribut de type
        let oAttributes = oFeature.get('attributes');
        oAttributes = this.vitisCheck.isObject(oAttributes) ? oAttributes : {};
        if (oModalFormValues[sFieldName]) {
          oAttributes[sFieldName] = oModalFormValues[sFieldName];
        }
        oFeature.set('attributes', oAttributes);

        // Set la style en fonction du type
        this.setStyleFromType_(oFeature);
      }

      // Remplit la grille
      setTimeout(() => {
        // remplit this.aFeatures
        this.loadFeatures_();

        // Remplit la grille
        this.fillGridFromFeatures_();
      }, 500);
    }
  }

  /**
   * Fonction appelée lorsqu'une feature est sélectionnée
   *
   * @param oFeature feature OL
   */
  public selectFeatureReciver(oFeature: Feature<Geometry>) {
    if (oFeature) {
      this.olSelectedFeature = oFeature;
      setTimeout(() => {
        this.olSelectedFeatureChange();
      });
    }
  }

  /**
   * remplit this.aFeatures
   */
  private loadFeatures_() {
    this.aFeatures = [];
    const aFeatures = this.oField.getFeatures();
    for (let i = 0; i < aFeatures.length; i++) {
      const olFeature = aFeatures[i];
      if (olFeature.get('isSelectFeatureContour') !== true) {
        this.aFeatures.push(olFeature);
      }
    }
  }
}
