import { Component, OnInit, ViewChild } from '@angular/core';
import { environment } from 'src/environments/environment';
import { FormBuilder, FormGroup, Validators, AbstractControl } from '@angular/forms';
import { VitisRequest, VitisCheck, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Params, ActivatedRoute } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderComponent } from '../../../formreader/formreader.component';
import { FormreaderService } from '../../../formreader/services/formreader.service';

@Component({
  selector: 'app-forgotten-password',
  templateUrl: './forgotten-password.component.html',
  styleUrls: ['./forgotten-password.component.scss']
})
export class ForgottenPasswordComponent implements OnInit {

  public form: FormGroup;

  public params: any;

  public bForm: boolean;
  public sMessage: string;
  public initFormValues: any = null;
  private sDataKey: string;

  @ViewChild(FormreaderComponent) oFormReader: FormreaderComponent;

  constructor(
    private fb: FormBuilder,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private route: ActivatedRoute,
    private translate: TranslateService,
    private formReaderService: FormreaderService,
  ) { }

  ngOnInit() {
    const sUrl = `${environment.properties.app_api_url}/vitis/accounts/fpwd/token`;

    this.route.queryParams.subscribe((oParams: Params) => {

      if (!this.vitisCheck.isDefAndNotNull(oParams.datakey)) {
        this.sMessage = 'MISSING_PARAMS';
      } else {
        this.vitisRequest.ajaxRequest({
          url: sUrl,
          method: 'POST',
          async: false,
          responseType: '',
          data: { datakey: oParams.datakey },
          displayErrorMessage: false,
          success: (resp: any) => {
            this.sDataKey = oParams.datakey;
            this.bForm = true;
            // Chargement et initialisation du formulaire des stats.
            this.loadForm().then((oForm: any) => {

              // Lance le formulaire dans FormReader
              this.oFormReader.initForm(oForm, 'update', this.initFormValues);

            });
          }, error: (resp: any) => {
            const data = JSON.parse(resp.data);
            this.sMessage = data.errorMessage;
          }
        });
      }
    });
  }

  /**
   * Check le formulaire et lance la requête de modification de son mot de passe
   * @returns
   */
  public resetPassword() {

    if (!this.oFormReader.isFormValid()) {
      return;
    }

    const values: any = this.oFormReader.getFormValues();

    if (values.password !== values.password_confirm) {
      this.translate.get('FORM_FPWD_PASSWORD_NOT_MATCH').subscribe((translate) => {
        vitisAlert.fireToast({
          type: 'error',
          title: translate,
        });
      });
      return;
    }

    // Récupération des données du form
    const formData = this.formReaderService.transformToDataForm(values);
    formData.append('datakey', this.sDataKey);
    this.vitisRequest.ajaxRequestPromise({
      method: 'PUT',
      url: environment.properties.app_api_url + '/vitis/accounts/fpwd/reset',
      data: formData
    }).then((resp) => {
      if (resp.data && resp.data.status == 1) {
        this.translate.get('FORM_FPWD_PASSWORD_HAS_CHANGE').subscribe((translate) => {
          vitisAlert.fireToast({
            type: 'success',
            text: translate,
          });
        });
        this.bForm = false;
        this.sMessage = 'FORM_FPWD_PASSWORD_HAS_CHANGE';
      }
    });

  }


  /**
   * Charge le formulaire json d'inscription
   */
  private loadForm() {
    return new Promise((resolve, reject) => {
      // Téléchargement du fichier.
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: 'assets/forms/fpwd/fpwd.json'
      }).then((resp: any) => {
        if (resp.status === 200 && resp.data) {
          const oForm = resp.data;
          if (this.vitisCheck.isDefAndNotNull(oForm) && this.vitisCheck.isDefAndNotNull(oForm.update)) {
            oForm.update.eventContainer = this;
            resolve(oForm);
          } else {
            console.error('Could not get fpwd form from request');
            reject();
          }
        } else {
          reject();
        }
      }).catch((err) => {
        reject();
      });
    });
  }
}
