import { Component, OnInit, Input, Output, EventEmitter, AfterViewInit } from '@angular/core';
import { VitisCheck, VitisRequest, VitisClone, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import moment from 'moment';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';

declare var $: any;

@Component({
  selector: 'app-customfilter',
  templateUrl: './customfilter.component.html',
  styleUrls: ['./customfilter.component.scss']
})
export class CustomfilterComponent implements AfterViewInit {

  @Output() submit = new EventEmitter<number>();

  @Input() oFilterTypes: any;
  @Input() oResults: any;
  @Input() oFilter: any;
  @Input() aColumns: any;
  @Input() sPlaceholder: string;
  @Input() sDefaultFilterColumn: string;

  public aSugestionColumns: Array<any> = [];
  public aSugestionValues: Array<any> = [];
  public oSearchColumn: any;
  public sElemId = 'workspacelist-customfilter-input-' + Math.floor(Math.random() * 1000000);

  public aCompareOperators: Array<string> = ['~', '=', '>', '<'];

  // Utilisé pour dates
  public oDatePickerField: any = {
    type: 'datetime',
    name: 'filterdate',
    label: '',
  };
  public showDateTimePicker: boolean = true;

  constructor(
    private vitisCheck: VitisCheck,
    private authService: AuthService,
    private vitisRequest: VitisRequest,
    private vitisClone: VitisClone,
    private translate: TranslateService,
  ) { }

  ngAfterViewInit() {

    // Input de recherche
    const input: any = document.getElementById(this.sElemId);

    // Événements clavier
    input.addEventListener('keyup', (event: any) => {

      // Touche "Enter"
      if (event.keyCode === 13) {

        // Supprime l'action en cours si besoin
        event.preventDefault();

        // Valide la recherche
        this.onValidSearchString(input.value);
      }
    });
  }


  /**
   * Fonction lancée au changement de la recherche
   * @param  {string} searchValue
   * @returns void
   */
  public onSearchChange(searchValue: string): void {

    // Suggestions de colonnes
    this.aSugestionColumns = this.getColumnSuggestions_(searchValue);

    // Suggestions de valeurs
    this.aSugestionValues = this.getValueSuggestions_(searchValue);

    // Force l'affichage de la dropdown
    this.showSuggestionsDorpdown();
  }

  /**
   * Fonction lancée à la validation de la recherche
   * @param  {string} searchValue
   * @returns void
   */
  public onValidSearchString(searchValue: string): void {
    if (this.vitisCheck.isString(searchValue)) {

      // Colonne
      this.oSearchColumn = this.extractSearchColumn_(searchValue);

      if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn)) {
        if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.translateTitle)
          && this.vitisCheck.isDefAndNotNull(this.oSearchColumn.field)) {

          // Valeur
          const sFilterValue = this.extractSearchValue_(searchValue);

          // operateur
          const sOperator = this.parseSearch_(searchValue).operator;

          this.addFilterNode(this.oSearchColumn, sOperator, sFilterValue);
        }
      } else {
        // Cas où la colonne ne soit pas renseignée : prend la colonne favorite
        const oSearchColumn = this.getColumnByField_(this.sDefaultFilterColumn);
        if (this.vitisCheck.isDefAndNotNull(oSearchColumn)) {
          this.oSearchColumn = oSearchColumn;

          let sCompareOperator = '~';
          switch (oSearchColumn.searchType) {
            case 'boolean':
            case 'date':
            case 'number':
            case 'strict':
              sCompareOperator = '=';
              break;
            default:
              break;
          }

          this.addFilterNode(this.oSearchColumn, sCompareOperator, searchValue);
        } else {
          // Cas où il n'y a pas non plus de colonne favorite
          this.aSugestionColumns = this.getColumnSuggestions_('');
          this.showSuggestionsDorpdown();

          // Message d'erreur
          this.translate.get('VITIS_WORKSPACELIST_FILTER_NO_COLUMN').subscribe((translation) => {
            vitisAlert.fireToast({
              type: 'error',
              title: translation
            });
          });
        }
      }
    }
  }

  /**
   * Crée le noeud de filtre et l'ajoute à this.oFilter.nodes
   * @param  {any} oSearchColumn
   * @param  {String} sCompareOperator
   * @param  {String} sValue
   * @returns void
   */
  public addFilterNode(oSearchColumn: any, sCompareOperator: String, sValue: String): void {

    if (this.vitisCheck.isDefAndNotNull(oSearchColumn)
      && this.vitisCheck.isString(sCompareOperator)
      && this.vitisCheck.isString(sValue)) {

      const oFilterNode = {
        column: oSearchColumn,
        compareOperator: sCompareOperator,
        value: sValue,
        displayValue: sValue
      };

      // Gesiton tableau de valeur avec libellé
      if (this.vitisCheck.isArray(oSearchColumn.searchSuggestions)) {
        for (let i = 0; i < oSearchColumn.searchSuggestions.length; i++) {
          const suggestion = oSearchColumn.searchSuggestions[i];
          if (this.vitisCheck.isDefAndNotNull(suggestion.label) &&
            this.vitisCheck.isDefAndNotNull(suggestion.value)) {
            if (suggestion.value == sValue) {
              oFilterNode.displayValue = suggestion.label;
            }
          }
        }
      }

      // Gestion dates
      if (oSearchColumn.searchType === 'date') {
        let sDateFormat = 'DD/MM/YYYY';
        const oUser = this.authService.getUserInfos();
        if (this.vitisCheck.isDefAndNotNull(oUser)) {
          if (this.vitisCheck.isDefAndNotNull(oUser.formatdate_id)) {
            sDateFormat = oUser.formatdate_id;
          }
        }
        oFilterNode.displayValue = moment.unix(Number(sValue)).format(sDateFormat + ' HH:mm');
      }

      // Gestion numériques
      if (oSearchColumn.searchType === 'number') {
        if (!this.vitisCheck.isNumber(Number(sValue))) {
          console.error('value is not a number');
          // Message d'erreur
          this.translate.get('VITIS_WORKSPACELIST_FILTER_VALUE_NOT_NUMBER').subscribe((translation) => {
            vitisAlert.fireToast({
              type: 'error',
              title: translation
            });
          });
          return;
        }
      }

      this.oFilter.nodes.push(oFilterNode);

      // Vide le champ de recherche
      const input: any = document.getElementById(this.sElemId);
      input.value = '';

      // Applique le filtre à la grille
      setTimeout(() => {
        this.oSearchColumn = undefined;
        this.submit.emit();
      });

      // Cache la dropdown
      $('.customfilter-input').find('.dropdown-menu.show').dropdown('toggle');
    }
  }

  /**
   * Supprime un noeud du filtre
   * @param  {any} oNode
   * @returns void
   */
  public removeFilterNode(oNode: any): void {

    const iNodeIndex = this.oFilter.nodes.indexOf(oNode);

    if (iNodeIndex !== -1) {
      this.oFilter.nodes.splice(iNodeIndex, 1);
    }

    // Applique le filtre à la grille
    this.submit.emit();
  }

  /**
   * Valide la colonne de recherche
   * @param  {any} oSearchColumn
   */
  public setSearchColumn(oSearchColumn: any) {

    const input: any = document.getElementById(this.sElemId);

    // Nouvelle valeur de chaine
    let sNewValue: string;
    switch (oSearchColumn.searchType) {
      case 'boolean':
      case 'date':
      case 'number':
      case 'strict':
        sNewValue = this.translate.instant(oSearchColumn.translateTitle) + ' = ';
        break;
      default:
        sNewValue = this.translate.instant(oSearchColumn.translateTitle) + ' ~ ';
        break;
    }

    // Renseigne le nom de la colonne dans la valeur
    input.value = sNewValue;

    // Focus l'input
    input.focus();

    // Affiche la dropdown
    setTimeout(() => {
      input.click();
    });

    // Lance event de changement
    this.onSearchChange(sNewValue);
  }

  /**
   * Valide la colonne de valeur
   * @param  {String} sSearchValue
   */
  public setSearchValue(sSearchValue: any) {

    if (this.vitisCheck.isNumber(sSearchValue)) {
      sSearchValue = String(sSearchValue);
    }

    if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn)) {
      if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.translateTitle)) {
        if (this.vitisCheck.isString(sSearchValue)) {

          const input: any = document.getElementById(this.sElemId);
          const sOperator = this.parseSearch_(input.value).operator;

          // Renseigne le nom de la colonne dans la valeur
          const sNewValue = this.translate.instant(this.oSearchColumn.translateTitle) + ' ' + sOperator + ' ' + sSearchValue;
          input.value = sNewValue;

          // Lance event de validation
          this.onValidSearchString(sNewValue);

          // Nettoie les listes après avoir fait un filtre
          this.clearLists_();
        }
      }
    }
  }

  /**
   * Force l'affichage de la dropdown de suggestions
   * @returns void
   */
  public showSuggestionsDorpdown(): void {

    setTimeout(() => {

      const inputElem: any = $('#' + this.sElemId);
      const isOpen = inputElem.attr('aria-expanded') === 'true';

      if (!isOpen) {
        inputElem.dropdown('toggle');
      }
    });
  }

  /**
   * Utilisé pour les dates
   * @param iDate
   */
  public onDateSubmit(iDate: any) {

    const year = moment.unix(iDate).year();
    const month = moment.unix(iDate).month() + 1;
    const day = moment.unix(iDate).date();
    const hour = moment.unix(iDate).hour();
    const minute = moment.unix(iDate).minute();

    if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn)) {
      if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.translateTitle)
        && this.vitisCheck.isDefAndNotNull(this.oSearchColumn.field)) {

        const input: any = document.getElementById(this.sElemId);
        const sOperator = this.parseSearch_(input.value).operator;

        if (sOperator === '>' || sOperator === '<') {
          this.addFilterNode(this.oSearchColumn, sOperator,
            String(moment(`${year}-${month}-${day} 00:00:00`).utc().unix()));
        } else {
          if (hour === 0 &&
            minute === 0) {
            this.addFilterNode(this.oSearchColumn, '>',
              String(moment(`${year}-${month}-${day} 00:00:00`).utc().unix()));
            this.addFilterNode(this.oSearchColumn, '<',
              String(moment(`${year}-${month}-${day} 23:59:59`).utc().unix()));
          } else {
            this.addFilterNode(this.oSearchColumn, '>',
              String(moment(`${year}-${month}-${day} ${hour}:${minute}`, `YYYY-MM-DD HH:mm`).format('x')));
            this.addFilterNode(this.oSearchColumn, '<',
              String(moment(`${year}-${month}-${day} ${hour}:${minute + 1}`, `YYYY-MM-DD HH:mm`).format('x')));
          }
        }
      }
    }
  }

  /**
   * Retourne les colonnes qui matchent avec searchValue
   * @param  {any} searchValue
   * @returns Array
   */
  private getColumnSuggestions_(searchValue: any): Array<any> {

    const aSugestionColumns = [];

    if (this.vitisCheck.isString(searchValue)) {

      this.oSearchColumn = this.extractSearchColumn_(searchValue);

      // Ajoute la liste des colonnes dans aSugestionColumns
      if (this.aColumns) {
        for (let i = 0; i < this.aColumns.length; i++) {

          // colonne
          const column = this.aColumns[i];
          let columnTitle = this.translate.instant(column.translateTitle);

          // Patch si le nom de la colonne est un chiffre
          if (!this.vitisCheck.isString(searchValue)) {
            searchValue = String(searchValue);
          }
          if (!this.vitisCheck.isString(columnTitle)) {
            columnTitle = String(columnTitle);
          }

          // Ajout de la colonne dans la liste aSugestionColumns
          if (this.vitisCheck.isString(columnTitle)) {

            // Si la colonne est recherchable
            if (column.searchable !== false) {

              // Si la recherche coïncide
              if (columnTitle.toLowerCase().search(searchValue.toLowerCase()) !== -1) {
                aSugestionColumns.push(column);
              }
            }
          }
        }
      }
    }

    return aSugestionColumns;
  }

  /**
   * Retourne les valeurs de la vue actuelle qui matchent avec searchValue
   * @param  {any} searchValue
   * @returns Array
   */
  private getValueSuggestions_(searchValue: any): Array<any> {

    let aSugestionValues = [];

    if (this.vitisCheck.isString(searchValue)) {

      this.oSearchColumn = this.extractSearchColumn_(searchValue);

      // Ajoute la liste des valeurs
      if (this.oSearchColumn) {

        // Type booléen
        if (this.oSearchColumn.searchType === 'boolean') {
          this.oSearchColumn.searchSuggestions = [{
            value: 'true',
            label: 'FORM_YES'
          }, {
            value: 'false',
            label: 'FORM_NO'
          }];
        }

        // Type date
        if (this.oSearchColumn.searchType === 'date') {
          this.oSearchColumn.searchSuggestions = false;
        }

        if (this.oSearchColumn.field) {
          if (this.oSearchColumn.searchSuggestions !== false) {
            if (this.vitisCheck.isArray(this.oSearchColumn.searchSuggestions)) {
              aSugestionValues = this.oSearchColumn.searchSuggestions;
            } else {
              if (this.vitisCheck.isDefAndNotNull(searchValue)) {
                if (this.oResults) {
                  if (this.oResults.data) {
                    for (let i = 0; i < this.oResults.data.length; i++) {
                      const oRow = this.oResults.data[i];

                      if (this.vitisCheck.isDefAndNotNull(oRow[this.oSearchColumn.field])) {

                        let sRowValue = oRow[this.oSearchColumn.field];

                        // Patch si la valeur est un chiffre
                        if (!this.vitisCheck.isString(sRowValue)) {
                          sRowValue = String(sRowValue);
                        }

                        if (sRowValue.length > 0) {

                          // Valeur recherchée (car searchValue vaut "[Colonne] = ...")
                          let sSearchValueValue = this.extractSearchValue_(searchValue);

                          // Patch si la valeur est un chiffre
                          if (!this.vitisCheck.isString(sSearchValueValue)) {
                            sSearchValueValue = String(sSearchValueValue);
                          }

                          // Ajoute au tableau si il n'y est pas déjà
                          if (sRowValue.toLowerCase().search(sSearchValueValue.toLowerCase()) !== -1) {
                            if (aSugestionValues.indexOf(sRowValue) === -1) {
                              aSugestionValues.push(sRowValue);
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }

          if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService)) {
            if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.resource) &&
              this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.value)) {
              let aParams: any = {};
              if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.params)) {
                aParams = this.oSearchColumn.searchWebService.params;
              }

              if (!this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.label)) {
                this.oSearchColumn.searchWebService.label = this.oSearchColumn.searchWebService.value;
              }
              aSugestionValues = [];
              this.vitisRequest.ajaxRequest({
                method: 'GET',
                url: environment.properties.app_api_url + '/' + this.oSearchColumn.searchWebService.resource,
                ajaxLoader: false,
                params: aParams,
                success: (resp: any) => {
                  if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {
                    this.aSugestionValues = [];
                    resp.data.data.forEach((element: any) => {
                      if (this.vitisCheck.isDefAndNotNull(element[this.oSearchColumn.searchWebService.value]) &&
                        this.vitisCheck.isDefAndNotNull(element[this.oSearchColumn.searchWebService.label])) {
                        this.aSugestionValues.push({
                          label: element[this.oSearchColumn.searchWebService.label],
                          value: element[this.oSearchColumn.searchWebService.value]
                        });
                      }
                    });
                    // mémorisation pour affichage de l'alias
                    this.oSearchColumn.searchSuggestions = [...this.aSugestionValues];
                  }
                },
                error: (err: any) => {
                  console.error('error while loading values :', err);
                }
              });
            }
          }

        }
      }
    }

    return aSugestionValues;
  }

  /**
   * Récupère la colonne depuis ce qu'il y a écrit dans le champ de recherche
   * @param  {string} searchValue
   * @returns any
   */
  private extractSearchColumn_(searchValue: string): any {

    let oSearchColumn: any;

    if (this.vitisCheck.isString(searchValue)) {

      const oSearch = this.parseSearch_(searchValue);

      if (this.vitisCheck.isDefAndNotNull(oSearch)) {
        if (this.vitisCheck.isDefAndNotNull(oSearch.column)) {
          oSearchColumn = this.getColumnByTitle_(oSearch.column);
        }
      }
    }

    return oSearchColumn;
  }

  /**
   * Récupère la valeur depuis ce qu'il y a écrit dans le champ de recherche
   * @param  {string} searchValue
   * @returns any
   */
  private extractSearchValue_(searchValue: string): any {

    let sReturn: any;

    if (this.vitisCheck.isString(searchValue)) {

      const oSearch = this.parseSearch_(searchValue);

      if (this.vitisCheck.isDefAndNotNull(oSearch)) {
        if (this.vitisCheck.isDefAndNotNull(oSearch.value)) {
          sReturn = oSearch.value;
        }
      }
    }

    return sReturn;
  }

  /**
   * Parse la chaine de recherche et retourne un objet contenant column, operator, value
   * @param  {string} sSearch
   * @returns any
   */
  private parseSearch_(sSearch: string): any {

    let oReturn: any;
    let aSearchValue: any;
    let sOperator: any;
    let sColumn: String;
    let sValue: String;

    // Teste l'ensembles des opérateurs et extrait un tableau
    for (let i = 0; i < this.aCompareOperators.length; i++) {

      const tmpOperator = this.aCompareOperators[i];
      const tmpTest = sSearch.split(tmpOperator);

      if (tmpTest.length > 1) {
        aSearchValue = tmpTest;
        sOperator = this.aCompareOperators[i];
        sColumn = tmpTest[0];
        sValue = tmpTest[1];

        const aValue = [];
        for (let ii = 1; ii < tmpTest.length; ii++) {
          aValue.push(tmpTest[ii]);
        }
        sValue = aValue.join(this.aCompareOperators[i]);
        break;
      }
    }

    if (this.vitisCheck.isArray(aSearchValue)) {
      if (aSearchValue.length > 1) {
        oReturn = {
          column: sColumn.trim(),
          operator: sOperator,
          value: sValue.trim()
        };
      }
    }

    return oReturn;
  }

  /**
   * Recherche dans les colonnes du mode celle dont le titre correspond à sColumnTitle
   * @param  {String} sColumnTitle
   * @returns any
   */
  private getColumnByTitle_(sColumnTitle: String): any {

    if (this.aColumns) {

      // Pour chaque colonne
      for (let i = 0; i < this.aColumns.length; i++) {
        const column = this.aColumns[i];

        if (this.translate.instant(column.translateTitle).trim() === sColumnTitle.trim()) {
          return this.vitisClone.clone(column);
        }
      }
    }
  }

  /**
   * Recherche dans les colonnes du mode celle dont le nom correspond à sColumnField
   * @param  {String} sColumnTitle
   * @returns any
   */
  private getColumnByField_(sColumnField: String): any {
    if (this.aColumns) {
      for (let i = 0; i < this.aColumns.length; i++) {
        const column = this.aColumns[i];
        if (column.field === sColumnField) {
          return this.vitisClone.clone(column);
        }
      }
    }
  }

  /**
   * Nettoie les listes après avoir fait un filtre
   */
  private clearLists_() {
    this.aSugestionColumns.length = 0;
    this.aSugestionValues.length = 0;
  }
}
