import { Pipe, PipeTransform } from '@angular/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

@Pipe({
  name: 'filesize'
})
export class FileSizePipe implements PipeTransform {

  constructor(
    private vitisCheck: VitisCheck
  ) { }

  /**
   * LOG de base pour un nombre = ln(nombre)/ln(base)
   * @param  {number} number
   * @param  {number} base la base de logorithme
   * @returns number
   */
  public getBaseLog(number: number, base: number): number {
    return Math.log(number) / Math.log(base);
  }

  /**
   * Formate la taille du fichier
   * @param  {number} iSize la taille en octet
   * @param  {string='b'} sUnit l'unité (bytes par defaut)
   * @returns string
   */
  transform(iSize: number, sUnit: string = 'b'): string {
    if (!this.vitisCheck.isNumber(iSize)) {
      return '' + iSize;
    }
    // Tableau des multiples d'unité
    const aUnits: Array<string> = ['', 'k', 'M', 'G', 'T', 'P', 'E'];
    // Logarithme de base 1000 pour la taille d'un fichier arrondie au plus petit nombre
    let iRoundLogarithm = Math.floor(this.getBaseLog(iSize, 1000));
    // Test si la valeur est trop grande l'affiche en exaoctet
    if (iRoundLogarithm >= aUnits.length) {
      iRoundLogarithm = 6;
    }
    const sExtension: string = aUnits[iRoundLogarithm] + sUnit;
    // Retourne la taille formaté
    return (iSize / Math.pow(1000, iRoundLogarithm)).toFixed(1) + ' ' + sExtension;
  }
}
