import { Component, OnInit } from '@angular/core';
import { StudioService } from '../../services/studio.service';
import { VitisCheck, VitisClone, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { TranslateService } from '@ngx-translate/core';

declare const $: any;

@Component({
  selector: 'app-attr-list',
  templateUrl: './attr-list.component.html',
  styleUrls: ['./attr-list.component.scss']
})
export class StudioAttrListComponent implements OnInit {

  public aFormRows: any;
  public oEditedField: any;
  public oSelectedField: any = {};
  public bDisplayLines = false;
  public draggedField = null;
  public draggedDest = null;
  public sSelectedFormType = null;

  private sFormName_: string;
  private oFormDefinition_: any;

  constructor(
    private studioService: StudioService,
    private formReaderService: FormreaderService,
    private translate: TranslateService,
    private vitisCheck: VitisCheck,
    private vitisClone: VitisClone
  ) { }

  ngOnInit() {

    // Observe la modification du formulaire
    this.studioService.getStudioForm().subscribe((oStudioForm: any) => {
      const aFormRows = this.formReaderService.getFormRows(oStudioForm, this.sFormName_);
      this.aFormRows = aFormRows;
      this.oFormDefinition_ = oStudioForm;
    });

    // Observe la validation de oEditedField
    this.studioService.getStudioEditedField().subscribe((oEditedField: any) => {
      this.oEditedField = oEditedField;

      // Déselectionne les éléments après l'édition
      if (oEditedField === null) {
        this.oSelectedField = {};
        setTimeout(() => {
          this.unselectFields_();
        });
      }
    });

    // Observe la modification de sFormName
    this.studioService.getStudioFormName().subscribe((sFormName: any) => {
      this.sFormName_ = sFormName;
      this.aFormRows = this.formReaderService.getFormRows(this.oFormDefinition_, this.sFormName_);
    });

    // Changement de type de formulaire (default, custom, published...)
    this.studioService.getStudioFormType().subscribe((sFormType: string) => {
      this.sSelectedFormType = this.vitisClone.clone(sFormType);
    });
  }

  /**
   * Retourne la définition du type de champ
   * @param sFieldType
   */
  public getFieldTypeDef(sFieldType: string) {
    return this.studioService.getFieldTypeDef(sFieldType);
  }

  /**
   * Édite le champ
   * @param oField
   */
  public editField(oField: any) {
    // Paramètre "studio_id_" obligatoire en édition.
    if (!this.vitisCheck.isDefAndNotNull(oField.studio_id_)) {
      oField.studio_id_ = this.studioService.getUniqueId_();
    }

    this.unselectFields_();
    oField.studio_selected_ = true;
    this.studioService.setStudioEditedField(oField);
  }

  /**
   * Déplace un élément de formulaire.
   * @param  {any} oField
   * @param  {string} sDirection
   * @returns void
   */
  public moveFieldUpDown(oField: any, sDirection: string) {

    let iDestRowIndex: number;
    let iDestFieldIndex: number = 0;
    let bCreateNewLine: boolean = false;

    const aFormRows = this.vitisClone.clone(this.aFormRows);
    const iRowIndex = this.studioService.findRowIndex(aFormRows, oField);
    const iFieldIndex = this.studioService.findFieldIndex(aFormRows, oField);
    const oRow = aFormRows[iRowIndex];

    if (!this.bDisplayLines) {
      if (sDirection === 'up') {
        iDestRowIndex = iRowIndex - 1;
      } else if (sDirection === 'down') {
        iDestRowIndex = iRowIndex + 2;
      }
    } else {

      // Ligne avec un unique champ
      if (this.vitisCheck.isArray(oRow.fields) &&
        oRow.fields.length === 1) {

        if (sDirection === 'up') {
          iDestRowIndex = iRowIndex - 1;
          iDestFieldIndex = this.aFormRows[iDestRowIndex].fields.length;
        } else if (sDirection === 'down') {
          iDestRowIndex = iRowIndex + 1;
          iDestFieldIndex = 0;
        }
      } else {
        if (sDirection === 'up') {

          // Si l'élément est le premier de sa ligne
          if (iFieldIndex === 0) {
            bCreateNewLine = true;
            iDestRowIndex = iRowIndex - 1;
          } else {
            iDestRowIndex = iRowIndex;
            iDestFieldIndex = iFieldIndex - 1;
          }

        } else if (sDirection === 'down') {

          // Si l'élément est le dernier de sa ligne
          if (iFieldIndex === this.aFormRows[iRowIndex].fields.length - 1) {
            bCreateNewLine = true;
            iDestRowIndex = iRowIndex + 1;
          } else {
            iDestRowIndex = iRowIndex;
            iDestFieldIndex = iFieldIndex + 2;
          }
        }
      }
    }

    this.moveField(oField, iDestRowIndex, iDestFieldIndex, bCreateNewLine);
  }

  /**
   * Bouge un champ à l'endroit désiré
   * @param oField Champ
   * @param iDestRowIndex numéro de ligne
   * @param iDestFieldIndex numéro de colonne
   */
  public moveField(oField: any, iDestRowIndex: number, iDestFieldIndex: number = 0, bCreateNewLine: boolean = false) {

    let aFormRows = this.vitisClone.clone(this.aFormRows);
    let iFromRowIndex = this.studioService.findRowIndex(aFormRows, oField);

    const iFromFieldIndex = this.studioService.findFieldIndex(aFormRows, oField);
    const oFromRow = aFormRows[iFromRowIndex];

    if (bCreateNewLine) {
      if (iFromRowIndex < iDestRowIndex) {
        this.createNewLine_(aFormRows, iDestRowIndex);
      } else {
        iDestRowIndex++;
        iFromRowIndex++;
        this.createNewLine_(aFormRows, iDestRowIndex);
      }
    }

    // Mode sans lignes
    if (!this.bDisplayLines) {

      // Ligne d'origine avec un unique champ
      if (this.vitisCheck.isArray(oFromRow.fields) &&
        oFromRow.fields.length === 1) {

        if (iFromRowIndex < iDestRowIndex) {
          aFormRows = this.arrayMove_(aFormRows, iFromRowIndex, iDestRowIndex - 1);
        } else {
          aFormRows = this.arrayMove_(aFormRows, iFromRowIndex, iDestRowIndex);
        }
      } else {

        // Crée la nouvelle ligne avec le champ à l'intérieur
        this.createNewLine_(aFormRows, iDestRowIndex, [oField]);

        if (iFromRowIndex < iDestRowIndex) {

          // Supprime l'élément de la ligne actuelle
          if (this.vitisCheck.isArray(aFormRows[iFromRowIndex].fields)) {
            aFormRows[iFromRowIndex].fields.splice(iFromFieldIndex, 1);
          }

          // Redimentionne les éléments
          this.resizeRowElements_(aFormRows, iFromRowIndex);
          this.resizeRowElements_(aFormRows, iDestRowIndex);

        } else {

          // Supprime l'élément de la ligne actuelle
          if (this.vitisCheck.isArray(aFormRows[iFromRowIndex + 1].fields)) {
            aFormRows[iFromRowIndex + 1].fields.splice(iFromFieldIndex, 1);
          }

          // Redimentionne les éléments
          this.resizeRowElements_(aFormRows, iFromRowIndex + 1);
          this.resizeRowElements_(aFormRows, iDestRowIndex);
        }
      }
    } else {

      // Ajoute le champ sur la nouvelle ligne
      aFormRows[iDestRowIndex].fields.splice(iDestFieldIndex, 0, oField);

      // Redimentionne la nouvelle ligne
      this.resizeRowElements_(aFormRows, iDestRowIndex);

      // Si les deux champs ne sont pas sur la même ligne
      if (iFromRowIndex !== iDestRowIndex) {

        // Supprime le champ de son ancienne ligne
        aFormRows[iFromRowIndex].fields.splice(iFromFieldIndex, 1);

        // Redimentionne l'ancienne ligne
        this.resizeRowElements_(aFormRows, iFromRowIndex);

        // Supprime la ligne si elle est vide
        if (aFormRows[iFromRowIndex].fields.length === 0) {
          aFormRows.splice(iFromRowIndex, 1);
        }
      } else {

        // Supprime le champ de la ligne
        if (iFromFieldIndex < iDestFieldIndex) {
          aFormRows[iFromRowIndex].fields.splice(iFromFieldIndex, 1);
        } else {
          aFormRows[iFromRowIndex].fields.splice(iFromFieldIndex + 1, 1);
        }
      }
    }

    // Met à jour le formulaire
    this.updateFormRows_(aFormRows);
  }

  /**
   * Événement appellé lors du mouseenter sur un champ
   * @param oField
   */
  public onMouseEnter(oField: any) {
    if (this.draggedField === null) {
      this.unselectFields_();
      oField.studio_hover_ = true;
    }
  }

  /**
   * Événement appellé lors du mouseleave sur un champ
   * @param oField
   */
  public onMouseLeave(oField: any) {
    oField.studio_hover_ = false;
  }

  /**
   * Événement au commencement du drag & drop
   * @param oField
   */
  public onDragStart(oField: any) {
    oField.studio_hover_ = false;
    this.draggedField = oField;
  }

  /**
   * Événement au relachement de drag & drop
   * @param oFromField
   */
  public onDragEnd(oFromField: any) {

    const aFormRows = this.vitisClone.clone(this.aFormRows);
    const iDestRowIndex = this.studioService.findRowIndex(aFormRows, this.draggedDest);
    const iDestFieldIndex = this.studioService.findFieldIndex(aFormRows, this.draggedDest);

    this.moveField(oFromField, iDestRowIndex, iDestFieldIndex);

    this.draggedField = null;
    this.draggedDest = null;
  }

  /**
   * Évnément au passage par dessus
   * @param oField
   */
  public onDragEnter(oField: any) {
    this.draggedDest = oField;

    // Sélectionne le champ survolé
    this.unselectFields_();
    oField.studio_hover_ = true;
  }

  /**
   * Événement à la fin du passage
   * @param oField
   */
  public onDragLeave(oField: any) { }

  /**
   * Événement appellé lors du click sur un champ
   * @param oField
   */
  public selectField(oField: any) {
    if (this.draggedField === null) {
      oField.studio_selected_ = true;
      if (oField.name != this.oSelectedField.name) {
        this.oSelectedField.studio_selected_ = false;
        this.oSelectedField = oField;
      } else {
        this.oSelectedField = {};
      }
    }
  }

  /**
   * Suppression d'un champ
   * @param oField
   */
  public removeField(oField: any) {

    // Modale de confirmation.
    this.translate.get(['STUDIO_DELETE_FIELD_WARNING']).subscribe((translations: any) => {
      vitisAlert.alert({
        icon: 'warning',
        html: translations['STUDIO_DELETE_FIELD_WARNING'],
        showCancelButton: true
      }).then(result => {
        if (result.value) {

          let removeRow = false;
          const aFormRows = this.vitisClone.clone(this.aFormRows);
          const iRowIndex = this.studioService.findRowIndex(aFormRows, oField);
          const iFieldIndex = this.studioService.findFieldIndex(aFormRows, oField);
          const oRow = aFormRows[iRowIndex];

          if (this.vitisCheck.isArray(oRow.fields)) {
            if (oRow.fields.length == 1) {
              removeRow = true;
            }

            // Supprime oField de la ligne
            oRow.fields.splice(iFieldIndex, 1);

            // Supprime la ligne
            if (removeRow) {
              aFormRows.splice(iRowIndex, 1);
            }

            // Met à jour le formulaire
            this.updateFormRows_(aFormRows);
          } else {
            console.error('Row not valid', oRow);
          }
        }
      });
    });
  }

  /**
   * Affiche le gestionnaire d'onglets
   */
  public displayTabsManager() {
    $('#studio_tabs_modal').modal('show');
  }

  /**
   * Met à jour le formulaire en settant les formRows
   * @param aFormRows
   */
  private updateFormRows_(aFormRows: Array<any>) {

    const oNewForm = this.vitisClone.clone(this.oFormDefinition_);

    if (oNewForm[this.sFormName_] &&
      oNewForm[this.sFormName_].rows) {
      oNewForm[this.sFormName_].rows = aFormRows;
      this.studioService.setStudioIsBeingModified(true);
      this.studioService.setStudioForm(oNewForm);
    }
  }

  /**
 * Déplace un élément dans un tableau
 * @param arr
 * @param oldIndex
 * @param newIndex
 */
  private arrayMove_(arr: Array<any>, oldIndex: number, newIndex: number) {

    if (newIndex >= 0 && newIndex < arr.length) {

      const elem = arr[oldIndex];

      // Ajoute
      if (oldIndex < newIndex) {
        arr.splice(newIndex + 1, 0, elem);
        arr.splice(oldIndex, 1);
      } else {
        arr.splice(newIndex, 0, elem);
        arr.splice(oldIndex + 1, 1);
      }
    }
    return arr;
  }

  /**
   * Crée une nouvelle ligne
   * @param aFormRows
   * @param iRowNumber
   * @param aFields
   */
  private createNewLine_(aFormRows: Array<any>, iRowNumber: number, aFields = []) {

    const newRow = {
      fields: aFields
    };
    aFormRows.splice(iRowNumber, 0, newRow);
  }

  /**
   * Redimensionne les éléments d'une même ligne
   * @param aFormRows
   * @param iRowNumber
   */
  private resizeRowElements_(aFormRows: Array<any>, iRowNumber: number) {
    if (this.vitisCheck.isArray(aFormRows[iRowNumber].fields)) {
      const fieldSize = Math.trunc(12 / aFormRows[iRowNumber].fields.length);
      for (let i = 0; i < aFormRows[iRowNumber].fields.length; i++) {
        const oField = aFormRows[iRowNumber].fields[i];
        oField.nb_cols = fieldSize;
      }
    }
  }

  /**
   * Désélectionne l'ensemble des champs
   */
  private unselectFields_() {
    for (let i = 0; i < this.aFormRows.length; i++) {
      const oRow = this.aFormRows[i];
      if (this.vitisCheck.isArray(oRow.fields)) {
        for (let ii = 0; ii < oRow.fields.length; ii++) {
          const oField = oRow.fields[ii];
          if (oField.name != this.oSelectedField.name) {
            oField.studio_selected_ = false;
            oField.studio_hover_ = false;
          }
        }
      }
    }
  }
}
