import { Component, OnInit, Input } from '@angular/core';
import { VitisCheck, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { environment } from 'src/environments/environment';
import moment from 'moment-timezone';

@Component({
  selector: 'app-wklist-date',
  templateUrl: './date.component.html',
  styleUrls: ['./date.component.scss']
})
export class WklistDateComponent implements OnInit {

  @Input() timestamp: any;
  @Input() type: string; // date, date_hour, hour, seconds, from_now
  @Input() align: string;

  public date: string;

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private auth: AuthService
  ) { }

  ngOnInit() {
    if (this.vitisCheck.isNumber(this.timestamp)) {
      const oUserInfos = this.auth.getUserInfos();

      switch (this.type) {
        case 'date':
          this.date = moment.unix(this.timestamp).tz(oUserInfos['timezone_id']).format(oUserInfos['formatdate_id']);
          break;
        case 'from_now':
          // chargement des traduction pour moment
          // peut avoir des conséquences sur le formatage des formats L, LL, LLL, LTS, ...
          moment.locale(environment.properties.language);
          this.date = moment.unix(this.timestamp).tz(oUserInfos['timezone_id']).fromNow();
          break;
        case 'hour':
          this.date = moment.unix(this.timestamp).format('HH:mm:ss');
          break;
        case 'seconds':
          this.date = moment.utc(this.timestamp * 1000).format('HH:mm:ss');
          break;
        default:
          this.date = moment.unix(this.timestamp).tz(oUserInfos['timezone_id']).format(oUserInfos['formatdate_id'] + ' HH:mm:ss');
          break;
      }
      // Supprime l'espace entre le "T" et l'heure (ISO 8601).
      this.date = String(this.date).replace('T ', 'T');
    } else {
      this.date = this.timestamp;
    }
  }
}
