import { Component, EventEmitter, OnInit, ViewChild } from '@angular/core';
import { environment } from 'src/environments/environment';
import { VitisCheck, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DataGridComponent } from 'src/app/vitis/data-grid/data-grid.component';
import { DataGridOptions } from 'src/app/vitis/data-grid/data-grid.model';
import { WorkspaceRelationListOptions } from '../workspacerelationlist.model';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { ActivatedRoute } from '@angular/router';
import { WorkspacerelationlistService } from '../workspacerelationlist.service';
import { takeUntil } from 'rxjs/operators';

declare var $: any;

@Component({
  selector: 'app-data-source-grid',
  templateUrl: './data-source-grid.component.html',
  styleUrls: ['./data-source-grid.component.scss']
})
export class DataSourceGridComponent implements OnInit {

  @ViewChild(DataGridComponent, { static: true }) oDataGrid: DataGridComponent;

  // Options de la grille de données.
  public oDataGridOptions: DataGridOptions = {
    datatableOptions: {},
    datagridOptions: WorkspaceRelationListOptions
  };

  // Largeur de page
  public pageLength: Number = environment.properties.rows_per_page;

  // Instance de la grille de données.
  public dataTableInstance: any;

  // Mode en cours
  public mode: any;
  public oParams: any;

  // Filtre en cours
  public oFilter: any = {
    type: 'custom',
    nodes: []
  };

  // filtre des ids déjà présent dans la ressource
  idsInRessource: Number[] = [];

  // Types de filtre disponible
  public oFilterTypes: any = {
    custom: 'VITIS_WORKSPACELIST_FILTER_CUSTOM',
    form: 'VITIS_WORKSPACELIST_FILTER_FORM'
  };

  // Résultat
  public oResults: any = {
    recordsTotal: 0,
    recordsFiltered: 0,
    data: []
  };

  // Paramètres passés dans l'URL
  public urlParams_: any;

  // Paramètres à passer à la requête
  public datatableRessourceParams_: any;

  public onRefresh = new EventEmitter<any>();

  // Variables privées
  private clickedCell_: any;

  // Modale affichée si aucun enregistrement dans la liste.
  public bDisplayNoRecordModal: boolean = false;
  public sNoRecordLabel: string = 'VITIS_WORKSPACELIST_NO_RECORD';
  public sInsertButtonLabel: string = 'VITIS_WORKSPACELIST_INSERT_BTN';

  constructor(
    private route: ActivatedRoute,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService,
    private auth: AuthService,
    private serviceWKR: WorkspacerelationlistService
  ) { }

  ngOnInit(): void {
    // Permet à oDataGrid de connaitre les colonnes etc..
    this.oDataGrid.oDataGridOptions = this.oDataGridOptions;

    // Mode en cours d'utilisation
    this.route.data.subscribe(mode => {
      this.mode = mode;

      // options app.json
      if (mode.options) {
        this.oDataGridOptions.datagridOptions = mode.options.source;
      }

      // Initialise la grille de données
      this.initDataGrid();
    });

    // Paramètres de la route en cours d'utilisation
    this.route.params.subscribe(oParams => {
      this.oParams = oParams;
    });

    const self = this;
    // Permet de récupérer le statut de la modale et de get les données si la modale est ouverte
    $('#workspacerelationlist-form-modal').on('classChange', function () {
      if ($('#workspacerelationlist-form-modal').hasClass('show')) {
        self.dataTableInstance.ajax.reload();
      }
    });
  }

  /**
   * Initialise la grille de données
   */
  public initDataGrid() {
    /**
     * Calcule la hauteur de la table
     */
    const calcDataTableHeight = () => {
      if (this.vitisCheck.isNumber(this.mode.options.height)) {
        return this.mode.options.height;
      } else {
        const modalBodyHeight = $('#modal-body').height();
        const filterHeight = $('.grid-filter').height();
        const tableHeight = modalBodyHeight - filterHeight;
        return tableHeight;
      }
    };

    /**
     * Réajuste les dimensions de la table
     */
    const autoAjustTables = () => {
      if ($('#' + this.oDataGrid.tableId).length) {
        this.dataTableInstance.columns.adjust();
        $('#' + this.oDataGrid.tableId + '_wrapper').find('.dataTables_scrollBody').css('height', calcDataTableHeight());
      }
    };
    const self = this;
    // Ordre
    let order = [2, 'desc'];

    // Offset
    let displayStart = 0;

    let rowId = '';

    // Options app.json
    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.order_by)) {
        order = this.oDataGridOptions.datagridOptions.order_by;
      }
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.id_field)) {
        rowId = this.oDataGridOptions.datagridOptions.id_field;
      }
    }

    // Paramètres URL
    if (this.urlParams_ &&
      this.urlParams_.wkgstart &&
      this.urlParams_.wkglength) {
      displayStart = Number(this.urlParams_.wkgstart);
    }

    // Paramètres de la grille de données.
    this.oDataGridOptions.datatableOptions = {
      select: {
        style: 'multi',
        selector: 'tr'
      },
      processing: true,
      serverSide: true,
      searching: false,
      lengthMenu: this.getLengthMenu_(),  // Tableau choix du nombre d'éléments par page
      pageLength: this.getPageLength_(),  // Nombre d'éléments par page
      order: order, // Ordre
      displayStart: displayStart,  // Offset
      scrollY: calcDataTableHeight(),
      scrollX: true,
      scrollCollapse: true,
      rowId: rowId,
      selected: 'selected',
      ajax: (data: any, callback: any, settings: any) => {
        this.getResourceData_(data).then((oResponse: any) => {

          // Set oResults
          this.oResults = oResponse;

          // Vide viewContainer
          this.oDataGrid.viewContainer.clear();

          // Remplace les données
          callback(this.oResults);

          // event onRefresh
          if (this.onRefresh) {
            this.onRefresh.emit();
          }
        }).catch(() => {
          this.oResults = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };
          callback(this.oResults);
        });
      }
    };

    // Initialise les boutons
    this.initButtons();

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions) &&
      this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.columns)) {

      // Traductions
      const aTranslations = [];
      for (let i = 0; i < this.oDataGridOptions.datagridOptions.columns.length; i++) {
        const column = this.oDataGridOptions.datagridOptions.columns[i];
        aTranslations.push(column.title);
      }
      this.translate.get(aTranslations).subscribe((res: any) => {

        // Enregistre le titre traduit
        for (let i = 0; i < this.oDataGridOptions.datagridOptions.columns.length; i++) {
          const column = this.oDataGridOptions.datagridOptions.columns[i];
          column.translateTitle = res[column.title];
        }

        // Initialise la table
        setTimeout(() => {
          // Colonnes de la liste (définies dans le mode).
          this.oDataGridOptions.datatableOptions.columns = this.getParsedColumns_();

          // Création de la liste.
          this.dataTableInstance = this.oDataGrid.initTable(this.oDataGridOptions);

          this.dataTableInstance.on('draw.dt', () => {
            this.serviceWKR.modalDataSourceGridObservable
              .subscribe(statut => {
                // Cas pour mettre à jour les header dans le datatable car il est ouvert dans une modale
                if (statut === true) {
                  const table = $('#' + this.oDataGrid.tableId).DataTable();
                  table.columns.adjust();
                }
              });

            // Format les champs avec un template
            this.oDataGrid.formatFields(this.oDataGridOptions);
            // Réajuste les dimensions de la table
            autoAjustTables();
          });
          // Click event (permet de récupérer la cellule cliquée)
          this.dataTableInstance.on('click', 'td', function () {
            self.clickedCell_ = self.dataTableInstance.cell(this);
          });

        });
      });
    }
  }

  /**
   * Parse les params envoyés par DataTables pour créer des paramètres interprétables par l'API Vitis
   * @param  {any} params
   * @returns any
   */
  private getRequestParams_(params: any): any {

    const requestParams: any = {};

    // Order
    if (this.vitisCheck.isDefAndNotNull(params.order)) {
      if (this.vitisCheck.isDefAndNotNull(params.order[0])) {
        if (this.vitisCheck.isString(params.order[0].dir)) {
          requestParams.sort_order = params.order[0].dir.toUpperCase();
        }
        if (this.vitisCheck.isNumber(params.order[0].column)) {
          const iColumnId = params.order[0].column;
          if (this.vitisCheck.isDefAndNotNull(params.columns[iColumnId])) {
            if (this.vitisCheck.isString(params.columns[iColumnId].data)) {
              requestParams.order_by = params.columns[iColumnId].data;
            }
          }
        }
      }
    }

    // Limit
    if (this.vitisCheck.isNumber(params.length)) {
      requestParams.limit = params.length;
    }

    // Offset
    if (this.vitisCheck.isNumber(params.start)) {
      requestParams.offset = params.start;
    }

    // Filter
    const filters = this.getFilters_();
    if (this.vitisCheck.isDefAndNotNull(filters) && filters.operators.length) {
      requestParams.filter = filters;
    }

    // Attributs
    if (this.vitisCheck.isDefAndNotNull(params.columns)) {
      requestParams['attributs'] = params.columns.filter(
        (aColumn: any) => this.vitisCheck.isString(aColumn.data)
      ).map(
        (aColumn: any) => aColumn.data
      ).join('|');
    }

    return requestParams;
  }

  /**
   * Récupère les donéées au format DataTables
   * @param  {any} params
   * @returns Promise
   */
  public getResourceData_(params: any): Promise<any> {
    return new Promise((resolve, reject) => {

      // Mémorise les paramètres
      this.datatableRessourceParams_ = params;

      this.oDataGridOptions.loadingData_ = true;

      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: environment.properties.app_api_url + '/' + this.mode.options.ressource + '/not_included',
        ajaxLoader: false,
        params: this.getRequestParams_(params),
        success: (resp: any) => {
          this.oDataGridOptions.loadingData_ = false;
          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (this.vitisCheck.isDefAndNotNull(resp.data.status === 1)) {
              if (this.vitisCheck.isArray(resp.data.data)) {
                oReturn.data = resp.data.data;
              }
              if (this.vitisCheck.isDefAndNotNull(resp.data.total_row_number)) {
                oReturn.recordsTotal = resp.data.list_count;
                oReturn.recordsFiltered = resp.data.total_row_number;
              }
            }
          }
          resolve(oReturn);
        },
        error: (err: any) => {
          console.error('error while loading values :', err);

          this.oDataGridOptions.loadingData_ = false;

          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          resolve(oReturn);
        }
      });
    });
  }

  /**
   * Génére l'url pour récupérer les données
   * @returns {string}
   */
  protected getRessourceUrl(): string {
    let sRessourceUrl = this.oDataGridOptions.datagridOptions.ressource;

    // Gestion du parent en cas de workspacelist dans une section
    if (this.mode.options &&
      this.mode.options.parent_id_field &&
      this.oParams &&
      this.oParams.id) {

      if (sRessourceUrl.indexOf(`:${this.mode.options.parent_id_field}`) !== -1) {
        // Remplacement dans l'URL
        sRessourceUrl = sRessourceUrl.replace(`:${this.mode.options.parent_id_field}`, String(this.oParams.id));
      }
    }

    return environment.properties.app_api_url + '/' + sRessourceUrl;
  }

  /**
   * Retourne le tableau contenant le nombre de lignes affichables
   */
  private getLengthMenu_(): Array<Number> {

    const lengthMenu = [5, 10, 20, 50, 100];

    if (this.oDataGridOptions &&
      this.oDataGridOptions.datagridOptions &&
      this.oDataGridOptions.datagridOptions.max_rows) {

      const iMaxRows = this.oDataGridOptions.datagridOptions.max_rows;

      for (let i = lengthMenu.length - 1; i >= 0; i--) {
        if (lengthMenu[i] > iMaxRows) {
          lengthMenu.splice(i, 1);
        }
      }
    }

    return lengthMenu;
  }

  /**
   * Retourne le nombre d'éléments à afficher
   */
  private getPageLength_(): Number {
    let pageLength = this.pageLength;

    // Paramètres URL
    if (this.urlParams_ &&
      this.urlParams_.wkgstart &&
      this.urlParams_.wkglength) {
      pageLength = Number(this.urlParams_.wkglength);
    }

    // Concordance avec le tableau de choix
    const aLengthMenu = this.getLengthMenu_();
    if (aLengthMenu.indexOf(pageLength) === -1) {
      if (pageLength > aLengthMenu[aLengthMenu.length - 1]) {
        pageLength = aLengthMenu[aLengthMenu.length - 1];
      } else {
        pageLength = 20;
      }
    }

    return pageLength;
  }

  /**
   * Récupère les colonnes définies dans le mode au format BootstrapTable
   * @returns Array
   */
  private getParsedColumns_(): Array<any> {

    const aColumns: any = [{
      targets: 0,
      data: null,
      defaultContent: '',
      orderable: false,
      className: 'select-checkbox',
      width: '40px'
    }];

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {

      // Colonnes du mode
      if (this.vitisCheck.isArray(this.oDataGridOptions.datagridOptions.columns)) {
        const aUserPrivileges = this.auth.getPrivileges();
        for (let i = 0; i < this.oDataGridOptions.datagridOptions.columns.length; i++) {
          const column = this.oDataGridOptions.datagridOptions.columns[i];
          // Privilège obligatoire pour afficher la colonne.
          if (this.vitisCheck.isDefAndNotNull(column.privilege)) {
            if (aUserPrivileges.indexOf(column.privilege) == -1) {
              continue;
            }
          }
          aColumns.push({
            title: column.translateTitle,
            data: column.field,
            name: column.field,
            visible: column.visible === false ? false : true,
            type: 'string',
            render: (data: any, type: any, row: any, meta: any) => {
              if (column.template) {
                data = '<field-render column_number="' + i + '">' + encodeURIComponent(JSON.stringify(row)) + '</field-render>';
              }
              return data;
            },
            orderable: column.sortable,
            // Alignement du contenu de la colonne.
            className: 'table-align-' + (column.align ? column.align : 'left')
          });
        }
      }
    }

    return aColumns;
  }

  /**
   * Récupère les filtres du mode
   */
  public getParsedFilters_() {
    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.filter)) {
      const oFilter = this.oDataGridOptions.datagridOptions.filter;

      if (this.vitisCheck.isDefAndNotNull(oFilter.lang) && oFilter.lang === '') {
        oFilter.lang = environment.properties.language;
      }

      const oGivenFilter = {
        relation: 'AND',
        operators: []
      };
      for (const key in oFilter) {
        if (oFilter.hasOwnProperty(key)) {
          oGivenFilter.operators.push({
            column: key,
            compare_operator: '=',
            value: oFilter[key]
          });
        }
      }

      return oGivenFilter;
    }
  }

  /**
   * Récupère les filtres à appliquer
   */
  private getFilters_() {
    const defaultFilter = this.getParsedFilters_();
    let currentFilter = this.oFilter.value;

    if (this.vitisCheck.isDefAndNotNull(defaultFilter) && this.vitisCheck.isDefAndNotNull(currentFilter)) {
      currentFilter.operators = currentFilter.operators.concat(defaultFilter.operators);
    } else if (this.vitisCheck.isDefAndNotNull(defaultFilter)) {
      currentFilter = defaultFilter;
    } else if (this.vitisCheck.isEmpty(currentFilter) && this.idsInRessource.length) {
      // Ajoute le filtre pour ne pas avoir doublons ressource/source
      currentFilter = {
        relation: 'AND',
        operators: [{
          'column': this.oDataGridOptions.datagridOptions.id_field,
          'compare_operator': 'NOT IN',
          'value': this.idsInRessource
        }]
      };
    }
    return currentFilter;
  }

  /**
   * Ajout d'un élément sur la ressource originale
   */
  public addSourceToRessource(): void {
    const aIds = this.oDataGrid.getSelectedIds_();

    if (!(aIds.length > 0)) {
      vitisAlert.alert({
        type: 'error',
        title: this.translate.instant('LIST_DELETE_NO_SELECTION'),
      });
    } else {

      if (this.vitisCheck.isDefAndNotNull(this.mode.options)) {
        if (this.vitisCheck.isDefAndNotNull(this.mode.options.ressource)) {

          // Chemin de la ressource REST
          let sRessourceURL = environment.properties.app_api_url;
          sRessourceURL += '/' + this.mode.options.ressource;

          this.vitisRequest.ajaxRequestPromise({
            method: 'POST',
            url: sRessourceURL,
            data: {
              data: aIds.reduce((acc, curr) => {
                return [...acc, {
                  [this.oDataGridOptions.datagridOptions.id_field]: curr,
                  [this.oDataGridOptions.datagridOptions.id_ressource_field]: this.oParams['id']
                }];
              }, [])
            }
          }).then((resp) => {
            if (this.vitisCheck.isDefAndNotNull(resp.data) &&
              this.vitisCheck.isDefAndNotNull(resp.data.status === 1)) {
              this.translate.get('LIST_ADD_OK').subscribe((translate) => {
                vitisAlert.fireToast({
                  type: 'success',
                  title: translate
                });
              });
              this.dataTableInstance.ajax.reload();
              // Ferme la modal et actualise la liste ressource
              this.serviceWKR.modalDataSourceGrid.next(false);
            } else {
              console.error('Error getting source datas');
            }
          });
        }
      }
    }
  }

  /**
   * Initialise les boutons
   * @returns void
   */
  public initButtons(): void {
    const aButtons: any = [];

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      // Bouton insert
      if (this.oDataGridOptions.datagridOptions.insert === true) {
        aButtons.unshift({
          type: 'button',
          class: 'btn-primary',
          label: 'LIST_BUTTON_ADD',
          icon: 'fa fa-plus',
          event: () => {
            this.addSourceToRessource();
          }
        });
      }

      // Autres boutons
      if (this.vitisCheck.isArray(this.oDataGridOptions.datagridOptions.buttons)) {
        // Ajoute les boutons
        for (let i = 0; i < this.oDataGridOptions.datagridOptions.buttons.length; i++) {
          const button = this.oDataGridOptions.datagridOptions.buttons[i];
          this.oDataGrid.initButtonEvent_(button, this);
          this.oDataGrid.initButtonDisplay_(button, this);
          aButtons.unshift(button);
        }
      }
    }

    this.oDataGridOptions.buttons = aButtons;
  }

  /**
   * Applique le filtre à la grille
   */
  public applyFilter(): void {

    // Filtre formaté pour la requête API
    this.oFilter.value = this.getAPIFilterFromNodes_(this.oFilter.nodes);

    // Rafraichit la vue
    if (this.dataTableInstance &&
      this.dataTableInstance.ajax) {
      this.dataTableInstance.ajax.reload();
    }

    // Mémorisation du filtre dans l'url
    this.setUrlParams_();
  }

  /**
   * Lit le tableau de noeuds et retourne un filtre compatible avec l'API Vitis
   * @param  {Array<any>} aFilterNodes
   * @returns any
   */
  public getAPIFilterFromNodes_(aFilterNodes: Array<any>): any {
    // Initialise à chaque changement de filtre pour ne pas récupérer les doublons
    const oFilter = {
      relation: 'AND',
      operators: []
    };

    if (this.vitisCheck.isArray(aFilterNodes)) {
      if (aFilterNodes.length > 0) {

        for (let i = 0; i < aFilterNodes.length; i++) {
          const oNode = aFilterNodes[i];

          if (this.vitisCheck.isDefAndNotNull(oNode.column)) {
            const oNodeFilter: any = {};

            if (this.vitisCheck.isDefAndNotNull(oNode.column.field)
              && this.vitisCheck.isDefAndNotNull(oNode.compareOperator)
              && this.vitisCheck.isDefAndNotNull(oNode.value)) {
              oNodeFilter.column = oNode.column.field;
              oNodeFilter.compare_operator = oNode.compareOperator;
              oNodeFilter.value = oNode.value;

              if (oNode.compareOperator === '~') {
                oNodeFilter.compare_operator = 'LIKE';
                oNodeFilter.compare_operator_options = {
                  'case_insensitive': true
                };
                oNodeFilter.value = '%' + oNodeFilter.value + '%';

              }
              // modification de la colonne à utiliser pour filtrer
              if (this.vitisCheck.isDefAndNotNull(oNode.column.searchColumn)) {
                oNodeFilter.column = oNode.column.searchColumn;
              }

              oFilter.operators.push(oNodeFilter);
            }
          }
        }
      }
    }

    return oFilter;
  }

  /**
   * Écrit le filtre dans l'URL
   */
  public setUrlParams_() {

    /**
     * Paramètres de l'url
     */
    const queryParams: any = {};

    // Filtre
    queryParams.q = null;
    if (this.oFilter) {
      const sQuery = this.getQueryByFilter_(this.oFilter);
      if (sQuery) {
        queryParams.q = sQuery;
      }
    }

    // Page
    if (this.datatableRessourceParams_) {

      // Valeurs vides
      queryParams.wkgstart = null;
      queryParams.wkglength = null;

      // Valeurs réelles
      if (this.datatableRessourceParams_.start > 0 &&
        this.datatableRessourceParams_.length > 0) {
        queryParams.wkgstart = this.datatableRessourceParams_.start;
        queryParams.wkglength = this.datatableRessourceParams_.length;
      }
    }
  }

  /**
   * Retourne le filtre au format URL
   * @param oFilter
   */
  public getQueryByFilter_(oFilter: any): string {

    let sQuery: any = null;
    if (oFilter.nodes) {
      for (let i = 0; i < oFilter.nodes.length; i++) {
        const node = oFilter.nodes[i];
        if (i === 0) {
          sQuery = '';
        } else {
          sQuery += '||';
        }

        let sNodeFilter = `${node.column.field}|${node.compareOperator}|${node.value}`;
        if (node.column.searchWebService && node.value != node.displayValue) {
          sNodeFilter += `|${node.displayValue}`;
        }
        sQuery += encodeURIComponent(sNodeFilter);
      }
    }

    return sQuery;
  }

}
