/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * affiche une modale pour aider l'utilisateur
   * @return {void}
   */
  scope.showHelp = function () {
    services.translate.get(['MODAL_HELP_CONTENT', 'FORM_RETURN_BTN_DOMAIN']).subscribe((translations) => {
      services.vitisAlert.alert({
        'width': 750,
        'html': translations['MODAL_HELP_CONTENT'],
        'showCancelButton': true,
        'cancelButtonText': translations['FORM_RETURN_BTN_DOMAIN'],
        'showConfirmButton': false,
      })
    });
  }

  /**
   * Import groups + enable users auto refresh 
   */
  scope.syncEnableGroups = function () {
    scope.importGroups(true);
  }
  /**
   * Import groups + enable users auto refresh 
   */
  scope.syncDisableGroups = function () {
    scope.importGroups(false);
  }

  /**
   * Import groups
   */
  scope.importGroups = function (bAutoRefresh) {
    const rows = scope.oDataGrid.dataTableInstance.rows({
      selected: true
    }).data();
    let data = {
      'length': rows.length,
      'auto_refresh': bAutoRefresh
    };
    for (let index = 0; index < data['length']; index++) {
      data[index] = rows[index];
    }
    scope.sendData(data);
  }

  /*
   * Envois des données
   */
  scope.sendData = function (data) {
    const domain_id = scope.oParams['id'];
    services.translate.get([
      'LIST_NO_SELECTION',
      'SUCCESSFUL_OPERATION',
      'FORM_VALIDATION_ERROR',
      'CONFIRM_IMPORT_DOMAIN_USERS_VITIS_USERS',
      'SUCCESFULL_IMPORTATION_ACTIVE_DIRECTORY_USERS_VITIS_USERS',
      'FORM_YES',
      'FORM_NO',
      'IMPORTEDS',
      'IMPORTED'
    ]).subscribe((translation) => {
      services.vitisRequest.ajaxRequestPromise({
        method: 'POST',
        url: env.properties.app_api_url + '/vitis/domains/' + domain_id + '/groups/import',
        data: data
      }).then((resp) => {
        if (resp['data'] && resp['data']['status'] != 1) {
          services.vitisAlert.fireToast({
            type: 'error',
            title: translation['FORM_VALIDATION_ERROR']
          });
          return;
        }
        if (resp['data']['list_count'] === resp['data']['total_row_number']) {
          services.vitisAlert.fireToast({
            type: 'success',
            title: translation['SUCCESSFUL_OPERATION']
          });
          return;
        }
        let imported = 'IMPORTEDS';
        if (resp['data']['list_count'] <= 1) {
          imported = 'IMPORTED';
        }
        services.vitisAlert.fireToast({
          type: 'warning',
          title: resp['data']['list_count'] + '/' + resp['data']['total_row_number'] + ' ' + translation[imported]
        });
      }).catch((error) => {
        console.log(error)
      });

    });
  }


  scope.initPgHba = function (oFormValues) {
    scope.processPgHba_(oFormValues);
  }

  scope.updatePgHba = function() {
    setTimeout(()=>{
      oFormValues = scope.getFormValues()
      scope.processPgHba_(oFormValues);
    })
  }

  scope.processPgHba_ = function (oFormValues) {
    const oField = scope.getFormField('pg_hba')

    // Fonction getPre
    const login_role = oFormValues['login_role'];

    if (!login_role || login_role === '') {
      return;
    }

    const server = services.vitisCheck.isDefAndNotNull(oFormValues['server']) ? oFormValues['server'] : 'server';
    let sBaseChain = 'host\tall\t+' + login_role + '\t127.0.0.1/32\tldap ldapserver=' + server + ' ldapport=' + oFormValues['port'];
    if (oFormValues['encrypttype'] === 'ssl') {
      sBaseChain += ' ldapscheme=ldaps';
    }
    if (oFormValues['encrypttype'] === 'tls') {
      sBaseChain += ' ldaptls=1';
    } else {
      sBaseChain += ' ldaptls=0';
    }
    let sPgHba = '';
    if (oFormValues['type'] === 'AD') {
      let sPrefix = (oFormValues['prefix']) ? oFormValues['prefix'] : '';
      sBaseChain += ' ldapprefix="' + sPrefix + '"';
      sPgHba = sBaseChain + ' ldapbindpasswd="password"';
    } else {

      let dn_search_user = 'ldapbasedn="DC=dn,DC=search,DC=user"';
      if (services.vitisCheck.isDefAndNotNull(oFormValues['dn_search_user'])) {
        dn_search_user = 'ldapbasedn ="' + oFormValues['dn_search_user'] + '"';
      }
      const ldapsearchfilter = 'ldapsearchfilter="(' + oFormValues['uid'] + '=$username)"';
      const ldapbinddn = 'ldapbinddn="' + oFormValues['login'] + '"';
      sPgHba = sBaseChain + ' ' + dn_search_user + ' ' + ldapsearchfilter + ' ' + ldapbinddn + ' ldapbindpasswd="password"';
    }

    setTimeout(() => {
      oField.setValue(sPgHba);
    }, 100)
  }

  /**
   * Sync groups
   */
  scope.syncGroups = function () {
    const domain_id = scope.oParams['id'];
    const rows = scope.oDataGrid.dataTableInstance.rows({
      selected: true
    }).data();
    let data = {};
    data['length'] = rows.length;
    for (let index = 0; index < data['length']; index++) {
      data[index] = rows[index];
    }
    services.translate.get([
      'LIST_NO_SELECTION',
      'SUCCESSFUL_OPERATION',
      'FORM_VALIDATION_ERROR',
      'CONFIRM_IMPORT_DOMAIN_USERS_VITIS_USERS',
      'SUCCESFULL_IMPORTATION_ACTIVE_DIRECTORY_USERS_VITIS_USERS',
      'FORM_YES',
      'FORM_NO',
      'IMPORTEDS',
      'IMPORTED'
    ]).subscribe((translation) => {
      services.vitisRequest.ajaxRequestPromise({
        method: 'POST',
        url: env.properties.app_api_url + '/vitis/domains/' + domain_id + '/groups/sync',
        data: data
      }).then((resp) => {
        if (resp['data'] && resp['data']['status'] != 1) {
          services.vitisAlert.fireToast({
            type: 'error',
            title: translation['FORM_VALIDATION_ERROR']
          });
          return;
        }
        if (resp['data']['list_count'] === resp['data']['total_row_number']) {
          services.vitisAlert.fireToast({
            type: 'success',
            title: translation['SUCCESSFUL_OPERATION']
          });
          //scope.router.navigate(['/dashboard/users/groups']);
          return;
        }
        let imported = 'IMPORTEDS';
        if (resp['data']['list_count'] <= 1) {
          imported = 'IMPORTED';
        }
        services.vitisAlert.fireToast({
          type: 'warning',
          title: resp['data']['list_count'] + '/' + resp['data']['total_row_number'] + ' ' + translation[imported]
        });
      }).catch((error) => {
        console.log(error)
      });

    });
  }

}