# app-struct.ts

Ce fichier est contenu dans le répertoire `conf/` du dépôt de l'application, toute la structure applicative affichée dans le navigateur est
définie dans ce fichier.

## Fonctionnement

La structure de ce fichier est de type `AppConfig` sous la forma suivante
```js
export class AppConfig {
  translate: Array<AppTranslateConfig>; // Configuration fichiers de traduction
  name: string; // Nom de l'application
  logo: AppLogoConfig; // Configuration du logo
  favicon?: string; // Favicon
  modes: Array<AppModeConfig>; // Modes du menu latéral, instancie automatiquement les routes associées
  headers?: Array<any>;
  footers?: Array<any>;
  routes?: Array<AppModeConfig>; // Permet de définir des routes additionnelles à celles créées par les modes,
  widgets?: Array<AppModeConfig>; // Permet de définir des routes additionnelles à celles créées par les modes,
  // en cas de conflit sur le path la fonction prendra en priorité celle définie dans le paramètre routes
}
```

À l'aide de ce fichier nous pourrons déterminer les menus à afficher dans notre application, les logos, widget etc.

Ce fichier est chargé dans l'environnement utilisé par l'application src/environments/environment.ts.

**Remarque**: vous constaterez la présence du fichier src/environments/environment.wab.ts celui-ci est exclusif à l'utilisation à travers WAB et chargera la structure applicative dans un fichier JSON **app.json**. Ceci permettra à wab d’offrir un studio pour la création d'applications.

Pour comprendre comment construire ce fichier, je vous invite à regarder directement son modèle dans `client/src/app/vitis/app.model.ts`

## Arborescence

Prenons exemple sur deux applications (DtNet et Extraction) qui utilisent le même module GTF, en passant par `app.json` si une modification est faite dans le module GTF alors il faut modifier les `app.json` de DtNet ainsi que d'Extraction.

En passant par du `*.ts` nous pouvons importer des configurations depuis l'extérieur, ainsi dans `src/module_***/module/struct` nous retrouvons un fichier `.ts` exportant l'ensemble des modes et configurations du module. Il suffit alors de l'importer dans `app-struct.ts` pour profiter de ces variables et simplifier et fichier.

exemple:
```js
export const appStruct: AppConfig = {
  'translate': [vitisTranslateConfig, gtfTranslateConfig],
  'name': 'Gtf',
  'logo': {
    'type': 'icon',
    'class': 'icon-vitis-vitis'
  },
  'favicon': 'conf/images/Veremes-Leaf-Frame-W.png',
  'modes': [{
    'type': 'separator',
    'label': 'GTF',
    'privilege': 'gtf_user'
  },
    gtfMyWorkMode,
  {
    'type': 'separator',
    'label': 'TITLE_SEPARATOR_GTF_PROJECT_MANAGEMENT',
    'privilege': 'gtf_admin'
  },
    gtfPublicationMode,
    gtfSupervitionMode,
    gtfStatisticsMode,
  {...}
}
```

Sur cet exemple on voit clairement que les définitions des modes "Mon travail", "Publication", "Supervision", "Statistiques" sont déportées
et définies ailleurs.

## Conclusion

Avec app-struct.ts il est possible de construire rapidement une application en définissant ses différents menus.

Ces menus peuvent être définis dans Vitis ou dans un des modules de l'application.
