import puppeteer, { Browser, Page } from 'puppeteer';
import fs from 'fs';
import { Logger } from "vitis-engine-lib";
import { Db } from "vitis-engine-lib";
import { LocalFS } from "vitis-engine-lib";
import { openPage } from './utils';

/**
 * Évalue un template et retourne son code HTML
 * @param oProperties 
 * @param oDb 
 * @param oLogger 
 * @param oLocalFS 
 * @param args 
 */
export async function renderHtml_(oProperties: any, oDb: Db, oLogger: Logger, oLocalFS: LocalFS, args: any) {

    const sPrintClientURL = `file:///${oProperties.vas_dir}/src/Module/Vitis/utils/printserver/client/renderer/index.html`;
    const sOutputFileName = args.OutputFileName;
    let sHTMLBody = args.HTMLInputFileName;
    let sJsonIncludes = args.JsonIncludes;
    let sJsonScope = args.JsonScope;
    let sFormat = args.Format ? args.Format : 'a4';
    let aIncludes = {};
    let oScope = {};

    // Si sJsonIncludes est un fichier
    if (fs.existsSync(sJsonIncludes)) {
        sJsonIncludes = fs.readFileSync(sJsonIncludes).toString();
    }
    // Si sJsonScope est un fichier
    if (fs.existsSync(sJsonScope)) {
        sJsonScope = fs.readFileSync(sJsonScope).toString();
    }
    //chargement du body HTML
    if (fs.existsSync(sHTMLBody)) {
        sHTMLBody = fs.readFileSync(sHTMLBody).toString();
    }

    try {
        aIncludes = JSON.parse(sJsonIncludes);
    } catch (e) { }
    try {
        oScope = JSON.parse(sJsonScope);
    } catch (e) { }

    // Ouvre la page
    const openedPage = await openPage({
        sPrintClientURL,
        sHTMLBody,
        sFormat,
        aIncludes,
        oScope,
        oProperties,
    }, oLogger);
    const page = openedPage.page;
    const browser = openedPage.browser;
    await saveAsHtml(page, sOutputFileName);
    await browser.close();
}

/**
 * Sauvegarde le contenu HTML d'une page
 * @param page 
 * @param oOptions 
 * @returns 
 */
async function saveAsHtml(page: Page, sOutputFileName: string): Promise<string | undefined> {
    // Sauvegarde le fichier html
    const sDivContent = await page.evaluate(function () {
        const elem = document.getElementById('element_render');
        if (elem) {
            return elem.innerHTML;
        }
    });
    const writeFile = fs.openSync(sOutputFileName, 'w');
    fs.writeSync(writeFile, sDivContent);
    return sDivContent;
}