/**
 *                        Task Runner
 * 
 * Ce fichier permet de lancer les taches de manière synchrone
 * 
 */

import yargs from 'yargs';
import { args } from './printService-yargs-config';
import { propertiesSrvc } from "vitis-engine-lib";
import { Logger } from "vitis-engine-lib";
import { Db } from "vitis-engine-lib";
import { LocalFS } from "vitis-engine-lib";
import { printTemplate_ } from './lib/printTemplate';
import { renderHtml_ } from './lib/renderHtml';
import { printExcel_ } from './lib/printExcel';
import { concatPdf_ } from './lib/concatPdf';
import { signPdf_ } from './lib/signPdf';

/**
 * Imprime un template
 * @param oProperties 
 * @param oDb 
 * @param oLogger 
 * @param oLocalFS 
 */
async function printTemplate(oProperties: any, oDb: Db, oLogger: Logger, oLocalFS: LocalFS) {
    await printTemplate_(oProperties, oDb, oLogger, oLocalFS, args);
}

/**
 * Compile un HTML
 * @param oProperties 
 * @param oDb 
 * @param oLogger 
 * @param oLocalFS 
 */
async function renderHtml(oProperties: any, oDb: Db, oLogger: Logger, oLocalFS: LocalFS) {
    await renderHtml_(oProperties, oDb, oLogger, oLocalFS, args);
}

/**
 * Compile un HTML
 * @param oProperties 
 * @param oDb 
 * @param oLogger 
 * @param oLocalFS 
 */
async function printExcel(oProperties: any, oDb: Db, oLogger: Logger, oLocalFS: LocalFS) {
    await printExcel_(oProperties, oDb, oLogger, oLocalFS, args);
}
/**
 * Fusionne des pdf
 * @param oProperties 
 * @param oDb 
 * @param oLogger 
 * @param oLocalFS 
 */
async function concatPdf(oProperties: any, oDb: Db, oLogger: Logger, oLocalFS: LocalFS) {
    await concatPdf_(oProperties, oDb, oLogger, oLocalFS, args);
}
/**
 * Fusionne des pdf
 * @param oProperties 
 * @param oDb 
 * @param oLogger 
 * @param oLocalFS 
 */
async function signPdf(oProperties: any, oDb: Db, oLogger: Logger, oLocalFS: LocalFS) {
    await signPdf_(oProperties, oDb, oLogger, oLocalFS, args);
}

// Lance la commande en fonction des arguments passés
if (args._[0]) {
    const command: string = args._[0];

    // Récupère les properties
    propertiesSrvc.getProperties().then((oProperties) => {

        const oLogger = new Logger(oProperties);
        const oLocalFS = new LocalFS(oProperties, oLogger);

        if (args.noConsole) {
            oLogger.disableConsoleMessage();
        }

        oLogger.log('Get application properties and logger initalisation', null, 'DEBUG');

        // Base de données
        const oDb = new Db(oProperties, {
            user: oProperties.db_superuser,
            host: oProperties.db_server,
            database: oProperties.db_name,
            password: oProperties.db_password,
            port: oProperties.db_port,
            autoSslConfig: true
        }, oLogger);

        if (args.timeout) {
            if (typeof args.timeout == 'number' && !Number.isNaN(args.timeout)) {
                oDb.setMaxConnectionTime(args.timeout);
            }
        }

        if (args.noTimeout) {
            oDb.disableTimeOut();
        }

        try {
            oLogger.log('Command Start', args, 'DEBUG');
            // si une fonction avec le nom de la commande existe alors elle sera appelée
            return eval(command)(oProperties, oDb, oLogger, oLocalFS);
        } catch (error) {
            oLogger.log(`Command error : function not valid : ${command}`);
            return false;
        }
    });
} else {
    console.log('Veuillez spécifier une commande');
    yargs.showHelp("log");
}