<?php

namespace App;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Kernel\MicroKernelTrait;
use Symfony\Component\Config\Loader\LoaderInterface;
use Symfony\Component\Config\Resource\FileResource;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\HttpKernel\Kernel as BaseKernel;
use Symfony\Component\Routing\RouteCollectionBuilder;

/**
 * Point d'entrée de l'API généré par symfony et légérement customisée.
 *
 * @author Anthony Borghi   <anthony.borghi@veremes.com>
 * @author Armand Bahi <armand.bahi@veremes.com>
 * @author Alexandre Mironenkov <alexandre.mironenkov@veremes.com>
 */

class Kernel extends BaseKernel {
  use MicroKernelTrait;

  private const CONFIG_EXTS = '.{php,xml,yaml,yml}';

  private const OPTION_CONTROLLER_NAME = 'App\\Module\\Vitis\\WebService\\Controller\\GenericController::getOptions';
  private const OPTION_ROUTE_NAME = '_options';
  private const OPTION_ROUTE_METHODS = ['OPTIONS'];

  public function registerBundles(): iterable {
    $contents = require $this->getProjectDir() . '/config/bundles.php';
    foreach ($contents as $class => $envs) {
      if ($envs[$this->environment] ?? $envs['all'] ?? false) {
        yield new $class();
      }
    }
  }

  public function getProjectDir(): string {
    return \dirname(__DIR__);
  }

  protected function configureContainer(ContainerBuilder $container, LoaderInterface $loader): void {
    $container->addResource(new FileResource($this->getProjectDir() . '/config/bundles.php'));
    $container->setParameter('container.dumper.inline_class_loader', true);
    $confDir = $this->getProjectDir() . '/config';

    $loader->load($confDir . '/{packages}/*' . self::CONFIG_EXTS, 'glob');
    $loader->load($confDir . '/{packages}/' . $this->environment . '/**/*' . self::CONFIG_EXTS, 'glob');
    $loader->load($confDir . '/{services}' . self::CONFIG_EXTS, 'glob');
    $loader->load($confDir . '/{services}_' . $this->environment . self::CONFIG_EXTS, 'glob');

    $loader->load($this->getProjectDir() . '/src/Module/**/{services}' . self::CONFIG_EXTS, 'glob');
  }

  protected function configureRoutes(RouteCollectionBuilder $routes): void {
    $oProperties = new Properties($this->getProjectDir());
    $oRouteService = new Route($oProperties);

    $oRouteConfigurations = $oRouteService->getConfiguration();
    $aRoutes = isset($oRouteConfigurations['routes']) ? $oRouteConfigurations['routes'] : [];
    foreach ($aRoutes as $sName => $aRoute) {
      if (isset($aRoute['methods'])) {
        // création de la route depuis routes.json
        $oRouteObject = $routes->add($aRoute['route'], $aRoute['controller'] . '::' . $aRoute['function'], $sName)
          ->setMethods($aRoute['methods']);
        // création de la route option si requis
        $bCreateOptions = isset($aRoute['auto_options']) ? $aRoute['auto_options'] : true;
        if ($bCreateOptions) {
          $oRouteOptionObject = $routes->add($aRoute['route'], self::OPTION_CONTROLLER_NAME, $sName . self::OPTION_ROUTE_NAME)
            ->setMethods(self::OPTION_ROUTE_METHODS);
        }
        // ajout des controle sur les paramètres
        $aRequirements = isset($aRoute['requirements']) ? $aRoute['requirements'] : [];
        foreach ($aRequirements as $sKey => $sRegex) {
          $oRouteObject->setRequirement($sKey, $sRegex);
          if ($bCreateOptions) {
            $oRouteOptionObject->setRequirement($sKey, $sRegex);
          }
        }
      } else {
        error_log('route ' . $sName . ' has no methods');
      }
    }
  }
}
