<?php

namespace App\Module\Vitis\WebService\Controller;

use App\Module\Vitis\WebService\Service\Configuration\Module;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\DatabaseConnection;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\JobsService;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\VitisRequest;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller qui permet de récupérer des informations sur le serveur et l'application.
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 * @author Alexandre Mironenkov <alexandre.mironenkov@veremes.com>
 */
class PropertiesController extends ApplicationData {
  /**
   * Map à utiliser pour lancer des actions spécifiques si une properties est modifiée
   * Les fonctions à appeler devront toujours ne pas avoir de paramètre, retourner un booléen et 
   * être public ou protected car appelé depuis le parent
   * @var array
   */
  protected const CUSTOM_ACTION_FOR_PROPERTIES = [
    'sync_domain_group' => 'manageSyncDomainGroup',
    'sync_domain_group_period' => 'manageSyncDomainGroup',
  ];

  /**
   * @var JobsService
   */
  private $oJobsService;

  /**
   * Initialise le controler et instancie les services requis
   * @param Properties $oPropertiesSrvc Service d'accès aux properties
   * @param VitisRequest $oVitisRequest Service d'abstraction de la requête Symfony
   * @param Route $oRouteConfig Service d'accès à la configuration de la route
   * @param LoggerManager $oLoggerManager Service d'utilsation des logs
   * @param DatabaseConnection $oDatabaseConnection Service d'accès à la base de données
   * @param FileManager $oFileManager Service permettant d'accéder aux fichiers
   * @param JobsService $oJobsService Service permet de gérer les jobs
   */
  public function __construct(Properties $oPropertiesSrvc, VitisRequest $oVitisRequest, Route $oRouteConfig, LoggerManager $oLoggerManager, DatabaseConnection $oDatabaseConnection, FileManager $oFileManager, JobsService $oJobsService) {
    parent::__construct($oPropertiesSrvc, $oVitisRequest, $oRouteConfig, $oLoggerManager, $oDatabaseConnection, $oFileManager);
    $this->oJobsService = $oJobsService;
  }

  /**
   * Traitement spécifique lié à la properties sync_domain_group
   * @return bool true si l'action s'est bien passé false sinon
   */
  protected function manageSyncDomainGroup(): bool {
    if ($this->oProperties->get('sync_domain_group') == true) {
      $sPeriod = $this->oProperties->get('sync_domain_group_period');
      if (!empty($sPeriod)) {
        return $this->oJobsService->addAutosyncgroups($sPeriod);
      } else {
        return false;
      }
    } else {
      return $this->oJobsService->removeAutosyncgroups();
    }
  }
}
