<?php

namespace App\Module\Vitis\WebService\Controller;

use Symfony\Component\HttpFoundation\Response;

/**
 * Controller Générique de Vitis.
 *
 * @author Armand Bahi <armand.bahi@veremes.com>
 */
class WebservicesController extends GenericController {

  /**
   * Permet de récupérer la liste des service web disponibles
   * @return Response
   */
  public function getWebservices() : Response {

    // réponse par défaut
    $aReturn = [
      'data' => [],
      'status' => 0,
    ];

    // Ensemble des routes disponibles
    $aRoutes = $this->getAllRoutes_();

    // Modules qui ont des routes
    $aReturn['data'] = $this->getWebservicesFromRoutes_($aRoutes);
    $aReturn['status'] = 1;

    return $this->formatResponse($aReturn);
  }

  /**
   * Permet de récupérer la liste des ressources d'un service web
   * @param mixed $webservice
   * @return Response
   */
  public function getWebserviceRessource($webservice) : Response {

    // réponse par défaut
    $aReturn = [
      'data' => [],
      'status' => 0,
    ];

    // Ensemble des routes disponibles
    $aRoutes = $this->getAllRoutes_();

    // Routes du webservice
    $aReturn['data'] = $this->getWebServiceRessources_($webservice, $aRoutes);
    $aReturn['status'] = 1;

    return $this->formatResponse($aReturn);
  }

  /**
   * Récupère l'ensemble des routes
   * @return array tableau de routes
   */
  private function getAllRoutes_(): array{

    $aRoutes = [];

    $aRouteConfig = $this->oRouteConfig->getConfiguration();

    if (!empty($aRouteConfig['routes'])) {
      foreach ($aRouteConfig['routes'] as $key => $value) {
        if (!empty($value['route']) && in_array('GET', $value['methods'])) {
          array_push($aRoutes, $value['route']);
        }
      }
    }

    return $aRoutes;
  }

  /**
   * Récupère la liste des webservices qui ont au moins une route
   * @param string[] $aRoutes
   * @return array tableau contenant un tableau ex : [{ name : gtf }, { name : vitis }]
   */
  private function getWebservicesFromRoutes_($aRoutes): array{

    $aWebServices = [];

    foreach ($aRoutes as $sRoute) {

      $aRoutePath = explode('/', $sRoute);
      foreach ($this->oProperties->get('_module_list') as $sModuleName) {
        if (strtolower($sModuleName) === $aRoutePath[1] || strtolower($sModuleName) === $aRoutePath[2]) {
          if (!in_array(strtolower($sModuleName), $aWebServices)) {
            array_push($aWebServices, strtolower($sModuleName));
          }
        }
      }
    }

    // mise en forme du résultat
    sort($aWebServices);
    foreach ($aWebServices as $key => $value) {
      $aWebServices[$key] = ['name' => $value];
    }

    return $aWebServices;
  }

  /**
   * Retourne les routes disponibles pour un service web
   * @param string $webservice
   * @param string[] $aRoutes
   * @return array tableau contenant un tableau ex : [{ name : Adresses }, { name : CommuneByGeom }]
   */
  private function getWebServiceRessources_($webservice, $aRoutes): array{
    $aWsRoutes = [];

    foreach ($aRoutes as $sRoute) {

      $aRoutePath = explode('/', $sRoute);
      $iWSPos = array_search($webservice, $aRoutePath);

      // Si la route appartient au webservice
      if ($iWSPos === 1 || $iWSPos === 2) {

        $sRessource = implode('/', array_slice($aRoutePath, $iWSPos + 1));

        // Si c'est une route sans accolades
        if (strpos($sRessource, '{') === false) {
          if (!in_array($sRessource, $aWsRoutes)) {
            array_push($aWsRoutes, $sRessource);
          }
        }
      }
    }

    // Mise en forme du résultat
    sort($aWsRoutes);
    foreach ($aWsRoutes as $key => $value) {
      $aWsRoutes[$key] = ['name' => $value];
    }

    return $aWsRoutes;
  }
}