<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\Common\Persistence\ObjectManagerAware;
use Doctrine\Mapping\Column;
use Doctrine\Mapping\Entity;
use Doctrine\Mapping\GeneratedValue;
use Doctrine\Mapping\Id;
use Doctrine\Mapping\OneToMany;
use Doctrine\Mapping\SequenceGenerator;
use Doctrine\Mapping\Table;
use Doctrine\ORM\EntityManager;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\DomainRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="domain")
 */
class Domain extends AbstractEntity implements ObjectManagerAware {
  /**
   * @var Properties
   */
  private $oProperties;

  /**
   * @var EntityManager
   */
  private $oEntityManager;
  /**
   * @var LoggerManager
   */
  private $oLoggerManager;

  /**
   * @var int
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vitis).seq_common")
   * @Column(type="integer", name="domain_id", nullable=false)
   */
  private $domain_id;

  /**
   * @var string|null
   * @Column(type="string", length=64, nullable=true)
   */
  private $name;

  /**
   * @var string|null
   * @Column(type="string", length=64, nullable=true)
   */
  private $login_role;

  /**
   * @var string|null
   * @Column(type="string", length=64, nullable=true)
   */
  private $suffix;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $prefix;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $uid;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $displayname;

  /**
   * @var string|null
   * @Column(type="string", length=255, nullable=true)
   */
  private $mail;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $server;

  /**
   * @var int|null
   * @Column(type="integer", nullable=true)
   */
  private $port;

  /**
   * @var string|null
   * @Column(type="string", length=255, nullable=true)
   */
  private $dn_search_user;

  /**
   * @var string
   * @Column(type="string", length=100)
   */
  private $filter_user;

  /**
   * @var string
   * @Column(type="string", length=255)
   */
  private $dn_search_group;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $filter_group;

  /**
   * @var string|null
   * @Column(type="string", length=50, nullable=true)
   */
  private $login;

  /**
   * @var string|null
   * @Column(type="string", length=255, nullable=true)
   */
  private $password;

  /**
   * @var string|null
   * @Column(type="string", length=10, nullable=true)
   */
  private $type;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $verify_rights;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $checkcert;

  /**
   * @var string|null
   * @Column(type="string", length=10, nullable=true)
   */
  private $encrypttype;

  /**
   * @var ArrayCollection<\App\Module\Vitis\WebService\Entity\User>
   * @OneToMany(targetEntity="App\Module\Vitis\WebService\Entity\User", mappedBy="domain")
   */
  private $users;

  /**
   * @var ArrayCollection<DomainGroup>
   * @OneToMany(targetEntity=DomainGroup::class, mappedBy="domain", cascade={"persist", "remove"})
   */
  private $domainGroups;

  public function __construct() {
    $this->users = new ArrayCollection();
    $this->domainGroups = new ArrayCollection();
  }

  /**
   * pour debug
   * @return string|null
   */
  public function __toString(): ?string {
    return $this->getName();
  }

  /**
   * @return integer|null identifiant du domaine
   */
  public function getDomainId(): ?int {
    return $this->domain_id;
  }

  /**
   * @return string|null
   */
  public function getName(): ?string {
    return $this->name;
  }

  /**
   * @param string|null $name nom du domaine
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getLoginRole(): ?string {
    return $this->login_role;
  }

  /**
   * @param string|null $login_role
   * @return self
   */
  public function setLoginRole(?string $login_role): self {
    $this->login_role = $login_role;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getSuffix(): ?string {
    return $this->suffix;
  }

  /**
   * @param string|null $suffix
   * @return self
   */
  public function setSuffix(?string $suffix): self {
    $this->suffix = $suffix;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getPrefix(): ?string {
    return $this->prefix;
  }

  /**
   * @param string|null $prefix
   * @return self
   */
  public function setPrefix(?string $prefix): self {
    $this->prefix = $prefix;

    return $this;
  }

  /**
   * @return string retourne le nom de la colonne uid ou un nom par défaut en fonction du type
   */
  public function getUid(): string {
    $sUID = $this->uid;
    if (empty($sUID)) {
      switch ($this->getType()) {
        case 'LDAP':
          $sUID = 'sAMAccountName';
          break;
        case 'AD':
        default:
          $sUID = 'userPrincipalName';
          break;
      }
    }
    return $sUID;
  }

  /**
   * @param string|null $uid
   * @return self
   */
  public function setUid(?string $uid): self {
    $this->uid = $uid;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getDisplayName(): ?string {
    return $this->displayname;
  }

  /**
   * @param string|null $displayname
   * @return self
   */
  public function setDisplayName(?string $displayname): self {
    $this->displayname = $displayname;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getMail(): ?string {
    return $this->mail;
  }

  /**
   * @param string|null $mail
   * @return self
   */
  public function setMail(?string $mail): self {
    $this->mail = $mail;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getServer(): ?string {
    return $this->server;
  }

  /**
   * @param string|null $server
   * @return self
   */
  public function setServer(?string $server): self {
    $this->server = $server;

    return $this;
  }

  /**
   * @return integer|null
   */
  public function getPort(): ?int {
    return $this->port;
  }

  /**
   * @param integer|null $port
   * @return self
   */
  public function setPort(?int $port): self {
    $this->port = $port;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getDnSearchUser(): ?string {
    return $this->dn_search_user;
  }

  /**
   * @param string|null $dn_search_user
   * @return self
   */
  public function setDnSearchUser(?string $dn_search_user): self {
    $this->dn_search_user = $dn_search_user;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getFilterUser(): ?string {
    return $this->filter_user;
  }

  /**
   * @param string|null $filter_user
   * @return self
   */
  public function setFilterUser(?string $filter_user): self {
    $this->filter_user = $filter_user;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getDnSearchGroup(): ?string {
    return $this->dn_search_group;
  }

  /**
   * @param string|null $dn_search_group
   * @return self
   */
  public function setDnSearchGroup(?string $dn_search_group): self {
    $this->dn_search_group = $dn_search_group;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getFilterGroup(): ?string {
    return $this->filter_group;
  }

  /**
   * @param string|null $filter_group
   * @return self
   */
  public function setFilterGroup(?string $filter_group): self {
    $this->filter_group = $filter_group;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getLogin(): ?string {
    return $this->login;
  }

  /**
   * @param string|null $login
   * @return self
   */
  public function setLogin(?string $login): self {
    $this->login = $login;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getPassword(): ?string {
    return $this->password;
  }

  /**
   * @param string|null $password
   * @return self
   */
  public function setPassword(?string $password): self {
    $this->password = $password;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getType(): ?string {
    return $this->type;
  }

  /**
   * @param string|null $type
   * @return self
   */
  public function setType(?string $type): self {
    $this->type = $type;

    return $this;
  }

  /**
   * @return bool|null
   */
  public function getVerifyRights(): ?bool {
    return $this->verify_rights;
  }

  /**
   * @param bool|null $verify_rights
   * @return self
   */
  public function setVerifyRights(?bool $verify_rights): self {
    $this->verify_rights = $verify_rights;

    return $this;
  }

  /**
   * @return bool|null
   */
  public function getCheckcert(): ?bool {
    return $this->checkcert;
  }

  /**
   * @param bool|null $checkcert
   * @return self
   */
  public function setCheckcert(?bool $checkcert): self {
    $this->checkcert = $checkcert;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getEncrypttype(): ?string {
    return $this->encrypttype;
  }

  /**
   * @param string|null $encrypttype
   * @return self
   */
  public function setEncrypttype(?string $encrypttype): self {
    $this->encrypttype = $encrypttype;

    return $this;
  }

  /**
   * @return Collection<User>
   */
  public function getUsers(): Collection {
    return $this->users;
  }

  /**
   * retourne la config pour se connecter à un LDAP/AD
   * @return array
   */
  public function getLdapConfig(): array{
    $aConfig = [
      'host' => $this->server,
      'debug' => $this->oProperties->get('log_level') === 'DEBUG',
      'version' => 3,
      'referrals' => false,
      'encryption' => 'none',
      'options' => [
        // ancienne variable d'environnement pour GTF
        'x_tls_require_cert' => LDAP_OPT_X_TLS_NEVER,
      ],
    ];
    // none, ssl, tls
    if (!empty($this->encrypttype) && in_array($this->encrypttype, ['ssl', 'tls'])) {
      $aConfig['encryption'] = $this->encrypttype;
    }
    if (isset($this->port)) {
      $aConfig['port'] = $this->port;
    }
    return $aConfig;
  }

  /**
   * Configuration pour la pagination de la récupération des entrées
   *
   * @return array
   */
  public function getLdapQueryOptions(): array{
    return [
      'pageSize' => $this->oProperties->get('ldap_entries_per_page'),
      'maxItems' => $this->oProperties->get('ldap_max_entries')
    ];
  }

  /**
   * @return Collection<DomainGroup>
   */
  public function getDomainGroups(): Collection {
    return $this->domainGroups;
  }

  /**
   * @param DomainGroup $domainGroup groupe à ajouter
   * @return self
   */
  public function addDomainGroup(DomainGroup $domainGroup): self {
    if (!$this->domainGroups->contains($domainGroup)) {
      $this->domainGroups[] = $domainGroup;
      $domainGroup->setDomain($this);
    }

    return $this;
  }

  /**
   * @param DomainGroup $domainGroup groupe à supprimer
   * @return self
   */
  public function removeDomainGroup(DomainGroup $domainGroup): self {
    if ($this->domainGroups->removeElement($domainGroup)) {
      // set the owning side to null (unless already changed)
      if ($domainGroup->getDomain() === $this) {
        $domainGroup->setDomain(null);
      }
    }

    return $this;
  }

  /**
   * Analyse l'objet retouner par LDAP/AD et le prépare pour être renvoyer au client pour un utilisateur
   * @param array $aData Donnée pour une ligne venant de decodeEntries
   * @return array $aData préparé pour le client
   */
  public function convertEntriesDataForUser(array $aData): array{
    $aData['uid'] = $this->testAndSetValueFromEntrie($aData, 'uid', $this->getUid());
    $aData['displayName'] = $this->testAndSetValueFromEntrie($aData, 'displayName', $this->getDisplayName());
    $aData['mail'] = $this->testAndSetValueFromEntrie($aData, 'mail', $this->getMail());
    $aData['domain_id'] = $this->getDomainId();
    return $aData;
  }

  /**
   * Analyse l'objet retouner par LDAP/AD et le prépare pour être renvoyer au client pour un groupe
   * @param array $aData Donnée pour une ligne venant de decodeEntries
   * @return array $aData préparé pour le client
   */
  public function convertEntriesDataForGroup(array $aData): array{
    $aData['mail'] = $this->testAndSetValueFromEntrie($aData, 'mail', $this->getMail());
    $aData['domain_id'] = $this->getDomainId();
    return $aData;
  }

  /**
   * @param array $aData Enties data
   * @param string $sClientKey Clé coté client pour datatable
   * @param string $sDomainKey clé dans la données
   * @return mixed
   */
  private function testAndSetValueFromEntrie(array $aData, string $sClientKey, ?string $sDomainKey) {
    // si la clé est déjà défini on ne change rien
    if (isset($aData[$sClientKey])) {
      return $aData[$sClientKey];
    }
    // si une clé spécifique est passé on fait le mappage
    if ($sDomainKey != null) {
      return $aData[$sDomainKey] ?? '';
    }
    // si rien ne match on retourne chaine vide
    return '';
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base (ObjectManagerAware)
   * @param ObjectManager $objectManager
   * @param ClassMetadata $classMetadata
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @param LoggerManager $oService
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @param Properties $oService
   * @return void
   */
  public function setPropertiesService(Properties $oService): void {
    $this->oProperties = $oService;
  }
}
