<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Entity\Domain;
use App\Module\Vitis\WebService\Entity\Group;
use Doctrine\Mapping\Column;
use Doctrine\Mapping\Entity;
use Doctrine\Mapping\Id;
use Doctrine\Mapping\JoinColumn;
use Doctrine\Mapping\OneToOne;
use Doctrine\Mapping\Table;

/**
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\DomainGroupRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="domain_group")
 */
class DomainGroup extends AbstractEntity {
  /**
   * @var int
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vitis).seq_common")
   * @Column(type="integer", name="domain_group_id", nullable=false)
   */
  private $domainGroupId;

  /**
   * @var Group
   * @OneToOne(targetEntity=Group::class, inversedBy="domainGroup", cascade={"persist", "remove"})
   * @JoinColumn(name="vitis_group_id", referencedColumnName="group_id")
   */
  private $vitisGroup;

  /**
   * @var Domain
   * @ManyToOne(targetEntity=Domain::class, inversedBy="domainGroups", cascade={"persist", "remove"})
   * @JoinColumn(name="domain_id", referencedColumnName="domain_id")
   */
  private $domain;

  /**
   * @var string
   * @Column(type="string", length=255, nullable=true)
   */
  private $dn;

  /**
   * @var bool
   * @Column(name="auto_refresh",type="boolean")
   */
  private $autoRefresh = false;

  public function toArray(): ?array{
    return [
      'domain_group_id' => $this->getDomainGroupId(),
      'name' => $this->vitisGroup->getName(),
      'displayname' => $this->vitisGroup->getName(),
      'domain' => $this->domain->getName(),
      'group_id' => $this->vitisGroup->getGroupId(),
      'domain_id' => $this->domain->getDomainId(),
      'dn' => $this->getDn(),
    ];
  }

  public function getDomainGroupId(): ?int {
    if (isset($this->domain)) {
      return $this->domain->getDomainId();
    }
    return NULL;
  }

  public function setDomainGroupId(int $domainGroupId): self {
    $this->domainGroupId = $domainGroupId;

    return $this;
  }

  public function getVitisGroup(): ?Group {
    return $this->vitisGroup;
  }

  public function setVitisGroup(?Group $VitisGroup): self {
    $this->vitisGroup = $VitisGroup;

    return $this;
  }

  public function getDomain(): ?Domain {
    return $this->domain;
  }

  public function setDomain(?Domain $Domain): self {
    $this->domain = $Domain;

    return $this;
  }

  public function getDn(): ?string {
    return $this->dn;
  }

  public function setDn(?string $dn): self {
    $this->dn = $dn;

    return $this;
  }

  public function getAutoRefresh(): ?bool {
    return $this->autoRefresh;
  }

  public function setAutoRefresh(bool $autoRefresh): self {
    $this->autoRefresh = $autoRefresh;

    return $this;
  }
}
