<?php

namespace App\Module\Vitis\WebService\Entity;

use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour userRequest
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="user_request")
 */
class UserRequest extends AbstractEntity {
  /**
   * @var string
   * @Id
   * @Column(type="string", length=64)
   */
  private $login;

  /**
   * @var string
   * @Column(type="string", length=80, nullable=true)
   */
  private $name;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $email;

  /**
   * @var string
   * @Column(type="string", length=80, nullable=true)
   */
  private $company;

  /**
   * @var string
   * @Column(type="string", length=80, nullable=true)
   */
  private $department;
  /**
   * @var string
   * @Column(type="string")
   */
  private $datakey;
  /**
   * @var int
   * @Column(type="integer")
   */
  private $status = 1;
  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $comp_data;

  /**
   * @return string login de l'utilisateur
   */
  public function getLogin(): string {
    return $this->login;
  }

  /**
   * Défini le login de l'utilisateur, réalisable uniquement lors de la création
   * @param string $login login de l'utilisateur
   * @return self
   */
  public function setLogin(string $login): self {
    if (!isset($this->login)) {
      $this->login = $login;
    }
    return $this;
  }

  /**
   * @return string|null nom de l'utilisateur
   */
  public function getName(): ?string {
    return $this->name;
  }

  /**
   * @param string $name nom de l'utilisateur
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string|null mail de l'utilisateur
   */
  public function getEmail(): ?string {
    return $this->email;
  }

  /**
   * @param string $email mail de l'utilisateur
   * @return self
   */
  public function setEmail(?string $email): self {
    $this->email = $email;
    return $this;
  }

  /**
   * @return string|null organisme de l'utilisateur
   */
  public function getCompany(): ?string {
    return $this->company;
  }

  /**
   * @param string $company organisme de l'utilisateur
   * @return self
   */
  public function setCompany(?string $company): self {
    $this->company = $company;
    return $this;
  }

  /**
   * @return string|null service de l'utilisateur
   */
  public function getDepartment(): ?string {
    return $this->department;
  }

  /**
   * @param string|null $department service de l'utilisateur
   * @return self
   */
  public function setDepartment(?string $department): self {
    $this->department = $department;
    return $this;
  }

  /**
   * @return string|null URL de confirmation
   */
  public function getDatakey(): ?string {
    return $this->datakey;
  }

  /**
   * @param string|null $sDatakey Url de confirmation
   * @return self
   */
  public function setDatakey(?string $sDatakey): self {
    $this->datakey = $sDatakey;
    return $this;
  }

  /**
   * @return int|null URL de confirmation
   */
  public function getStatus(): ?int {
    return $this->status;
  }

  /**
   * @param int $iStatus Url de confirmation
   * @return self
   */
  public function setStatus(int $iStatus): self {
    $this->status = $iStatus;
    return $this;
  }

  /**
   * @return string|null donnée complémentaire
   */
  public function getCompData(): ?string {
    return $this->comp_data;
  }

  /**
   * @param array|string $mData donnée à ajouter en array ou json
   * @return self|null en cas d'erreur
   */
  public function setCompData($mData): ?self {

    if (\is_array($mData)){
      $mData = json_encode($mData);
    }

    if (\is_string($mData)){
      $this->comp_data = $mData;
      return $this;
    }

    return null;
  }
}
