<?php

namespace App\Module\Vitis\WebService\Service\DataAccessor;

/**
 * DataAccessor utilise les options de la route pour accéder au données en base
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

interface DataAccessorInterface {
  /**
   * Récupère une entité.
   * @param mixed $id id de l'entité à trouver
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array 
   *          'data' : entité demandée
   *          'status' : détermine s'il y a eu une erreur
   */
  public function getEntity($id, array $aData, bool $useCustomFields = true) : array;
  /**
   * Récupère toutes les entités.
   * @param array $aData paramètres de restriction du retour de données
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array 
   *          'data' : entités demandées
   *          'list_count' : nombre d'entités retournées
   *          'total_row_number' : nombre total d'entités
   *          'status' : détermine s'il y a eu une erreur
   */
  public function getEntities(array $aData, bool $useCustomFields = true) : array;
  /**
   * Créé une entité.
   * @param string $sData Donnée à inséré format Json
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array retour de getEntity si l'insertion s'est bien passé, le statut seulement sinon
   */
  public function insertEntity(string $sData, bool $useCustomFields = true) : array;
  /**
   * Créé plusieurs entités.
   * @param array $aData Donnée à inséré format Json
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array retour de getEntity si l'insertion s'est bien passé, le statut seulement sinon
   */
  public function insertEntities(array $aData, bool $useCustomFields = true) : array;
  /**
   * Met à jour une entité.
   * @param mixed $id id de l'entité à mettre à jour
   * @param string $sData Donnée à mettre à jour format Json
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array retour de getEntity si l'insertion s'est bien passé, le statut seulement sinon
   */
  public function updateEntity($id, string $sData, bool $useCustomFields = true) : array;
  /**
   * Supprime une entité.
   * @param mixed $id id de l'entité à Supprimer
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array 
   *            data : id supprimé
   *            status : état
   */
  public function deleteEntity($id, bool $useCustomFields = true) : array;
  /**
   * Supprime des entité.
   * @param mixed[] $aId id des entités à Supprimer
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array id des entités supprimés
   *            data : liste des id supprimé
   *            total_row_removed : nombre d'élément supprimé
   *            status : état
   */
  public function deleteEntities(array $aId, bool $useCustomFields = true) : array;

  /**
   * Met à jour des entités.
   * @param mixed[] $aId id des l'entités à mettre à jour
   * @param string $sData Donnée à mettre à jour format Json
   * @param bool $useCustomFields définit s'il faut charger les champs custom
   * @return array résultat de getEntities, uniquement le status en cas d'erreur
   */
  public function updateEntities(array $aId, string $sData, bool $useCustomFields = true) : array;
}
