<?php

namespace App\Module\Vitis\WebService\Service\DataAccessor\Fields;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\Fields\FieldInterface;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;

/**
 * Classe abstraite pour gérer un champ.
 * ne fait que des return sans modification.
 *
 *  Configuration basique d'un champ
 *    "type"      (string) : type du champ (string, number, ...)
 *    "name"      (string) : Nom du champ
 *
 *  exemple :
 *    {
 *      "type": "string",
 *      "name": "my_field"
 *    }
 *
 * Cette classe ne peut/doit pas être instancié directement
 *
 * @author Anthony Borghi   <anthony.borghi@veremes.com>
 */

abstract class AbstractCustomField implements FieldInterface {
  /**
   * @var array
   */
  protected const FORCE_ATTRIBUTE = [];
  /**
   * @var mixed
   */
  protected const FORCE_VALUE = null;

  /**
   * @var Properties
   */
  protected $oProperties;
  /**
   * @var Route
   */
  protected $oRouteConfig;
  /**
   * @var LoggerManager
   */
  protected $oLoggerManager;

  /**
   * Cette classe ne doit pas être instanciée directement
   * @param Properties $oPropertiesSrvc Un objet pour accéder aux properties de l'application
   * @param Route $oRoute Service permettant d'accéder à la configuration d'une route
   * @param LoggerManager $oLoggerManager service d'accès aux logs
   */
  protected function __construct(Properties $oPropertiesSrvc, Route $oRoute, LoggerManager $oLoggerManager) {
    $this->oProperties = $oPropertiesSrvc;
    $this->oRouteConfig = $oRoute;
    $this->oLoggerManager = $oLoggerManager;
  }

  /**
   * Retourne l'expression utilisée en tant que colonne dans le requête GET
   * {@inheritdoc}
   */
  public function getFieldColumn(string $sSchema, string $sField): string{
    return $sSchema . '.' . $sField;
  }

  /**
   * Retourne la fonction utilisée en tant que colonne dans le requête PUT ou insert
   * {@inheritdoc}
   */
  public function getFieldColumnFunction(string $sField): ?string{
    return null;
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }

  /**
   * {@inheritdoc}
   */
  public function insertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }
  /**
   * {@inheritdoc}
   */
  public function afterInsertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }
  /**
   * {@inheritdoc}
   */
  public function updateFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }
  /**
   * {@inheritdoc}
   */
  public function updateMassiveFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }
  /**
   * {@inheritdoc}
   */
  public function forceAttribute(string $sMethodName): bool {
    return static::FORCE_ATTRIBUTE[$sMethodName] ?? false;
  }
  /**
   * {@inheritdoc}
   */
  public function getForceValue() {
    return static::FORCE_VALUE;
  }
  /**
   * {@inheritdoc}
   */
  public function fieldIsSet(array $aEntity, array $aFieldOptions) : bool {
    return isset($aEntity[$aFieldOptions['name']]);
  }
}

?>