<?php

namespace App\Module\Vitis\WebService\Service\Mail;

use App\Module\Vitis\WebService\Entity\MailTemplate;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\Mail\Mail;
use App\Module\Vitis\WebService\Service\Mail\MailContextInterface;
use App\Module\Vitis\WebService\Service\PrintService\HtmlRenderer;

/**
 * Permet d'évaluer un template avec un contexte mail.
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

class TemplateRenderer {
  /**
   * @var HtmlRenderer  Renderer phantomJs
   */
  private $oRenderer;
  /**
   * @var Properties service d'accés aux properties
   */
  private $oProperties;
  /**
   * @var LoggerManager Service d'accès aux logs
   */
  private $oLoggerManager;
  /**
   * @var FileManager Service d'accès aux fichiers
   */
  private $oFileManager;
  /**
   * @param HtmlRenderer $oRenderer Renderer phantomJs
   * @param Properties $oProperties service d'accés aux properties
   * @param LoggerManager $oLogger Service d'accès aux logs
   * @param FileManager $oFileManager Service d'accès aux fichiers
   */
  public function __construct(HtmlRenderer $oRenderer, Properties $oProperties, LoggerManager $oLogger, FileManager $oFileManager) {
    $this->oRenderer = $oRenderer;
    $this->oLoggerManager = $oLogger;
    $this->oProperties = $oProperties;
    $this->oFileManager = $oFileManager;
  }
  /**
   * @param MailTemplate $oTemplate Entité contenant le template de mail
   * @param MailContextInterface $oMailContext Entité implémentant l'interface MailContext
   * @return Mail|null un mail envoyable ou null en cas d'erreur
   */
  public function evalTemplateWithContext(MailTemplate $oTemplate, MailContextInterface $oMailContext): ?Mail {
    // on appelle la fonction de MailContext pour savoir s'il faut envoyer le mail
    if (!$oMailContext->haveToSendMail()) {
      return null;
    }
    $aScope = $oTemplate->getJsonScope() ?? [];
    // fusion du scope avec le contexte
    $aScope = array_merge_recursive($oMailContext->getContextData(), $aScope);
    $aIncludes = $oTemplate->getJsonIncludes() ?? [];
    // on récupère le body, s'il est vide on ne render pas le mail
    $sBody = $oTemplate->getBody() ?? '';
    $sRenderedBody = '';
    if (!empty($sBody)) {
      $sRenderedBody = $this->oRenderer->render($oTemplate->getBody(), $aScope, $aIncludes);
    }
    $sMailParameters = json_encode($oTemplate->getMailParameters());
    // Replace \\' par \' pour annulé l'effet de json_encode sur \' et échapper correctement l'apostrophe
    $sMailParameters = str_replace("\\\\'", "\\'", $sMailParameters);

    $sRenderedParameters = $this->oRenderer->render($sMailParameters, $aScope, []);

    $aMailStruct = json_decode($sRenderedParameters, true);
    $aMailStruct['body'] = $sRenderedBody;

    $oMail = new Mail($this->oProperties, $this->oLoggerManager, $this->oFileManager);
    // charge la structure du évalué dans un objet mail
    $oMail->loadFromArray($aMailStruct);

    return $oMail;
  }
}