<?php

namespace App\Module\Vitis\WebService\Service\PhantomJs;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\FileAccessor\LocalAccessor;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;

abstract class AbstractCaller {
  /**
   * @var \Monolog\Logger
   */
  protected $oLogger;
  /**
   * @var Properties
   */
  protected $oProperties;
  /**
   * @var LocalAccessor
   */
  protected $oLocalFileAccessor;
  /**
   * @param LoggerManager $oLogger Service d'accès aux logs
   * @param Properties $properties service d'accés aux properties
   * @param FileManager $oFileManager service d'accés aux fichiers
   */
  protected function __construct(LoggerManager $oLogger, Properties $properties, FileManager $oFileManager) {
    $this->oLogger = $oLogger->get('API_ERROR');
    $this->oProperties = $properties;
    $this->oLocalFileAccessor = $oFileManager->getLocalFileAccessor();
  }
/**
 * @param string $sScriptPath chemin vers le script à appeler avec phantomjs
 * @param string[] $aArguments arguments à utiliser pour le script phantomjs
 * @throws ProcessFailedException si le process termine en erreur
 * @return string retour de la commande
 */
  protected function call(string $sScriptPath, array $aArguments): string {
    $aCommand = array_merge([$this->oProperties->get('phantomjs_executable'), $sScriptPath], $aArguments);
    $process = new Process($aCommand);
    $process->run();

    // executes after the command finishes
    if (!$process->isSuccessful()) {
      throw new ProcessFailedException($process);
    }

    return $process->getOutput();
  }
}