<?php

namespace App\Module\Vitis\WebService\Service\PrintService;

use App\Module\Vitis\WebService\Service\FileAccessor\LocalAccessor;

/**
 * Concatène des PDF
 * @author Anthony Borghi<anthony.borghi@veremes.com>
 */
class PdfConcatenator extends AbstractPrinter {
  /**
   * merge plusieurs pdf en un
   * @param string $sInputFolder dossier contenant les PDF
   * @param string $sOutputFile emplacement du PDF final
   * @return bool|null true en cas d esuccés null sinon
   */
  public function concat(string $sInputFolder, string $sOutputFile): ?bool {
    try {
      // check si le dossier en entré existe
      if (!$this->oLocalFileAccessor->file_exists($sInputFolder)) {
        throw new \Exception($sInputFolder . ' doesn\'t exist');
      }

      // créer le dossier de sortie s'il n'existe pas
      $sDir = dirname($sOutputFile);
      if (!$this->oLocalFileAccessor->file_exists($sDir)) {
        $this->oLocalFileAccessor->mkdir($sDir, LocalAccessor::DEFAULT_LINUX_PERMISSION, true);
      }
      // génération du tableau des arguments
      $aArguments = $this->prepareArguments($sInputFolder, $sOutputFile);

      //exécution de la commande
      $sReturn = $this->callPrinter($aArguments);
      $this->oDebugLogger->debug($sReturn);
    } catch (\Exception $e) {
      $this->oErrorLogger->error('Impossible to merge PDF : ' . $e->getMessage());
      return null;
    }
    return true;
  }

  /**
   * Prépare les arguments pour la CLI
   * @param string $sInputFolder dossier pdf
   * @param string $sOutputFile pdf de sortie
   * @throws \Exception si le dossier temporaire ne peut pas être créé
   * @return array tableau d'argument à utiliser pour appeler phantomjs
   */
  private function prepareArguments(string $sInputFolder, string $sOutputFile): array{
    // génération d'un dossier temporaire
    $aArguments = ['concatPdf'];
    // chemin vers les ficheir à concaténer
    $aArguments[] = '--inputFolder="' . $sInputFolder . '"';
    // chemin vers la sortie PDF
    $aArguments[] = '--OutputFileName="' . $sOutputFile . '"';

    return $aArguments;
  }

}
