<?php

namespace App\Module\Vitis\WebService\Service;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Monolog\Logger;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;

/**
 * Service permettant d'intéragir avec process et de l'encadrer
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

class VitisProcess extends Process {
  /**
   * @var Properties
   */
  private $oProperties;

  /**
   * @var Logger
   */
  private $oLogger;

  /**
   * @param array $aCommandLine voir parent
   * @param string $sCwd dossier d'exécution
   * @param Properties $properties service d'accés aux properties
   * @param LoggerManager $oLogger service d'accés aux logger
   */
  public function __construct(array $aCommandLine, string $sCwd = null, Properties $properties, LoggerManager $oLogger) {
    parent::__construct($aCommandLine, $sCwd);

    $this->oProperties = $properties;
    $this->oLogger = $oLogger->get('API_ERROR');

    // Valeur du timeout en fonction de la propertie max_execution_time
    $iTimeout = $this->oProperties->get('max_execution_time');
    if (is_numeric($iTimeout)) {
      $this->setTimeout($iTimeout);
    }

  }

  /**
   * Identique à start mais retourne un string
   * @param callable $callback voir start dans le parent
   * @param array $env environnement, voir start dans le parent
   * @return string dernière ligne provenant de STDOUT
   * @throws ProcessFailedException
   */
  public function startWithReturn(callable $callback = null, array $env = []): string {
    parent::start($callback, $env);
    $sLastLine = '';
    foreach ($this as $sType => $sData) {
      if (parent::OUT === $sType) {
        $this->oLogger->debug('Read from stdout : ' . $sData);
        $tmp = explode("\n", trim($sData));
        $sLastLine = end($tmp);
      } else { // parent::ERR === $sType
        $this->oLogger->error('Read from stderr : ' . $sData);
        throw new ProcessFailedException($this);
      }
    }

    return trim($sLastLine);
  }
}
