/* global phantom */

/**
 * @author: Armand Bahi
 * @author: Anthony Borghi
 * @description: Fichier PhantomJS permettant de générer du HTML évalué
 */

var page = require('webpage').create();
var system = require('system');
var fs = require('fs');

page.onError = function (msg, trace) {
  console.log(msg);
  trace.forEach(function (item) {
    console.log('  ', item.file, ':', item.line);
  });
};

if (system.args.length !== 6) {
  console.log('Usage: [phantomjs renderhtml.js PrintClientURL HTMLInputFileName OutputFileName JsonIncludes JsonScope]');
  phantom.exit(1);
} else {

  for (var i = 0; i < system.args.length; i++) {
    console.log("system.args[" + i + "]: ", system.args[i]);
  }

  // Temps avant de lancer checkReadyState
  var printTimeout = 2000;
  // Temps pour lancer l'impression sans checkReadyState
  var printLoadTimeout = 10000;
  // traitement des arguments
  var sPrintClientURL = system.args[1];
  var sHTMLBody = system.args[2]
  var sOutputFileName = system.args[3];
  var sIncludes = system.args[4];
  var sScope = system.args[5];

  // Si sIncludes est un fichier
  if (fs.exists(sIncludes)) {
    sIncludes = fs.read(sIncludes);
  }
  // Si sScope est un fichier
  if (fs.exists(sScope)) {
    sScope = fs.read(sScope);
  }
  console.log(sScope);
  //chargement du body HTML
  if (fs.exists(sHTMLBody)) {
    sHTMLBody = fs.read(sHTMLBody);
  }

  try {
    var oIncludes = JSON.parse(sIncludes);
  } catch (e) {
    var oIncludes = {};
  }
  try {
    var oScope = JSON.parse(sScope);
  } catch (e) {
    var oScope = {};
  }

  // Récuprère les messages envoyés par le client
  page.onCallback = function (data) {
    var cmd = data['cmd'];
    if (cmd === 'error') {
      if (typeof data['message'] !== undefined) {
        console.log(data['message']);
      }
      phantom.exit();
    } else if (cmd === 'consoleLog') {
      if (typeof data['message'] !== undefined) {
        console.log(data['message']);
      }
    }
  };
  page.open(sPrintClientURL, function (status) {

    if (status !== 'success') {
      console.log('Unable to load the sPrintClientURL, status:', status);
      phantom.exit(1);
    } else {
      /**
       * récupère le contenu HTML de la div
       * @param {string} readyState nom de l'état du document
       * @returns {boolean}
       */
      var saveAsHtml = function (readyState) {
        console.log("readyState: ", readyState);
        // Sauvegarde le fichier html
        var sDivContent = page.evaluate(function () {
            return document.getElementById('element_render').innerHTML;
        });
        fs.write(sOutputFileName, sDivContent, 'w');
        console.log("Done");
        phantom.exit();
      };

      var oOptions = {
        includes: oIncludes,
        scope: oScope,
        htmlbody: sHTMLBody
      };

      // Initialize the map with the given params
      page.evaluate(function (oOptions) {
        initRender(oOptions);
        copyStyles();
      }, oOptions);

      // Effectue l'impression lorsque la page sera chargée
      function checkReadyState() {
        setTimeout(function () {
          var readyState = page.evaluate(function () {
            return document.readyState;
          });
          if ("complete" === readyState) {
            saveAsHtml(readyState);
          } else {
            checkReadyState();
          }
        });
      }

      setTimeout(function () {
        checkReadyState();
      }, printTimeout);

      // Au cas où la page ne soit pas chargée apres printTimeout
      setTimeout(function () {
        saveAsHtml("timeout");
      }, printLoadTimeout);
    }
  });
}
